@@ -29,10 +29,10 @@
 #ifndef _WIN32
 #include <sys/select.h> /* FD_* select */
 #include <sys/time.h> /* timeval */
-#include <sys/wait.h> /* WEXITSTATUS() waitpid() */
+#include <sys/wait.h> /* WEXITSTATUS() WIFEXITED() waitpid() */
 #endif
 #include <signal.h> /* kill() */
-#include <unistd.h> /* select() */
+#include <unistd.h> /* execve() fork() select() */
 
 #include <assert.h> /* assert() */
 #include <errno.h> /* errno */
@@ -78,7 +78,7 @@
  * that has associated external process has the following life cycle:
  *  1. Created by main thread and passed to error thread through new_err_jobs.
  *  2. Either gets marked by signal handler or its stream reaches EOF.
- *  3. It's in_use flag is reset.
+ *  3. Its in_use flag is reset.
  *  4. Main thread frees corresponding entry.
  */
 
@@ -110,8 +110,8 @@ typedef struct
 }
 background_task_args;
 
-static void job_check(bg_job_t *const job);
-static void job_free(bg_job_t *const job);
+static void job_check(bg_job_t *job);
+static void job_free(bg_job_t *job);
 #ifndef _WIN32
 static void * error_thread(void *p);
 static int update_error_jobs(bg_job_t **jobs, fd_set *set);
@@ -245,7 +245,7 @@ bg_check(void)
 /* Checks status of the job.  Processes error stream or checks whether process
  * is still running. */
 static void
-job_check(bg_job_t *const job)
+job_check(bg_job_t *job)
 {
 #ifndef _WIN32
 	char *new_errors;
@@ -282,7 +282,7 @@ job_check(bg_job_t *const job)
 /* Frees resources allocated by the job as well as the bg_job_t structure
  * itself.  The job can be NULL. */
 static void
-job_free(bg_job_t *const job)
+job_free(bg_job_t *job)
 {
 	if(job == NULL)
 	{
@@ -314,72 +314,6 @@ job_free(bg_job_t *const job)
 	free(job);
 }
 
-/* Used for FUSE mounting and unmounting only. */
-int
-bg_and_wait_for_status(char cmd[],
-		const struct cancellation_t *cancellation, int *cancelled)
-{
-#ifndef _WIN32
-	pid_t pid;
-	int status;
-
-	if(cancellation_possible(cancellation))
-	{
-		*cancelled = 0;
-	}
-
-	if(cmd == NULL)
-	{
-		return 1;
-	}
-
-	(void)set_sigchld(1);
-
-	pid = fork();
-	if(pid == (pid_t)-1)
-	{
-		(void)set_sigchld(0);
-		LOG_SERROR_MSG(errno, "Forking has failed.");
-		return -1;
-	}
-
-	if(pid == (pid_t)0)
-	{
-		extern char **environ;
-
-		(void)set_sigchld(0);
-
-		(void)execve(get_execv_path(cfg.shell), make_execv_array(cfg.shell, cmd),
-				environ);
-		_Exit(127);
-	}
-
-	while(waitpid(pid, &status, 0) == -1)
-	{
-		if(errno != EINTR)
-		{
-			LOG_SERROR_MSG(errno, "Failed waiting for process: %" PRINTF_ULL,
-					(unsigned long long)pid);
-			status = -1;
-			break;
-		}
-		process_cancel_request(pid, cancellation);
-	}
-
-	if(cancellation_requested(cancellation))
-	{
-		*cancelled = 1;
-	}
-
-	(void)set_sigchld(0);
-
-	return status;
-
-#else
-	return -1;
-#endif
-}
-
 int
 bg_and_wait_for_errors(char cmd[], const struct cancellation_t *cancellation)
 {
@@ -405,7 +339,7 @@ bg_and_wait_for_errors(char cmd[], const struct cancellation_t *cancellation)
 	if(pid == 0)
 	{
 		(void)set_sigchld(0);
-		run_from_fork(error_pipe, 1, cmd);
+		run_from_fork(error_pipe, 1, 0, cmd);
 	}
 	else
 	{
