@@ -109,7 +109,7 @@ cfg_init(void)
 	cfg.history_len = 15;
 
 	cfg.auto_execute = 0;
-	cfg.time_format = strdup(" %m/%d %H:%M");
+	cfg.time_format = strdup("%m/%d %H:%M");
 	cfg.wrap_quick_view = 1;
 	cfg.undo_levels = 100;
 	cfg.sort_numbers = 0;
@@ -133,8 +133,7 @@ cfg_init(void)
 	cfg.ignore_case = 0;
 	cfg.smart_case = 0;
 	cfg.hl_search = 1;
-	cfg.vifm_info = VIFMINFO_MARKS | VIFMINFO_BOOKMARKS;
-	cfg.auto_ch_pos = 1;
+	cfg.vifm_info = VINFO_MARKS | VINFO_BOOKMARKS;
 	cfg.scroll_off = 0;
 	cfg.gdefault = 0;
 	cfg.scroll_bind = 0;
@@ -160,9 +159,11 @@ cfg_init(void)
 	cfg.grep_prg = strdup("grep -n -H -I -r %i %a %s");
 	cfg.locate_prg = strdup("locate %a");
 	cfg.delete_prg = strdup("");
+	cfg.media_prg = format_str("%s/vifm-media", get_installed_data_dir());
 
 	cfg.trunc_normal_sb_msgs = 0;
 	cfg.shorten_title_paths = 1;
+	cfg.short_term_mux_titles = 0;
 
 	cfg.slow_fs_list = strdup("");
 
@@ -211,6 +212,12 @@ cfg_init(void)
 	cfg.sizefmt.base = 1024;
 	cfg.sizefmt.precision = 0;
 	cfg.sizefmt.ieci_prefixes = 0;
+
+	cfg.pane_tabs = 0;
+	cfg.show_tab_line = STL_MULTIPLE;
+
+	cfg.auto_ch_pos = 1;
+	cfg.ch_pos_on = CHPOS_STARTUP | CHPOS_DIRMARK | CHPOS_ENTER;
 }
 
 void
@@ -260,12 +267,12 @@ try_userprofile_envvar_for_home(void)
 #ifndef _WIN32
 	return 0;
 #else
-	char home[PATH_MAX];
+	char home[PATH_MAX + 1];
 	const char *userprofile = env_get("USERPROFILE");
 	if(userprofile == NULL || !is_dir(userprofile))
 		return 0;
 	copy_str(home, sizeof(home), userprofile);
-	to_forward_slash(home);
+	system_to_internal_slashes(home);
 	env_set(HOME_EV, home);
 	return 1;
 #endif
@@ -281,7 +288,7 @@ try_homepath_envvar_for_home(void)
 #ifndef _WIN32
 	return 0;
 #else
-	char home[PATH_MAX];
+	char home[PATH_MAX + 1];
 	const char *homedrive = env_get("HOMEDRIVE");
 	const char *homepath = env_get("HOMEPATH");
 	if(homedrive == NULL || !is_dir(homedrive))
@@ -290,7 +297,7 @@ try_homepath_envvar_for_home(void)
 		return 0;
 
 	snprintf(home, sizeof(home), "%s%s", homedrive, homepath);
-	to_forward_slash(home);
+	system_to_internal_slashes(home);
 	env_set(HOME_EV, home);
 	return 1;
 #endif
@@ -332,7 +339,7 @@ try_exe_directory_for_conf(void)
 {
 	LOG_FUNC_ENTER;
 
-	char exe_dir[PATH_MAX];
+	char exe_dir[PATH_MAX + 1];
 
 	if(get_exe_dir(exe_dir, sizeof(exe_dir)) != 0)
 	{
@@ -355,7 +362,7 @@ try_home_envvar_for_conf(int force)
 {
 	LOG_FUNC_ENTER;
 
-	char vifm[PATH_MAX];
+	char vifm[PATH_MAX + 1];
 
 	const char *home = env_get(HOME_EV);
 	if(home == NULL || !is_dir(home))
@@ -383,12 +390,12 @@ try_appdata_for_conf(void)
 #ifndef _WIN32
 	return 0;
 #else
-	char vifm[PATH_MAX];
+	char vifm[PATH_MAX + 1];
 	const char *appdata = env_get("APPDATA");
 	if(appdata == NULL || !is_dir(appdata))
 		return 0;
 	snprintf(vifm, sizeof(vifm), "%s/Vifm", appdata);
-	to_forward_slash(vifm);
+	system_to_internal_slashes(vifm);
 	env_set(VIFM_EV, vifm);
 	return 1;
 #endif
@@ -472,8 +479,8 @@ try_exe_directory_for_vifmrc(void)
 {
 	LOG_FUNC_ENTER;
 
-	char exe_dir[PATH_MAX];
-	char vifmrc[PATH_MAX];
+	char exe_dir[PATH_MAX + 1];
+	char vifmrc[PATH_MAX + 1];
 
 	if(get_exe_dir(exe_dir, sizeof(exe_dir)) != 0)
 	{
@@ -497,7 +504,7 @@ try_vifm_vifmrc_for_vifmrc(void)
 {
 	LOG_FUNC_ENTER;
 
-	char vifmrc[PATH_MAX];
+	char vifmrc[PATH_MAX + 1];
 	const char *vifm = env_get(VIFM_EV);
 	if(vifm == NULL || !is_dir(vifm))
 		return 0;
@@ -605,7 +612,7 @@ static void
 create_scripts_dir(void)
 {
 	char scripts[PATH_MAX + 16];
-	char readme[PATH_MAX];
+	char readme[PATH_MAX + 1];
 	FILE *fp;
 
 	snprintf(scripts, sizeof(scripts), "%s/" SCRIPTS_DIR, cfg.config_dir);
@@ -846,11 +853,11 @@ cfg_resize_histories(int new_size)
 int
 cfg_set_fuse_home(const char new_value[])
 {
-	char canonicalized[PATH_MAX];
+	char canonicalized[PATH_MAX + 1];
 #ifdef _WIN32
 	char with_forward_slashes[strlen(new_value) + 1];
 	strcpy(with_forward_slashes, new_value);
-	to_forward_slash(with_forward_slashes);
+	system_to_internal_slashes(with_forward_slashes);
 	new_value = with_forward_slashes;
 #endif
 	canonicalize_path(new_value, canonicalized, sizeof(canonicalized));
@@ -902,5 +909,11 @@ cfg_confirm_delete(int to_trash)
 	     : (cfg.confirm & CONFIRM_PERM_DELETE);
 }
 
+int
+cfg_ch_pos_on(ChposWhen when)
+{
+	return cfg.auto_ch_pos && (cfg.ch_pos_on & when);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
