@@ -71,33 +71,33 @@ static void get_history(view_t *view, int reread, const char dir[],
 static void set_manual_filter(view_t *view, const char value[]);
 static void set_view_property(view_t *view, char type, const char value[]);
 static int copy_file(const char src[], const char dst[]);
-static int copy_file_internal(FILE *const src, FILE *const dst);
+static int copy_file_internal(FILE *src, FILE *dst);
 static void update_info_file(const char filename[], int merge);
 static void process_hist_entry(view_t *view, const char dir[],
 		const char file[], int pos, char ***lh, int *nlh, int **lhp, size_t *nlhp);
 static char * convert_old_trash_path(const char trash_path[]);
-static void write_options(FILE *const fp);
+static void write_options(FILE *fp);
 static void write_assocs(FILE *fp, const char str[], char mark,
 		assoc_list_t *assocs, int prev_count, char *prev[]);
 static void write_doubling_commas(FILE *fp, const char str[]);
-static void write_commands(FILE *const fp, char *cmds_list[], char *cmds[],
+static void write_commands(FILE *fp, char *cmds_list[], char *cmds[],
 		int ncmds);
-static void write_marks(FILE *const fp, const char non_conflicting_marks[],
+static void write_marks(FILE *fp, const char non_conflicting_marks[],
 		char *marks[], const int timestamps[], int nmarks);
-static void write_bmarks(FILE *const fp, char *bmarks[], const int timestamps[],
+static void write_bmarks(FILE *fp, char *bmarks[], const int timestamps[],
 		int nbmarks);
 static void write_bmark(const char path[], const char tags[], time_t timestamp,
 		void *arg);
-static void write_tui_state(FILE *const fp);
+static void write_tui_state(FILE *fp);
 static void write_view_history(FILE *fp, view_t *view, const char str[],
 		char mark, int prev_count, char *prev[], int pos[]);
 static void write_history(FILE *fp, const char str[], char mark, int prev_count,
 		char *prev[], const hist_t *hist);
-static void write_registers(FILE *const fp, char *regs[], int nregs);
-static void write_dir_stack(FILE *const fp, char *old_dir_stack[],
+static void write_registers(FILE *fp, char *regs[], int nregs);
+static void write_dir_stack(FILE *fp, char *old_dir_stack[],
 		int nold_dir_stack);
-static void write_trash(FILE *const fp, char *trash[], int ntrash);
-static void write_general_state(FILE *const fp);
+static void write_trash(FILE *fp, char *trash[], int ntrash);
+static void write_general_state(FILE *fp);
 static char * read_vifminfo_line(FILE *fp, char buffer[]);
 static void remove_leading_whitespace(char line[]);
 static const char * escape_spaces(const char *str);
@@ -240,7 +240,7 @@ read_info_file(int reread)
 		else if(type == LINE_TYPE_QUICK_VIEW_STATE)
 		{
 			const int i = atoi(line_val);
-			curr_stats.preview.on = (i == 1);
+			stats_set_quickview(i == 1);
 		}
 		else if(type == LINE_TYPE_WIN_COUNT)
 		{
@@ -551,7 +551,7 @@ copy_file(const char src[], const char dst[])
 /* Internal sub-function of the copy_file() function.  Returns zero on
  * success. */
 static int
-copy_file_internal(FILE *const src, FILE *const dst)
+copy_file_internal(FILE *src, FILE *dst)
 {
 	char buffer[4*1024];
 	size_t nread;
@@ -597,8 +597,8 @@ update_info_file(const char filename[], int merge)
 	if(cfg.vifm_info == 0)
 		return;
 
-	cmds_list = list_udf();
-	while(cmds_list[++ncmds_list] != NULL);
+	cmds_list = vle_cmds_list_udcs();
+	ncmds_list = count_strings(cmds_list);
 
 	non_conflicting_marks = strdup(valid_marks);
 
@@ -812,12 +812,12 @@ update_info_file(const char filename[], int merge)
 		fprintf(fp, "# You can edit this file by hand, but it's recommended not to "
 				"do that.\n");
 
-		if(cfg.vifm_info & VIFMINFO_OPTIONS)
+		if(cfg.vifm_info & VINFO_OPTIONS)
 		{
 			write_options(fp);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_FILETYPES)
+		if(cfg.vifm_info & VINFO_FILETYPES)
 		{
 			write_assocs(fp, "Filetypes", LINE_TYPE_FILETYPE, &filetypes, nft, ft);
 			write_assocs(fp, "X Filetypes", LINE_TYPE_XFILETYPE, &xfiletypes, nfx,
@@ -826,75 +826,75 @@ update_info_file(const char filename[], int merge)
 					fv);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_COMMANDS)
+		if(cfg.vifm_info & VINFO_COMMANDS)
 		{
 			write_commands(fp, cmds_list, cmds, ncmds);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_MARKS)
+		if(cfg.vifm_info & VINFO_MARKS)
 		{
 			write_marks(fp, non_conflicting_marks, marks, bt, nmarks);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_BOOKMARKS)
+		if(cfg.vifm_info & VINFO_BOOKMARKS)
 		{
 			write_bmarks(fp, bmarks, bmt, nbmarks);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_TUI)
+		if(cfg.vifm_info & VINFO_TUI)
 		{
 			write_tui_state(fp);
 		}
 
-		if((cfg.vifm_info & VIFMINFO_DHISTORY) && cfg.history_len > 0)
+		if((cfg.vifm_info & VINFO_DHISTORY) && cfg.history_len > 0)
 		{
 			write_view_history(fp, &lwin, "Left", LINE_TYPE_LWIN_HIST, nlh, lh, lhp);
 			write_view_history(fp, &rwin, "Right", LINE_TYPE_RWIN_HIST, nrh, rh, rhp);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_CHISTORY)
+		if(cfg.vifm_info & VINFO_CHISTORY)
 		{
 			write_history(fp, "Command line", LINE_TYPE_CMDLINE_HIST,
 					MIN(ncmdh, cfg.history_len - curr_stats.cmd_hist.pos), cmdh,
 					&curr_stats.cmd_hist);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_SHISTORY)
+		if(cfg.vifm_info & VINFO_SHISTORY)
 		{
 			write_history(fp, "Search", LINE_TYPE_SEARCH_HIST, nsrch, srch,
 					&curr_stats.search_hist);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_PHISTORY)
+		if(cfg.vifm_info & VINFO_PHISTORY)
 		{
 			write_history(fp, "Prompt", LINE_TYPE_PROMPT_HIST, nprompt, prompt,
 					&curr_stats.prompt_hist);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_FHISTORY)
+		if(cfg.vifm_info & VINFO_FHISTORY)
 		{
 			write_history(fp, "Local filter", LINE_TYPE_FILTER_HIST, nfilter, filter,
 					&curr_stats.filter_hist);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_REGISTERS)
+		if(cfg.vifm_info & VINFO_REGISTERS)
 		{
 			write_registers(fp, regs, nregs);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_DIRSTACK)
+		if(cfg.vifm_info & VINFO_DIRSTACK)
 		{
 			write_dir_stack(fp, dir_stack, ndir_stack);
 		}
 
 		write_trash(fp, trash, ntrash);
 
-		if(cfg.vifm_info & VIFMINFO_STATE)
+		if(cfg.vifm_info & VINFO_STATE)
 		{
 			write_general_state(fp);
 		}
 
-		if(cfg.vifm_info & VIFMINFO_CS)
+		if(cfg.vifm_info & VINFO_CS)
 		{
 			fputs("\n# Color scheme:\n", fp);
 			fprintf(fp, "c%s\n", cfg.cs.name);
@@ -965,7 +965,7 @@ convert_old_trash_path(const char trash_path[])
 
 /* Writes current values of all options into vifminfo file. */
 static void
-write_options(FILE *const fp)
+write_options(FILE *fp)
 {
 	const char *str;
 
@@ -975,10 +975,8 @@ write_options(FILE *const fp)
 	fprintf(fp, "=cdpath=%s\n", cfg.cd_path);
 	fprintf(fp, "=%schaselinks\n", cfg.chase_links ? "" : "no");
 	fprintf(fp, "=columns=%d\n", cfg.columns);
-	fprintf(fp, "=cpoptions=%s%s%s\n",
-			cfg.filter_inverted_by_default ? "f" : "",
-			cfg.selection_is_primary ? "s" : "",
-			cfg.tab_switches_pane ? "t" : "");
+	fprintf(fp, "=cpoptions=%s\n",
+			escape_spaces(get_option_value("cpoptions", OPT_GLOBAL)));
 	fprintf(fp, "=deleteprg=%s\n", escape_spaces(cfg.delete_prg));
 	fprintf(fp, "=%sfastrun\n", cfg.fast_run ? "" : "no");
 	if(strcmp(cfg.border_filler, " ") != 0)
@@ -990,6 +988,8 @@ write_options(FILE *const fp)
 	fprintf(fp, "=fusehome=%s\n", escape_spaces(cfg.fuse_home));
 	fprintf(fp, "=%sgdefault\n", cfg.gdefault ? "" : "no");
 	fprintf(fp, "=grepprg=%s\n", escape_spaces(cfg.grep_prg));
+	fprintf(fp, "=histcursor=%s\n",
+			escape_spaces(get_option_value("histcursor", OPT_GLOBAL)));
 	fprintf(fp, "=history=%d\n", cfg.history_len);
 	fprintf(fp, "=%shlsearch\n", cfg.hl_search ? "" : "no");
 	fprintf(fp, "=%siec\n", cfg.sizefmt.ieci_prefixes ? "" : "no");
@@ -999,7 +999,9 @@ write_options(FILE *const fp)
 	fprintf(fp, "=%stitle\n", cfg.set_title ? "" : "no");
 	fprintf(fp, "=lines=%d\n", cfg.lines);
 	fprintf(fp, "=locateprg=%s\n", escape_spaces(cfg.locate_prg));
+	fprintf(fp, "=mediaprg=%s\n", escape_spaces(cfg.media_prg));
 	fprintf(fp, "=mintimeoutlen=%d\n", cfg.min_timeout_len);
+	fprintf(fp, "=%squickview\n", curr_stats.preview.on ? "" : "no");
 	fprintf(fp, "=rulerformat=%s\n", escape_spaces(cfg.ruler_format));
 	fprintf(fp, "=%srunexec\n", cfg.auto_execute ? "" : "no");
 	fprintf(fp, "=%sscrollbind\n", cfg.scroll_bind ? "" : "no");
@@ -1007,6 +1009,8 @@ write_options(FILE *const fp)
 	fprintf(fp, "=shell=%s\n", escape_spaces(cfg.shell));
 	fprintf(fp, "=shortmess=%s\n",
 			escape_spaces(get_option_value("shortmess", OPT_GLOBAL)));
+	fprintf(fp, "=showtabline=%s\n",
+			escape_spaces(get_option_value("showtabline", OPT_GLOBAL)));
 	fprintf(fp, "=sizefmt=%s\n",
 			escape_spaces(get_option_value("sizefmt", OPT_GLOBAL)));
 #ifndef _WIN32
@@ -1015,14 +1019,16 @@ write_options(FILE *const fp)
 	fprintf(fp, "=%ssmartcase\n", cfg.smart_case ? "" : "no");
 	fprintf(fp, "=%ssortnumbers\n", cfg.sort_numbers ? "" : "no");
 	fprintf(fp, "=statusline=%s\n", escape_spaces(cfg.status_line));
+	fprintf(fp, "=syncregs=%s\n",
+			escape_spaces(get_option_value("syncregs", OPT_GLOBAL)));
+	fprintf(fp, "=tabscope=%s\n",
+			escape_spaces(get_option_value("tabscope", OPT_GLOBAL)));
 	fprintf(fp, "=tabstop=%d\n", cfg.tab_stop);
-	fprintf(fp, "=timefmt=%s\n", escape_spaces(cfg.time_format + 1));
+	fprintf(fp, "=timefmt=%s\n", escape_spaces(cfg.time_format));
 	fprintf(fp, "=timeoutlen=%d\n", cfg.timeout_len);
 	fprintf(fp, "=%strash\n", cfg.use_trash ? "" : "no");
-	fprintf(fp, "=tuioptions=%s%s%s\n",
-			cfg.extra_padding ? "p" : "",
-			cfg.side_borders_visible ? "s" : "",
-			cfg.use_unicode_characters ? "u" : "");
+	fprintf(fp, "=tuioptions=%s\n",
+			escape_spaces(get_option_value("tuioptions", OPT_GLOBAL)));
 	fprintf(fp, "=undolevels=%d\n", cfg.undo_levels);
 	fprintf(fp, "=vicmd=%s%s\n", escape_spaces(cfg.vi_command),
 			cfg.vi_cmd_bg ? " &" : "");
@@ -1056,106 +1062,24 @@ write_options(FILE *const fp)
 	fprintf(fp, "=[previewprg=%s\n", escape_spaces(lwin.preview_prg_g));
 	fprintf(fp, "=]previewprg=%s\n", escape_spaces(rwin.preview_prg_g));
 
-	fprintf(fp, "%s", "=confirm=");
-	if(cfg.confirm & CONFIRM_DELETE)
-		fprintf(fp, "%s", "delete,");
-	if(cfg.confirm & CONFIRM_PERM_DELETE)
-		fprintf(fp, "%s", "permdelete,");
-	fprintf(fp, "\n");
-
-	fprintf(fp, "%s", "=dotdirs=");
-	if(cfg.dot_dirs & DD_ROOT_PARENT)
-		fprintf(fp, "%s", "rootparent,");
-	if(cfg.dot_dirs & DD_NONROOT_PARENT)
-		fprintf(fp, "%s", "nonrootparent,");
-	fprintf(fp, "\n");
-
-	fprintf(fp, "%s", "=caseoptions=");
-	if(cfg.case_override & CO_PATH_COMPL)
-		fprintf(fp, "%c", (cfg.case_ignore & CO_PATH_COMPL) ? 'p' : 'P');
-	if(cfg.case_override & CO_GOTO_FILE)
-		fprintf(fp, "%c", (cfg.case_ignore & CO_GOTO_FILE) ? 'g' : 'G');
-	fprintf(fp, "\n");
-
-	fprintf(fp, "%s", "=suggestoptions=");
-	if(cfg.sug.flags & SF_NORMAL)
-		fprintf(fp, "%s", "normal,");
-	if(cfg.sug.flags & SF_VISUAL)
-		fprintf(fp, "%s", "visual,");
-	if(cfg.sug.flags & SF_VIEW)
-		fprintf(fp, "%s", "view,");
-	if(cfg.sug.flags & SF_OTHERPANE)
-		fprintf(fp, "%s", "otherpane,");
-	if(cfg.sug.flags & SF_KEYS)
-		fprintf(fp, "%s", "keys,");
-	if(cfg.sug.flags & SF_FOLDSUBKEYS)
-		fprintf(fp, "%s", "foldsubkeys,");
-	if(cfg.sug.flags & SF_MARKS)
-		fprintf(fp, "%s", "marks,");
-	if(cfg.sug.flags & SF_DELAY)
-	{
-		if(cfg.sug.delay == 500)
-		{
-			fprintf(fp, "%s", "delay,");
-		}
-		else
-		{
-			fprintf(fp, "delay:%d,", cfg.sug.delay);
-		}
-	}
-	if(cfg.sug.flags & SF_REGISTERS)
-	{
-		if(cfg.sug.maxregfiles == 5)
-		{
-			fprintf(fp, "%s", "registers,");
-		}
-		else
-		{
-			fprintf(fp, "registers:%d,", cfg.sug.maxregfiles);
-		}
-	}
-	fprintf(fp, "\n");
-
-	fprintf(fp, "%s", "=iooptions=");
-	if(cfg.fast_file_cloning)
-		fprintf(fp, "%s", "fastfilecloning,");
-	fprintf(fp, "\n");
+	fprintf(fp, "=confirm=%s\n",
+			escape_spaces(get_option_value("confirm", OPT_GLOBAL)));
+	fprintf(fp, "=dotdirs=%s\n",
+			escape_spaces(get_option_value("dotdirs", OPT_GLOBAL)));
+	fprintf(fp, "=caseoptions=%s\n",
+			escape_spaces(get_option_value("caseoptions", OPT_GLOBAL)));
+	fprintf(fp, "=suggestoptions=%s\n",
+			escape_spaces(get_option_value("suggestoptions", OPT_GLOBAL)));
+	fprintf(fp, "=iooptions=%s\n",
+			escape_spaces(get_option_value("iooptions", OPT_GLOBAL)));
 
 	fprintf(fp, "=dirsize=%s", cfg.view_dir_size == VDS_SIZE ? "size" : "nitems");
 
 	str = classify_to_str();
 	fprintf(fp, "=classify=%s\n", escape_spaces(str == NULL ? "" : str));
 
-	fprintf(fp, "=vifminfo=options");
-	if(cfg.vifm_info & VIFMINFO_FILETYPES)
-		fprintf(fp, ",filetypes");
-	if(cfg.vifm_info & VIFMINFO_COMMANDS)
-		fprintf(fp, ",commands");
-	if(cfg.vifm_info & VIFMINFO_MARKS)
-		fprintf(fp, ",bookmarks");
-	if(cfg.vifm_info & VIFMINFO_TUI)
-		fprintf(fp, ",tui");
-	if(cfg.vifm_info & VIFMINFO_DHISTORY)
-		fprintf(fp, ",dhistory");
-	if(cfg.vifm_info & VIFMINFO_STATE)
-		fprintf(fp, ",state");
-	if(cfg.vifm_info & VIFMINFO_CS)
-		fprintf(fp, ",cs");
-	if(cfg.vifm_info & VIFMINFO_SAVEDIRS)
-		fprintf(fp, ",savedirs");
-	if(cfg.vifm_info & VIFMINFO_CHISTORY)
-		fprintf(fp, ",chistory");
-	if(cfg.vifm_info & VIFMINFO_SHISTORY)
-		fprintf(fp, ",shistory");
-	if(cfg.vifm_info & VIFMINFO_PHISTORY)
-		fprintf(fp, ",phistory");
-	if(cfg.vifm_info & VIFMINFO_FHISTORY)
-		fprintf(fp, ",fhistory");
-	if(cfg.vifm_info & VIFMINFO_DIRSTACK)
-		fprintf(fp, ",dirstack");
-	if(cfg.vifm_info & VIFMINFO_REGISTERS)
-		fprintf(fp, ",registers");
-	fprintf(fp, "\n");
+	fprintf(fp, "=vifminfo=%s\n",
+			escape_spaces(get_option_value("vifminfo", OPT_GLOBAL)));
 
 	fprintf(fp, "=%svimhelp\n", cfg.use_vim_help ? "" : "no");
 	fprintf(fp, "=%swildmenu\n", cfg.wild_menu ? "" : "no");
@@ -1229,7 +1153,7 @@ write_doubling_commas(FILE *fp, const char str[])
  * terminated list of commands existing in current session, cmds is a list of
  * length ncmds with unseen commands read from vifminfo. */
 static void
-write_commands(FILE *const fp, char *cmds_list[], char *cmds[], int ncmds)
+write_commands(FILE *fp, char *cmds_list[], char *cmds[], int ncmds)
 {
 	int i;
 
@@ -1247,7 +1171,7 @@ write_commands(FILE *const fp, char *cmds_list[], char *cmds[], int ncmds)
 /* Writes marks to vifminfo file.  marks is a list of length nmarks marks read
  * from vifminfo. */
 static void
-write_marks(FILE *const fp, const char non_conflicting_marks[],
+write_marks(FILE *fp, const char non_conflicting_marks[],
 		char *marks[], const int timestamps[], int nmarks)
 {
 	int active_marks[NUM_MARKS];
@@ -1281,7 +1205,7 @@ write_marks(FILE *const fp, const char non_conflicting_marks[],
 /* Writes bookmarks to vifminfo file.  bmarks is a list of length nbmarks marks
  * read from vifminfo. */
 static void
-write_bmarks(FILE *const fp, char *bmarks[], const int timestamps[],
+write_bmarks(FILE *fp, char *bmarks[], const int timestamps[],
 		int nbmarks)
 {
 	int i;
@@ -1310,7 +1234,7 @@ write_bmark(const char path[], const char tags[], time_t timestamp, void *arg)
 
 /* Writes state of the TUI to vifminfo file. */
 static void
-write_tui_state(FILE *const fp)
+write_tui_state(FILE *fp)
 {
 	fputs("\n# TUI:\n", fp);
 	fprintf(fp, "a%c\n", (curr_view == &rwin) ? 'r' : 'l');
@@ -1340,7 +1264,7 @@ write_view_history(FILE *fp, view_t *view, const char str[], char mark,
 		fprintf(fp, "%c%s\n\t%s\n%d\n", mark, view->history[i].dir,
 				view->history[i].file, view->history[i].rel_pos);
 	}
-	if(cfg.vifm_info & VIFMINFO_SAVEDIRS)
+	if(cfg.vifm_info & VINFO_SAVEDIRS)
 	{
 		fprintf(fp, "%c\n", mark);
 	}
@@ -1366,7 +1290,7 @@ write_history(FILE *fp, const char str[], char mark, int prev_count,
 /* Writes registers to vifminfo file.  regs is a list of length nregs registers
  * read from vifminfo. */
 static void
-write_registers(FILE *const fp, char *regs[], int nregs)
+write_registers(FILE *fp, char *regs[], int nregs)
 {
 	int i;
 
@@ -1395,7 +1319,7 @@ write_registers(FILE *const fp, char *regs[], int nregs)
 /* Writes directory stack to vifminfo file.  old_dir_stack is a list of length
  * nold_dir_stack entries (4 lines per entry) read from vifminfo. */
 static void
-write_dir_stack(FILE *const fp, char *old_dir_stack[], int nold_dir_stack)
+write_dir_stack(FILE *fp, char *old_dir_stack[], int nold_dir_stack)
 {
 	fputs("\n# Directory stack (oldest to newest):\n", fp);
 	if(dir_stack_changed())
@@ -1422,7 +1346,7 @@ write_dir_stack(FILE *const fp, char *old_dir_stack[], int nold_dir_stack)
 /* Writes trash entries to vifminfo file.  trash is a list of length ntrash
  * entries read from vifminfo. */
 static void
-write_trash(FILE *const fp, char *trash[], int ntrash)
+write_trash(FILE *fp, char *trash[], int ntrash)
 {
 	int i;
 	fputs("\n# Trash content:\n", fp);
@@ -1438,7 +1362,7 @@ write_trash(FILE *const fp, char *trash[], int ntrash)
 
 /* Writes general state to vifminfo file. */
 static void
-write_general_state(FILE *const fp)
+write_general_state(FILE *fp)
 {
 	fputs("\n# State:\n", fp);
 	fprintf(fp, "f%s\n", matcher_get_expr(lwin.manual_filter));
