@@ -198,7 +198,8 @@ complete_args(int id, const cmd_info_t *cmd_info, int arg_pos, void *extra_arg)
 		}
 	}
 	else if((id == COM_CD || id == COM_PUSHD || id == COM_EXECUTE ||
-			id == COM_SOURCE || id == COM_EDIT) && dollar != NULL && dollar > slash)
+			id == COM_SOURCE || id == COM_EDIT || id == COM_GOTO_PATH ||
+			id == COM_TABNEW) && dollar != NULL && dollar > slash)
 	{
 		start = dollar + 1;
 		complete_envvar(start);
@@ -695,6 +696,7 @@ complete_highlight_arg(const char *str)
 				{ "reverse",   "reversed colors" },
 				{ "inverse",   "reversed colors" },
 				{ "standout",  "like bold or similar to it" },
+				{ "italic",    "on unsupported systems becomes reverse" },
 				{ "none",      "no attributes" },
 			};
 
@@ -943,7 +945,7 @@ filename_completion(const char str[], CompletionType type,
 
 	if(!skip_canonicalization)
 	{
-		char canonic_path[PATH_MAX];
+		char canonic_path[PATH_MAX + 1];
 		to_canonic_path(dirname, flist_get_dir(curr_view), canonic_path,
 				sizeof(canonic_path));
 		if(replace_string(&dirname, canonic_path) != 0)
@@ -959,7 +961,7 @@ filename_completion(const char str[], CompletionType type,
 			(stroscmp(dirname, ".") == 0 && is_unc_root(curr_view->curr_dir)) ||
 			(stroscmp(dirname, "/") == 0 && is_unc_path(curr_view->curr_dir)))
 	{
-		char buf[PATH_MAX];
+		char buf[PATH_MAX + 1];
 		if(!is_unc_root(dirname))
 			copy_str(buf,
 					strchr(curr_view->curr_dir + 2, '/') - curr_view->curr_dir + 1,
@@ -974,7 +976,7 @@ filename_completion(const char str[], CompletionType type,
 	}
 	if(is_unc_path(curr_view->curr_dir))
 	{
-		char buf[PATH_MAX];
+		char buf[PATH_MAX + 1];
 		if(is_path_absolute(dirname) && !is_unc_root(curr_view->curr_dir))
 			copy_str(buf,
 					strchr(curr_view->curr_dir + 2, '/') - curr_view->curr_dir + 2,
@@ -1186,7 +1188,7 @@ file_matches(const char fname[], const char prefix[], size_t prefix_len)
 int
 external_command_exists(const char cmd[])
 {
-	char path[PATH_MAX];
+	char path[PATH_MAX + 1];
 
 	if(get_cmd_path(cmd, sizeof(path), path) == 0)
 	{
