@@ -46,6 +46,7 @@ enum
 	COM_FILE,
 	COM_FIND,
 	COM_GOTO,
+	COM_GOTO_PATH,
 	COM_GREP,
 	COM_HELP,
 	COM_HIGHLIGHT,
@@ -64,6 +65,7 @@ enum
 	COM_SOURCE,
 	COM_SPLIT,
 	COM_SYNC,
+	COM_TABNEW,
 	COM_TOUCH,
 	COM_UNLET,
 	COM_VSPLIT,
