@@ -27,6 +27,7 @@
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isspace() */
 #include <errno.h> /* errno */
+#include <limits.h> /* INT_MAX */
 #include <signal.h>
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* snprintf() */
@@ -116,7 +117,7 @@ static int cmd_should_be_processed(int cmd_id);
 TSTATIC char ** break_cmdline(const char cmdline[], int for_menu);
 static int is_out_of_arg(const char cmd[], const char pos[]);
 TSTATIC int line_pos(const char begin[], const char end[], char sep,
-		int rquoting);
+		int rquoting, int max_args);
 static CmdArgsType get_cmd_args_type(const char cmd[]);
 static char * skip_to_cmd_name(const char cmd[]);
 static int repeat_command(view_t *view, CmdInputType type);
@@ -199,7 +200,7 @@ get_and_execute_command(const char line[], size_t line_pos, CmdInputType type)
 char *
 get_ext_command(const char beginning[], size_t line_pos, CmdInputType type)
 {
-	char cmd_file[PATH_MAX];
+	char cmd_file[PATH_MAX + 1];
 	char *cmd = NULL;
 
 	generate_tmp_file_name("vifm.cmdline", cmd_file, sizeof(cmd_file));
@@ -300,7 +301,12 @@ execute_extcmd(const char command[], CmdInputType type)
 {
 	if(type == CIT_COMMAND)
 	{
+		commands_scope_start();
 		curr_stats.save_msg = exec_commands(command, curr_view, type);
+		if(commands_scope_finish() != 0)
+		{
+			curr_stats.save_msg = 1;
+		}
 	}
 	else
 	{
@@ -407,14 +413,14 @@ init_commands(void)
 {
 	if(cmds_conf.inner != NULL)
 	{
-		init_cmds(1, &cmds_conf);
+		vle_cmds_init(1, &cmds_conf);
 		return;
 	}
 
 	/* We get here when init_commands() is called the first time. */
 
-	init_cmds(1, &cmds_conf);
-	add_builtin_commands((const cmd_add_t *)&cmds_list, cmds_list_size);
+	vle_cmds_init(1, &cmds_conf);
+	vle_cmds_add(cmds_list, cmds_list_size);
 
 	/* Initialize modules used by this one. */
 	init_bracket_notation();
@@ -460,13 +466,13 @@ execute_command(view_t *view, const char command[], int menu)
 
 	if(!menu)
 	{
-		init_cmds(1, &cmds_conf);
+		vle_cmds_init(1, &cmds_conf);
 		cmds_conf.begin = 0;
 		cmds_conf.current = view->list_pos;
 		cmds_conf.end = view->list_rows - 1;
 	}
 
-	id = get_cmd_id(command);
+	id = vle_cmds_identify(command);
 
 	if(!cmd_should_be_processed(id))
 	{
@@ -480,12 +486,12 @@ execute_command(view_t *view, const char command[], int menu)
 		snprintf(undo_msg, sizeof(undo_msg), "in %s: %s",
 				replace_home_part(flist_get_dir(view)), command);
 
-		cmd_group_begin(undo_msg);
-		cmd_group_end();
+		un_group_open(undo_msg);
+		un_group_close();
 	}
 
 	keep_view_selection = 0;
-	result = execute_cmd(command);
+	result = vle_cmds_run(command);
 
 	if(result >= 0)
 		return result;
@@ -534,9 +540,6 @@ execute_command(view_t *view, const char command[], int menu)
 		case CMDS_ERR_UDF_IS_AMBIGUOUS:
 			ui_sb_err("Ambiguous use of user-defined command");
 			break;
-		case CMDS_ERR_ZERO_COUNT:
-			ui_sb_err("Zero count");
-			break;
 		case CMDS_ERR_INVALID_ARG:
 			ui_sb_err("Invalid argument");
 			break;
@@ -599,7 +602,10 @@ cmd_should_be_processed(int cmd_id)
 	return 0;
 }
 
-/* Determines current position in the command line.  Returns:
+/* Determines current position in the command line.  The rquoting parameter when
+ * non-zero specifies number of rquoted arguments by its absolute value, while
+ * negative sign means that last argument doesn't need to be terminated.
+ * Returns:
  *  - 0, if not inside an argument;
  *  - 1, if next character should be skipped (XXX: what does it mean?);
  *  - 2, if inside escaped argument;
@@ -607,13 +613,16 @@ cmd_should_be_processed(int cmd_id)
  *  - 4, if inside double quoted argument;
  *  - 5, if inside regexp quoted argument. */
 TSTATIC int
-line_pos(const char begin[], const char end[], char sep, int rquoting)
+line_pos(const char begin[], const char end[], char sep, int rquoting,
+		int max_args)
 {
-	int state;
 	int count;
-	enum { BEGIN, NO_QUOTING, S_QUOTING, D_QUOTING, R_QUOTING };
+	enum { BEGIN, NO_QUOTING, S_QUOTING, D_QUOTING, R_QUOTING } state;
+	int args_left = max_args;
+	/* Actual separator used for rargs or '\0' if unset. */
+	char rsep = '\0';
 
-	const char *args = get_cmd_args(begin);
+	const char *args = vle_cmds_args(begin);
 	if(args >= end)
 	{
 		return 0;
@@ -628,6 +637,11 @@ line_pos(const char begin[], const char end[], char sep, int rquoting)
 		count = 1;
 	}
 
+	if(rquoting < 0)
+	{
+		rquoting = -(rquoting + 1);
+	}
+
 	state = BEGIN;
 	while(begin != end)
 	{
@@ -638,12 +652,14 @@ line_pos(const char begin[], const char end[], char sep, int rquoting)
 					state = S_QUOTING;
 				else if(sep == ' ' && *begin == '"')
 					state = D_QUOTING;
-				else if(sep == ' ' && *begin == '/' && rquoting)
+				else if(rquoting && ((sep == ' ' && *begin == '/') || *begin == sep) &&
+						(rsep == '\0' || *begin == rsep))
 					state = R_QUOTING;
 				else if(*begin == '&' && begin == end - 1)
 					state = BEGIN;
 				else if(*begin != sep)
-					state = NO_QUOTING;
+					state = rquoting && (rsep == '\0' || *begin == rsep) ? R_QUOTING
+					                                                     : NO_QUOTING;
 				break;
 			case NO_QUOTING:
 				if(*begin == sep)
@@ -683,9 +699,18 @@ line_pos(const char begin[], const char end[], char sep, int rquoting)
 				}
 				break;
 			case R_QUOTING:
-				if(*begin == '/')
+				if(rsep == '\0')
 				{
-					state = BEGIN;
+					rsep = *(begin - 1);
+				}
+				if(*begin == rsep || *begin == sep)
+				{
+					--args_left;
+					--rquoting;
+					if(args_left == 0 || rquoting == 0)
+					{
+						state = BEGIN;
+					}
 				}
 				else if(*begin == '\\')
 				{
@@ -716,7 +741,7 @@ line_pos(const char begin[], const char end[], char sep, int rquoting)
 		{
 			case S_QUOTING: return 3;
 			case D_QUOTING: return 4;
-			case R_QUOTING: return 5;
+			case R_QUOTING: return (max_args <= 1 || args_left > 1 ? 5 : 2);
 
 			default:
 				assert(0 && "Unexpected state.");
@@ -781,7 +806,7 @@ break_cmdline(const char cmdline[], int for_menu)
 	 * the cmds module of the engine that use context. */
 	if(!for_menu)
 	{
-		init_cmds(1, &cmds_conf);
+		vle_cmds_init(1, &cmds_conf);
 	}
 
 	cmdline = skip_to_cmd_name(cmdline);
@@ -868,11 +893,18 @@ is_out_of_arg(const char cmd[], const char pos[])
 {
 	const CmdLineLocation location = get_cmdline_location(cmd, pos);
 
-	if(location == CLL_NO_QUOTING && get_cmd_args_type(cmd) == CAT_EXPR &&
-			pos != cmd && *pos == '|' && pos[-1] != '|' && pos[1] != '|')
+	if(location == CLL_NO_QUOTING)
 	{
-		/* For "*[^|]|[^|]*" report that we're out of argument. */
-		return 1;
+		if(get_cmd_args_type(cmd) == CAT_REGULAR)
+		{
+			return 1;
+		}
+		if(get_cmd_args_type(cmd) == CAT_EXPR &&
+				pos != cmd && *pos == '|' && pos[-1] != '|' && pos[1] != '|')
+		{
+			/* For "*[^|]|[^|]*" report that we're out of argument. */
+			return 1;
+		}
 	}
 
 	return location == CLL_OUT_OF_ARG;
@@ -885,18 +917,27 @@ get_cmdline_location(const char cmd[], const char pos[])
 	int regex_quoting;
 
 	cmd_info_t info;
-	const int cmd_id = get_cmd_info(cmd, &info);
+	const cmd_t *const c = vle_cmds_parse(cmd, &info);
+	const int id = (c == NULL ? -1 : c->id);
+	const int max_args = (c == NULL)
+	                   ? -1
+	                   : (c->max_args == NOT_DEF ? INT_MAX : c->max_args);
 
-	switch(cmd_id)
+	switch(id)
 	{
+		case COM_HIGHLIGHT:
 		case COM_FILTER:
+		case COM_SELECT:
 			separator = ' ';
-			regex_quoting = 1;
+			regex_quoting = -2;
 			break;
 		case COM_SUBSTITUTE:
+			separator = info.sep;
+			regex_quoting = -3;
+			break;
 		case COM_TR:
 			separator = info.sep;
-			regex_quoting = 1;
+			regex_quoting = 2;
 			break;
 
 		default:
@@ -905,7 +946,7 @@ get_cmdline_location(const char cmd[], const char pos[])
 			break;
 	}
 
-	switch(line_pos(cmd, pos, separator, regex_quoting))
+	switch(line_pos(cmd, pos, separator, regex_quoting, max_args))
 	{
 		case 0: return CLL_OUT_OF_ARG;
 		case 1: /* Fall through. */
@@ -925,12 +966,13 @@ get_cmdline_location(const char cmd[], const char pos[])
 static CmdArgsType
 get_cmd_args_type(const char cmd[])
 {
-	const int cmd_id = get_cmd_id(cmd);
+	const int cmd_id = vle_cmds_identify(cmd);
 	switch(cmd_id)
 	{
 		case COMMAND_CMD_ID:
 		case COM_AUTOCMD:
 		case COM_EXECUTE:
+		case COM_CABBR:
 		case COM_CMAP:
 		case COM_CNOREMAP:
 		case COM_COMMAND:
@@ -974,8 +1016,7 @@ find_last_command(const char cmds[])
 			q += (p[1] == '|') ? 1 : 2;
 			p += 2;
 		}
-		else if(*p == '\0' || (*p == '|' &&
-				line_pos(cmds, q, ' ', starts_with_lit(cmds, "fil")) == 0))
+		else if(*p == '\0' || (*p == '|' && is_out_of_arg(cmds, q)))
 		{
 			if(*p != '\0')
 			{
@@ -1185,6 +1226,12 @@ cmds_scoped_endif(void)
 	return 0;
 }
 
+int
+cmds_scoped_empty(void)
+{
+	return int_stack_is_empty(&if_levels);
+}
+
 /* Checks that bottom of block scope is reached.  Returns non-zero if so,
  * otherwise zero is returned. */
 static int
@@ -1208,18 +1255,25 @@ eval_arglist(const char args[], const char **stop_ptr)
 		const char *tmp_result = NULL;
 
 		var_t result = var_false();
-		const ParsingErrors parsing_error = parse(args, &result);
+		const ParsingErrors parsing_error = parse(args, 1, &result);
 		if(parsing_error == PE_INVALID_EXPRESSION && is_prev_token_whitespace())
 		{
 			result = get_parsing_result();
-			tmp_result = free_this = var_to_string(result);
-			args = get_last_parsed_char();
+			if(result.type != VTYPE_ERROR)
+			{
+				tmp_result = free_this = var_to_str(result);
+				args = get_last_parsed_char();
+			}
 		}
 		else if(parsing_error == PE_NO_ERROR)
 		{
-			tmp_result = free_this = var_to_string(result);
+			tmp_result = free_this = var_to_str(result);
 			args = get_last_position();
 		}
+		else if(parsing_error != PE_INVALID_EXPRESSION)
+		{
+			report_parsing_error(parsing_error);
+		}
 
 		if(tmp_result == NULL)
 		{
