@@ -22,7 +22,7 @@
 #include <stddef.h> /* size_t */
 #include <stdint.h> /* INTPTR_MAX INT64_MAX */
 #include <stdio.h> /* FILE fclose() feof() fopen() fread() */
-#include <stdlib.h> /* free() malloc() qsort() */
+#include <stdlib.h> /* free() malloc() */
 #include <string.h> /* memcmp() */
 
 #include "compat/fs_limits.h"
@@ -40,6 +40,7 @@
 #include "utils/str.h"
 #include "utils/string_array.h"
 #include "utils/trie.h"
+#include "utils/utils.h"
 #include "filelist.h"
 #include "fops_cpmv.h"
 #include "fops_misc.h"
@@ -120,8 +121,9 @@ compare_two_panes(CompareType ct, ListType lt, int group_paths, int skip_empty)
 	{
 		/* Sort both lists according to unique file numbers to group identical files
 		 * (sorting is stable, tags are set in make_diff_list()). */
-		qsort(curr.entries, curr.nentries, sizeof(*curr.entries), &id_sorter);
-		qsort(other.entries, other.nentries, sizeof(*other.entries), &id_sorter);
+		safe_qsort(curr.entries, curr.nentries, sizeof(*curr.entries), &id_sorter);
+		safe_qsort(other.entries, other.nentries, sizeof(*other.entries),
+				&id_sorter);
 	}
 
 	if(lt == LT_UNIQUE)
@@ -394,12 +396,7 @@ compare_one_pane(view_t *view, CompareType ct, ListType lt, int skip_empty)
 		return 1;
 	}
 
-	if(curr.nentries > 0)
-	{
-		/* When there are no entries, qsort() might be called with a NULL parameter,
-		 * which isn't allowed by the standard. */
-		qsort(curr.entries, curr.nentries, sizeof(*curr.entries), &id_sorter);
-	}
+	safe_qsort(curr.entries, curr.nentries, sizeof(*curr.entries), &id_sorter);
 
 	flist_custom_start(view, title);
 
@@ -581,7 +578,7 @@ list_view_entries(const view_t *view, strlist_t *list)
 	{
 		if(!fentry_is_dir(&view->dir_entry[i]))
 		{
-			char full_path[PATH_MAX];
+			char full_path[PATH_MAX + 1];
 			void *data = &view->dir_entry[i];
 			get_full_path_of(&view->dir_entry[i], sizeof(full_path), full_path);
 			fsdata_set(tree, full_path, &data, sizeof(data));
@@ -604,7 +601,7 @@ append_valid_nodes(const char name[], int valid, const void *parent_data,
 
 	if(valid)
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full_path), full_path);
 		list->nitems = add_to_string_array(&list->items, list->nitems, 1,
 				full_path);
@@ -866,7 +863,7 @@ free_compare_records(void *ptr)
 int
 compare_move(view_t *from, view_t *to)
 {
-	char from_path[PATH_MAX], to_path[PATH_MAX];
+	char from_path[PATH_MAX + 1], to_path[PATH_MAX + 1];
 	char *from_fingerprint, *to_fingerprint;
 
 	const CompareType ct = from->custom.diff_cmp_type;
@@ -902,8 +899,8 @@ compare_move(view_t *from, view_t *to)
 
 	if(fentry_is_fake(other))
 	{
-		char to_path[PATH_MAX];
-		char canonical[PATH_MAX];
+		char to_path[PATH_MAX + 1];
+		char canonical[PATH_MAX + 1];
 		snprintf(to_path, sizeof(to_path), "%s/%s/%s", flist_get_dir(to),
 				curr->origin + strlen(flist_get_dir(from)), curr->name);
 		canonicalize_path(to_path, canonical, sizeof(canonical));
