@@ -53,11 +53,20 @@ KeyType;
 typedef struct key_chunk_t
 {
 	wchar_t key;
-	int no_remap;
 	size_t children_count;
-	int enters;            /* To prevent stack overflow and manager lifetime. */
-	int deleted;           /* Postpone free() call for proper lazy deletion. */
-	KeyType type;          /* General key type. */
+	/* Number of current uses.  To prevent stack overflow and manager lifetime. */
+	int enters;
+	/* General key type. */
+	KeyType type : 2;
+	/* Whether RHS should be treated as if there are no user mappings. */
+	unsigned int no_remap : 1;
+	/* Postpone free() call for proper lazy deletion. */
+	unsigned int deleted : 1;
+	/* Postpone UI updates until RHS is done executing. */
+	unsigned int silent : 1;
+	/* Do not use short wait to resolve conflict against builtin mapping, do long
+	 * wait instead. */
+	unsigned int wait : 1;
 	key_conf_t conf;
 	struct key_chunk_t *child;
 	struct key_chunk_t *parent;
@@ -80,6 +89,8 @@ static size_t counter;
 static size_t enters_counter;
 /* Shows whether a mapping handler is being executed at the moment. */
 static int inside_mapping;
+/* User-provided callback for silencing UI. */
+static vle_silence_func silence_ui;
 
 static void free_forest(key_chunk_t *forest, size_t size);
 static void free_tree(key_chunk_t *root);
@@ -101,6 +112,7 @@ static int contains_chain(key_chunk_t *root, const wchar_t *begin,
 static int execute_next_keys(key_chunk_t *curr, const wchar_t keys[],
 		key_info_t *key_info, keys_info_t *keys_info, int has_duplicate,
 		int no_remap);
+static int needs_waiting(const key_chunk_t *curr);
 static int dispatch_key(key_info_t key_info, keys_info_t *keys_info,
 		key_chunk_t *curr, const wchar_t keys[]);
 static int has_def_handler(void);
@@ -121,12 +133,12 @@ static int add_list_of_keys(key_chunk_t *root, keys_add_info_t cmds[],
 static key_chunk_t * add_keys_inner(key_chunk_t *root, const wchar_t *keys);
 static void list_chunk(const key_chunk_t *chunk, const wchar_t lhs[],
 		void *arg);
-static void inc_counter(const keys_info_t *const keys_info, const size_t by);
+static void inc_counter(const keys_info_t *keys_info, size_t by);
 static int is_recursive(void);
-static int execute_mapping_handler(const key_conf_t *const info,
-		key_info_t key_info, keys_info_t *const keys_info);
-static void pre_execute_mapping_handler(const keys_info_t *const keys_info);
-static void post_execute_mapping_handler(const keys_info_t *const keys_info);
+static int execute_mapping_handler(const key_conf_t *info,
+		key_info_t key_info, keys_info_t *keys_info);
+static void pre_execute_mapping_handler(const keys_info_t *keys_info);
+static void post_execute_mapping_handler(const keys_info_t *keys_info);
 static void keys_suggest(const key_chunk_t *root, const wchar_t keys[],
 		const wchar_t prefix[], vle_keys_list_cb cb, int custom_only,
 		int fold_subkeys);
@@ -138,13 +150,15 @@ static void suggest_chunk(const key_chunk_t *chunk, const wchar_t lhs[],
 		void *arg);
 
 void
-vle_keys_init(int modes_count, int *key_mode_flags)
+vle_keys_init(int modes_count, int *key_mode_flags, vle_silence_func silence)
 {
 	assert(key_mode_flags != NULL);
 	assert(modes_count > 0);
+	assert(silence != NULL);
 
 	max_modes = modes_count;
 	mode_flags = key_mode_flags;
+	silence_ui = silence;
 
 	builtin_cmds_root = calloc(modes_count, sizeof(*builtin_cmds_root));
 	assert(builtin_cmds_root != NULL);
@@ -583,9 +597,16 @@ execute_next_keys(key_chunk_t *curr, const wchar_t keys[], key_info_t *key_info,
 
 		if(wait_point)
 		{
-			const int with_input = (mode_flags[vle_mode_get()] & MF_USES_INPUT);
 			if(!keys_info->after_wait)
 			{
+				if(needs_waiting(curr))
+				{
+					/* Wait flag on a user mapping should turn short wait into indefinite
+					 * wait when there is a conflict with builtin mapping, pretending
+					 * that there is no such conflict is enough to get the effect. */
+					has_duplicate = 0;
+				}
+				const int with_input = (mode_flags[vle_mode_get()] & MF_USES_INPUT);
 				return (with_input || has_duplicate) ? KEYS_WAIT_SHORT : KEYS_WAIT;
 			}
 		}
@@ -609,6 +630,27 @@ execute_next_keys(key_chunk_t *curr, const wchar_t keys[], key_info_t *key_info,
 	return dispatch_key(*key_info, keys_info, curr, keys);
 }
 
+/* Checks whether any child of the node has wait flag set.  Returns non-zero if
+ * so, otherwise zero is returned. */
+static int
+needs_waiting(const key_chunk_t *curr)
+{
+	if(curr->wait)
+	{
+		return 1;
+	}
+
+	for(curr = curr->child; curr != NULL; curr = curr->next)
+	{
+		if(needs_waiting(curr))
+		{
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
 /* Performs action associated with the key (in curr), if any.  Returns error
  * code. */
 static int
@@ -634,6 +676,11 @@ dispatch_key(key_info_t key_info, keys_info_t *keys_info, key_chunk_t *curr,
 	}
 	else
 	{
+		if(curr->silent)
+		{
+			silence_ui(1);
+		}
+
 		int result = has_def_handler() ? 0 : KEYS_UNKNOWN;
 
 		/* Protect chunk from deletion while it's in use. */
@@ -674,6 +721,11 @@ dispatch_key(key_info_t key_info, keys_info_t *keys_info, key_chunk_t *curr,
 			}
 		}
 
+		if(curr->silent)
+		{
+			silence_ui(0);
+		}
+
 		/* Release the chunk, this will free it if deletion was attempted. */
 		leave_chunk(curr);
 
@@ -859,7 +911,8 @@ combine_counts(int count_a, int count_b)
 }
 
 int
-vle_keys_user_add(const wchar_t lhs[], const wchar_t rhs[], int mode, int no_r)
+vle_keys_user_add(const wchar_t lhs[], const wchar_t rhs[], int mode,
+		int flags)
 {
 	key_chunk_t *curr = add_keys_inner(&user_cmds_root[mode], lhs);
 	if(curr == NULL)
@@ -869,12 +922,14 @@ vle_keys_user_add(const wchar_t lhs[], const wchar_t rhs[], int mode, int no_r)
 
 	if(curr->type == USER_CMD)
 	{
-		free((void*)curr->conf.data.cmd);
+		free(curr->conf.data.cmd);
 	}
 
 	curr->type = USER_CMD;
 	curr->conf.data.cmd = vifm_wcsdup(rhs);
-	curr->no_remap = no_r;
+	curr->no_remap = ((flags & KEYS_FLAG_NOREMAP) != 0);
+	curr->silent = ((flags & KEYS_FLAG_SILENT) != 0);
+	curr->wait = ((flags & KEYS_FLAG_WAIT) != 0);
 	return 0;
 }
 
@@ -1029,6 +1084,8 @@ add_keys_inner(key_chunk_t *root, const wchar_t *keys)
 			c->enters = 0;
 			c->deleted = 0;
 			c->no_remap = 1;
+			c->silent = 0;
+			c->wait = 0;
 			if(prev == NULL)
 				curr->child = c;
 			else
@@ -1099,7 +1156,7 @@ vle_keys_counter(void)
 /* Increments counter if we are at the first level of key parsing recursion and
  * the key sequence isn't mapped. */
 static void
-inc_counter(const keys_info_t *const keys_info, const size_t by)
+inc_counter(const keys_info_t *keys_info, size_t by)
 {
 	assert(enters_counter > 0);
 
@@ -1125,8 +1182,8 @@ vle_keys_inside_mapping(void)
 /* Executes handler for a mapping, if any.  Error or success code is
  * returned. */
 static int
-execute_mapping_handler(const key_conf_t *const info, key_info_t key_info,
-		keys_info_t *const keys_info)
+execute_mapping_handler(const key_conf_t *info, key_info_t key_info,
+		keys_info_t *keys_info)
 {
 	if(info->data.handler != NULL)
 	{
@@ -1140,7 +1197,7 @@ execute_mapping_handler(const key_conf_t *const info, key_info_t key_info,
 
 /* Pre-execution of a mapping handler callback. */
 static void
-pre_execute_mapping_handler(const keys_info_t *const keys_info)
+pre_execute_mapping_handler(const keys_info_t *keys_info)
 {
 	inside_mapping += keys_info->mapped != 0;
 	assert(inside_mapping >= 0 && "Calls to pre/post funcs should be balanced");
@@ -1148,7 +1205,7 @@ pre_execute_mapping_handler(const keys_info_t *const keys_info)
 
 /* Post-execution of a mapping handler callback. */
 static void
-post_execute_mapping_handler(const keys_info_t *const keys_info)
+post_execute_mapping_handler(const keys_info_t *keys_info)
 {
 	inside_mapping -= keys_info->mapped != 0;
 	assert(inside_mapping >= 0 && "Calls to pre/post funcs should be balanced");
