@@ -89,7 +89,6 @@ static int extract_name(const char **in, VariableType *type, size_t buf_len,
 static int extract_op(const char **in, VariableOperation *vo);
 static int parse_name(const char **in, const char first[], const char other[],
 		size_t buf_len, char buf[]);
-static void report_parsing_error(ParsingErrors error);
 static int is_valid_op(const char name[], VariableType vt,
 		VariableOperation vo);
 static int perform_op(const char name[], VariableType vt,
@@ -254,7 +253,7 @@ let_variables(const char cmd[])
 
 	cmd = skip_whitespace(cmd);
 
-	parsing_error = parse(cmd, &res_var);
+	parsing_error = parse(cmd, 1, &res_var);
 	if(parsing_error != PE_NO_ERROR)
 	{
 		report_parsing_error(parsing_error);
@@ -274,7 +273,7 @@ let_variables(const char cmd[])
 		return -1;
 	}
 
-	str_val = var_to_string(res_var);
+	str_val = var_to_str(res_var);
 
 	error = perform_op(name, type, op, str_val);
 
@@ -397,33 +396,6 @@ parse_name(const char **in, const char first[], const char other[],
 	return 0;
 }
 
-/* Appends error message with details to the error stream. */
-static void
-report_parsing_error(ParsingErrors error)
-{
-	switch(error)
-	{
-		case PE_NO_ERROR:
-			/* Not an error. */
-			break;
-		case PE_INVALID_EXPRESSION:
-			vle_tb_append_linef(vle_err, "%s: %s", "Invalid expression",
-					get_last_position());
-			break;
-		case PE_INVALID_SUBEXPRESSION:
-			vle_tb_append_linef(vle_err, "%s: %s", "Invalid subexpression",
-					get_last_position());
-			break;
-		case PE_MISSING_QUOTE:
-			vle_tb_append_linef(vle_err, "%s: %s",
-					"Invalid :let expression (missing quote)", get_last_position());
-			break;
-		case PE_INTERNAL:
-			vle_tb_append_line(vle_err, "Internal error");
-			break;
-	}
-}
-
 /* Validates operation on a specific variable type.  Returns non-zero for valid
  * operation, otherwise zero is returned. */
 static int
@@ -815,7 +787,7 @@ complete_builtinvars(const char var[], const char **start)
 	{
 		if(strncmp(builtin_vars[i].name, var, len) == 0)
 		{
-			char *const str_val = var_to_string(builtin_vars[i].val);
+			char *const str_val = var_to_str(builtin_vars[i].val);
 			vle_compl_add_match(builtin_vars[i].name, str_val);
 			free(str_val);
 		}
