@@ -384,6 +384,10 @@ get_char_async_loop(WINDOW *win, wint_t *c, int timeout)
 				{
 					*c = K(*c);
 				}
+				else if(*c == L'\0')
+				{
+					*c = WC_C_SPACE;
+				}
 				return result;
 			}
 
@@ -403,12 +407,6 @@ process_scheduled_updates(void)
 
 	ui_stat_job_bar_check_for_updates();
 
-	if(vle_mode_is(CMDLINE_MODE))
-	{
-		/* Redrawing implicitly updates hardware cursor, so temporary disable it. */
-		curs_set(0);
-	}
-
 	if(vle_mode_get_primary() != MENU_MODE)
 	{
 		need_redraw += (process_scheduled_updates_of_view(curr_view) != 0);
@@ -421,11 +419,6 @@ process_scheduled_updates(void)
 	{
 		modes_redraw();
 	}
-
-	if(vle_mode_is(CMDLINE_MODE))
-	{
-		curs_set(1);
-	}
 }
 
 /* Performs postponed updates for the view, if any.  Returns non-zero if
@@ -463,7 +456,7 @@ update_hardware_cursor(void)
 	if(ui_sb_multiline())
 	{
 		checked_wmove(status_bar, 0, 0);
-		wrefresh(status_bar);
+		ui_refresh_win(status_bar);
 		return;
 	}
 
@@ -471,18 +464,18 @@ update_hardware_cursor(void)
 	{
 		case MENU_MODE:
 		case FILE_INFO_MODE:
-		case MORE_MODE: wrefresh(menu_win);       break;
+		case MORE_MODE: ui_refresh_win(menu_win);       break;
 		case CHANGE_MODE:
-		case ATTR_MODE: wrefresh(change_win);     break;
-		case MSG_MODE:  wrefresh(error_win);      break;
-		case VIEW_MODE: wrefresh(curr_view->win); break;
-		case SORT_MODE: wrefresh(sort_win);       break;
+		case ATTR_MODE: ui_refresh_win(change_win);     break;
+		case MSG_MODE:  ui_refresh_win(error_win);      break;
+		case VIEW_MODE: ui_refresh_win(curr_view->win); break;
+		case SORT_MODE: ui_refresh_win(sort_win);       break;
 
 		case NORMAL_MODE:
 		case VISUAL_MODE:
 			if(should_check_views_for_changes())
 			{
-				wrefresh(curr_view->win);
+				ui_refresh_win(curr_view->win);
 			}
 			break;
 	}
@@ -515,7 +508,7 @@ update_input_buf(void)
 	{
 		werase(input_win);
 		wprintw(input_win, "%ls", (curr_input_buf == NULL) ? L"" : curr_input_buf);
-		wrefresh(input_win);
+		ui_refresh_win(input_win);
 	}
 }
 
@@ -559,6 +552,8 @@ display_suggestion_box(const wchar_t input[])
 	if((cfg.sug.flags & SF_REGISTERS) &&
 			input[prefix] == L'"' && input[prefix + 1U] == L'\0')
 	{
+		regs_sync_from_shared_memory();
+
 		/* No vle_compl_finish_group() after this to prevent sorting and
 		 * deduplication. */
 		regs_suggest(&process_suggestion, cfg.sug.maxregfiles);
@@ -616,7 +611,7 @@ draw_suggestion_box(void)
 				max_title_width);
 	}
 
-	wrefresh(win);
+	ui_refresh_win(win);
 }
 
 /* Picks window to use for suggestion box and prepares it for displaying data.
