@@ -47,10 +47,13 @@
 #include "int/fuse.h"
 #include "modes/dialogs/msg_dialog.h"
 #include "modes/modes.h"
+#include "modes/view.h"
 #include "ui/cancellation.h"
+#include "ui/column_view.h"
 #include "ui/fileview.h"
 #include "ui/statusbar.h"
 #include "ui/statusline.h"
+#include "ui/tabs.h"
 #include "ui/ui.h"
 #include "utils/dynarray.h"
 #include "utils/env.h"
@@ -81,7 +84,6 @@
 #include "status.h"
 #include "types.h"
 
-static void init_view(view_t *view);
 static void init_flist(view_t *view);
 static void reset_view(view_t *view);
 static void init_view_history(view_t *view);
@@ -131,6 +133,7 @@ static int add_file_entry_to_view(const char name[], const void *data,
 		void *param);
 static void sort_dir_list(int msg, view_t *view);
 static void merge_lists(view_t *view, dir_entry_t *entries, int len);
+TSTATIC void check_file_uniqueness(view_t *view);
 static void add_to_trie(trie_t *trie, view_t *view, dir_entry_t *entry);
 static int is_in_trie(trie_t *trie, view_t *view, dir_entry_t *entry,
 		void **data);
@@ -141,11 +144,11 @@ static void add_parent_entry(view_t *view, dir_entry_t **entries, int *count);
 static void init_dir_entry(view_t *view, dir_entry_t *entry, const char name[]);
 static dir_entry_t * alloc_dir_entry(dir_entry_t **list, int list_size);
 static int tree_has_changed(const dir_entry_t *entries, size_t nchildren);
-TSTATIC void pick_cd_path(view_t *view, const char base_dir[],
-		const char path[], int *updir, char buf[], size_t buf_size);
 static void find_dir_in_cdpath(const char base_dir[], const char dst[],
 		char buf[], size_t buf_size);
 static entries_t list_sibling_dirs(view_t *view);
+static entries_t flist_list_in(view_t *view, const char path[], int only_dirs,
+		int can_include_parent);
 static dir_entry_t * pick_sibling(view_t *view, entries_t parent_dirs,
 		int offset, int wrap, int *wrapped);
 static int iter_entries(view_t *view, dir_entry_t **entry,
@@ -175,16 +178,15 @@ init_filelists(void)
 {
 	fview_init();
 
-	init_view(&rwin);
-	init_view(&lwin);
+	flist_init_view(&rwin);
+	flist_init_view(&lwin);
 
 	curr_view = &lwin;
 	other_view = &rwin;
 }
 
-/* Loads initial display values into view structure. */
-static void
-init_view(view_t *view)
+void
+flist_init_view(view_t *view)
 {
 	init_flist(view);
 	fview_view_init(view);
@@ -203,6 +205,7 @@ init_flist(view_t *view)
 	view->history_num = 0;
 	view->history_pos = 0;
 	view->on_slow_fs = 0;
+	view->has_dups = 0;
 
 	view->matches = 0;
 
@@ -222,6 +225,95 @@ init_flist(view_t *view)
 	view->list_rows = 1;
 }
 
+void
+flist_free_view(view_t *view)
+{
+	/* For the application, we don't need to zero out fields after freeing them,
+	 * but doing so allows reusing this function in tests. */
+
+	int i;
+
+	for(i = 0; i < view->list_rows; ++i)
+	{
+		fentry_free(view, &view->dir_entry[i]);
+	}
+	dynarray_free(view->dir_entry);
+	view->dir_entry = NULL;
+	view->list_rows = 0;
+
+	for(i = 0; i < view->custom.entry_count; ++i)
+	{
+		fentry_free(view, &view->custom.entries[i]);
+	}
+	dynarray_free(view->custom.entries);
+	view->custom.entries = NULL;
+	view->custom.entry_count = 0;
+
+	update_string(&view->custom.next_title, NULL);
+	update_string(&view->custom.orig_dir, NULL);
+	update_string(&view->custom.title, NULL);
+	trie_free(view->custom.excluded_paths);
+	trie_free(view->custom.paths_cache);
+	view->custom.excluded_paths = NULL;
+	view->custom.paths_cache = NULL;
+
+	for(i = 0; i < view->local_filter.entry_count; ++i)
+	{
+		fentry_free(view, &view->local_filter.entries[i]);
+	}
+	dynarray_free(view->local_filter.entries);
+	view->local_filter.entries = NULL;
+	view->local_filter.entry_count = 0;
+
+	/* Two pointer fields below don't contain valid data that needs to be freed,
+	 * zeroing them for tests and to at least mention them to signal that they
+	 * weren't forgotten. */
+	view->local_filter.unfiltered = NULL;
+	view->local_filter.saved = NULL;
+	view->local_filter.unfiltered_count = 0;
+
+	update_string(&view->local_filter.prev, NULL);
+	free(view->local_filter.poshist);
+	view->local_filter.poshist = NULL;
+
+	filter_dispose(&view->local_filter.filter);
+	filter_dispose(&view->auto_filter);
+	matcher_free(view->manual_filter);
+	view->manual_filter = NULL;
+	update_string(&view->prev_manual_filter, NULL);
+	update_string(&view->prev_auto_filter, NULL);
+
+	view->custom.type = CV_REGULAR;
+
+	fswatch_free(view->watch);
+	view->watch = NULL;
+
+	flist_free_cache(view, &view->left_column);
+	flist_free_cache(view, &view->right_column);
+
+	free_string_array(view->saved_selection, view->nsaved_selection);
+	view->nsaved_selection = 0;
+	view->saved_selection = NULL;
+
+	flist_hist_resize(view, 0);
+
+	cs_reset(&view->cs);
+
+	columns_free(view->columns);
+	view->columns = NULL;
+
+	update_string(&view->view_columns, NULL);
+	update_string(&view->view_columns_g, NULL);
+	update_string(&view->sort_groups, NULL);
+	update_string(&view->sort_groups_g, NULL);
+	update_string(&view->preview_prg, NULL);
+	update_string(&view->preview_prg_g, NULL);
+
+	view_info_free(view->vi);
+
+	regfree(&view->primary_group);
+}
+
 void
 reset_views(void)
 {
@@ -388,7 +480,7 @@ navigate_to_file(view_t *view, const char dir[], const char file[],
 
 	if(paths_are_equal(view->curr_dir, dir))
 	{
-		(void)ensure_file_is_selected(view, file);
+		(void)fpos_ensure_selected(view, file);
 	}
 }
 
@@ -398,7 +490,7 @@ static int
 navigate_to_file_in_custom_view(view_t *view, const char dir[],
 		const char file[])
 {
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 
 	snprintf(full_path, sizeof(full_path), "%s/%s", dir, file);
 
@@ -436,7 +528,7 @@ change_directory(view_t *view, const char directory[])
 {
 	/* TODO: refactor this big function change_directory(). */
 
-	char dir_dup[PATH_MAX];
+	char dir_dup[PATH_MAX + 1];
 	const int was_in_custom_view = flist_custom_active(view);
 	int location_changed;
 
@@ -451,7 +543,7 @@ change_directory(view_t *view, const char directory[])
 	}
 	else
 	{
-		char newdir[PATH_MAX];
+		char newdir[PATH_MAX + 1];
 		const char *const dir = flist_get_dir(view);
 #ifdef _WIN32
 		if(directory[0] == '/')
@@ -466,21 +558,17 @@ change_directory(view_t *view, const char directory[])
 		canonicalize_path(newdir, dir_dup, sizeof(dir_dup));
 	}
 
-#ifdef _WIN32
-	to_forward_slash(dir_dup);
-#endif
+	system_to_internal_slashes(dir_dup);
 
 	if(cfg.chase_links)
 	{
-		char real_path[PATH_MAX];
+		char real_path[PATH_MAX + 1];
 		if(os_realpath(dir_dup, real_path) == real_path)
 		{
 			/* Do this on success only, if realpath() fails, just go with the original
 			 * path. */
 			canonicalize_path(real_path, dir_dup, sizeof(dir_dup));
-#ifdef _WIN32
-			to_forward_slash(real_path);
-#endif
+			system_to_internal_slashes(dir_dup);
 		}
 	}
 
@@ -497,13 +585,12 @@ change_directory(view_t *view, const char directory[])
 
 	location_changed = (stroscmp(dir_dup, flist_get_dir(view)) != 0);
 
-	/* Check if we're exiting from a FUSE mounted top level directory and the
-	 * other pane isn't in it or any of it subdirectories.
-	 * If so, unmount & let FUSE serialize */
+	/* Check if we're exiting from a FUSE mounted top level directory. */
 	if(is_parent_dir(directory) && fuse_is_mount_point(view->curr_dir))
 	{
-		view_t *other = (view == curr_view) ? other_view : curr_view;
-		if(!path_starts_with(other->curr_dir, view->curr_dir))
+		/* No other pane in any tab should be inside subtree that might be
+		 * unmounted. */
+		if(tabs_visitor_count(view->curr_dir) == 1)
 		{
 			const int r = fuse_try_unmount(view);
 			if(r != 0)
@@ -696,7 +783,7 @@ char *
 get_typed_entry_fpath(const dir_entry_t *entry)
 {
 	const char *type_suffix;
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 
 	type_suffix = fentry_is_dir(entry) ? "/" : "";
 
@@ -1019,7 +1106,7 @@ flist_custom_finish_internal(view_t *view, CVType type, int reload,
 	sort_dir_list(0, view);
 
 	ui_view_schedule_redraw(view);
-	flist_ensure_pos_is_valid(view);
+	fpos_ensure_valid_pos(view);
 
 	return 0;
 }
@@ -1029,6 +1116,8 @@ flist_custom_finish_internal(view_t *view, CVType type, int reload,
 static void
 on_location_change(view_t *view, int force)
 {
+	view->has_dups = 0;
+
 	free_dir_entries(view, &view->local_filter.entries,
 			&view->local_filter.entry_count);
 
@@ -1085,7 +1174,7 @@ flist_custom_exclude(view_t *view, int selection_only)
 	excluded = trie_create();
 	while(iter_selection_or_current(view, &entry))
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 
 		entry->temporary = 1;
 
@@ -1181,7 +1270,7 @@ got_excluded(view_t *view, const dir_entry_t *entry, void *arg)
 	trie_t *const excluded = arg;
 	int excluded_entry;
 
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 	get_full_path_of(entry, sizeof(full_path), full_path);
 
 	excluded_entry = (trie_get(excluded, full_path, &data) == 0);
@@ -1198,7 +1287,7 @@ exclude_temporary_entries(view_t *view)
 	(void)zap_entries(view, view->custom.entries, &view->custom.entry_count,
 			&is_temporary, NULL, 1, 1);
 
-	flist_ensure_pos_is_valid(view);
+	fpos_ensure_valid_pos(view);
 	ui_view_schedule_redraw(view);
 
 	flist_sel_recount(view);
@@ -1355,7 +1444,7 @@ flist_get_dir(const view_t *view)
 void
 flist_goto_by_path(view_t *view, const char path[])
 {
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 	const char *const name = get_last_path_component(path);
 
 	get_current_full_path(view, sizeof(full_path), full_path);
@@ -1368,10 +1457,10 @@ flist_goto_by_path(view_t *view, const char path[])
 			strcmp(name, "..") == 0)
 	{
 		int pos;
-		char dir_only[PATH_MAX];
+		char dir_only[PATH_MAX + 1];
 
 		snprintf(dir_only, sizeof(dir_only), "%.*s", (int)(name - path), path);
-		pos = flist_find_entry(view, name, dir_only);
+		pos = fpos_find_entry(view, name, dir_only);
 		if(pos != -1)
 		{
 			view->list_pos = pos;
@@ -1386,7 +1475,7 @@ dir_entry_t *
 entry_from_path(view_t *view, dir_entry_t *entries, int count,
 		const char path[])
 {
-	char canonic_path[PATH_MAX];
+	char canonic_path[PATH_MAX + 1];
 	const char *fname;
 	int i;
 
@@ -1396,7 +1485,7 @@ entry_from_path(view_t *view, dir_entry_t *entries, int count,
 	fname = get_last_path_component(canonic_path);
 	for(i = 0; i < count; ++i)
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		dir_entry_t *const entry = &entries[i];
 
 		if(stroscmp(entry->name, fname) != 0)
@@ -1445,7 +1534,7 @@ fentry_get_dir_info(const view_t *view, const dir_entry_t *entry,
 
 	if(nitems != NULL)
 	{
-		if(nitems_res.value == DCACHE_UNKNOWN && !is_slow_fs)
+		if(!nitems_res.is_valid && !is_slow_fs)
 		{
 			nitems_res.value = entry_calc_nitems(entry);
 		}
@@ -1895,7 +1984,7 @@ update_entries_data(view_t *view)
 	int i;
 	for(i = 0; i < view->list_rows; ++i)
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		dir_entry_t *const entry = &view->dir_entry[i];
 
 		/* Fake entries do not map onto files in file system. */
@@ -1994,7 +2083,7 @@ zap_entries(view_t *view, dir_entry_t *entries, int *count, zap_filter filter,
 		parent = i - entry->child_pos - (i - j);
 		if(remove_subtrees && parent == j - 1 && entries[parent].child_count == 0)
 		{
-			char full_path[PATH_MAX];
+			char full_path[PATH_MAX + 1];
 			char *path;
 
 			int pos = i + (entry->child_count + 1);
@@ -2125,6 +2214,8 @@ start_dir_list_change(view_t *view, dir_entry_t **entries, int *len, int reload)
 static void
 finish_dir_list_change(view_t *view, dir_entry_t *entries, int len)
 {
+	check_file_uniqueness(view);
+
 	if(entries != NULL)
 	{
 		merge_lists(view, entries, len);
@@ -2178,7 +2269,7 @@ add_file_entry_to_view(const char name[], const void *data, void *param)
 void
 resort_dir_list(int msg, view_t *view)
 {
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 	const int top_delta = view->list_pos - view->top_line;
 	if(view->list_pos < view->list_rows)
 	{
@@ -2247,32 +2338,63 @@ merge_lists(view_t *view, dir_entry_t *entries, int len)
 		view->selected_files += (entry->selected != 0);
 
 		/* Update cursor position in a smart way. */
-		dist = (dir_entry_t*)data - entries - prev_pos;
+		dist = (dir_entry_t *)data - entries - prev_pos;
 		closest_dist = correct_pos(view, i, dist, closest_dist);
 	}
 
 	trie_free(prev_names);
 }
 
-/* Adds view entry into the trie mapping its name to entry structure. */
+/* Checks that entries don't have the same name (for non-cv).  And if there are
+ * duplicates shows a warning to the user (shown each time broken directory is
+ * entered) and drops duplicates. */
+TSTATIC void
+check_file_uniqueness(view_t *view)
+{
+	trie_t *file_names = trie_create();
+
+	int had_dups = view->has_dups;
+
+	int i;
+	for(i = 0; i < view->list_rows; ++i)
+	{
+		add_to_trie(file_names, view, &view->dir_entry[i]);
+	}
+
+	trie_free(file_names);
+
+	if(view->has_dups)
+	{
+		(void)exclude_temporary_entries(view);
+		if(!had_dups)
+		{
+			show_error_msg("Broken File System", "Underlying file system seems to "
+					"report duplicated file names.  Only one entry will be shown.");
+		}
+	}
+}
+
+/* Adds view entry into the trie mapping its name to entry structure.
+ * Duplicated entries of a non-custom view are marked as temporary. */
 static void
 add_to_trie(trie_t *trie, view_t *view, dir_entry_t *entry)
 {
-	int error;
-
 	if(flist_custom_active(view))
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full_path), full_path);
-		error = trie_set(trie, full_path, entry);
-	}
-	else
-	{
-		error = trie_set(trie, entry->name, entry);
+
+		int error = trie_set(trie, full_path, entry);
+		assert(error == 0 && "Duplicated file names in the list?");
+		(void)error;
+		return;
 	}
 
-	assert(error == 0 && "Duplicated file names in the list?");
-	(void)error;
+	if(trie_set(trie, entry->name, entry) != 0)
+	{
+		entry->temporary = 1;
+		view->has_dups = 1;
+	}
 }
 
 /* Looks up entry in the trie by its name.  Retrieves directory entry stored by
@@ -2285,7 +2407,7 @@ is_in_trie(trie_t *trie, view_t *view, dir_entry_t *entry, void **data)
 
 	if(flist_custom_active(view))
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full_path), full_path);
 		error = trie_get(trie, full_path, data);
 	}
@@ -2483,7 +2605,7 @@ fentry_free(const view_t *view, dir_entry_t *entry)
 	free(entry->name);
 	entry->name = NULL;
 
-	if(entry->origin != &view->curr_dir[0])
+	if(entry->origin != &lwin.curr_dir[0] && entry->origin != &rwin.curr_dir[0])
 	{
 		free(entry->origin);
 		entry->origin = NULL;
@@ -2621,7 +2743,7 @@ tree_has_changed(const dir_entry_t *entries, size_t nchildren)
 		const dir_entry_t *const entry = &entries[pos];
 		if(entry->type == FT_DIR && !is_parent_dir(entry->name))
 		{
-			char full_path[PATH_MAX];
+			char full_path[PATH_MAX + 1];
 			struct stat s;
 
 			get_full_path_of(entry, sizeof(full_path), full_path);
@@ -2689,14 +2811,29 @@ flist_free_cache(view_t *view, cached_entries_t *cache)
 	cache->watch = NULL;
 }
 
+void
+flist_update_origins(view_t *view, const char from[], char to[])
+{
+	int i;
+	for(i = 0; i < view->list_rows; ++i)
+	{
+		dir_entry_t *const entry = &view->dir_entry[i];
+		if(entry->origin == from)
+		{
+			entry->origin = to;
+		}
+	}
+}
+
 int
-cd_is_possible(const char *path)
+cd_is_possible(const char path[])
 {
 	if(!is_valid_dir(path))
 	{
 		LOG_SERROR_MSG(errno, "Can't access \"%s\"", path);
 
-		show_error_msgf("Destination doesn't exist", "\"%s\"", path);
+		show_error_msgf("Destination doesn't exist or isn't a directory", "\"%s\"",
+				path);
 		return 0;
 	}
 	else if(!directory_accessible(path))
@@ -2743,7 +2880,7 @@ load_saving_pos(view_t *view)
 }
 
 int
-window_shows_dirlist(const view_t *const view)
+window_shows_dirlist(const view_t *view)
 {
 	if(curr_stats.number_of_windows == 1 && view == other_view)
 	{
@@ -2802,11 +2939,11 @@ pane_in_dir(const view_t *view, const char path[])
 int
 cd(view_t *view, const char base_dir[], const char path[])
 {
-	char dir[PATH_MAX];
-	char canonic_dir[PATH_MAX];
+	char dir[PATH_MAX + 1];
+	char canonic_dir[PATH_MAX + 1];
 	int updir;
 
-	pick_cd_path(view, base_dir, path, &updir, dir, sizeof(dir));
+	flist_pick_cd_path(view, base_dir, path, &updir, dir, sizeof(dir));
 	to_canonic_path(dir, base_dir, canonic_dir, sizeof(canonic_dir));
 
 	if(updir)
@@ -2832,11 +2969,9 @@ cd(view_t *view, const char base_dir[], const char path[])
 	return 0;
 }
 
-/* Picks new directory or requested going up one level judging from supplied
- * base directory, desired location and current location of the view. */
-TSTATIC void
-pick_cd_path(view_t *view, const char base_dir[], const char path[], int *updir,
-		char buf[], size_t buf_size)
+void
+flist_pick_cd_path(view_t *view, const char base_dir[], const char path[],
+		int *updir, char buf[], size_t buf_size)
 {
 	char *arg;
 
@@ -2935,7 +3070,7 @@ go_to_sibling_dir(view_t *view, int offset, int wrap)
 	entry = pick_sibling(view, parent_dirs, offset, wrap, &save_msg);
 	if(entry != NULL)
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full_path), full_path);
 		if(change_directory(view, full_path) >= 0)
 		{
@@ -2986,7 +3121,9 @@ list_sibling_dirs(view_t *view)
 	return parent_dirs;
 }
 
-entries_t
+/* Lists files of specified directory.  Returns the list, which is of length -1
+ * on error. */
+static entries_t
 flist_list_in(view_t *view, const char path[], int only_dirs,
 		int can_include_parent)
 {
@@ -3003,7 +3140,6 @@ flist_list_in(view_t *view, const char path[], int only_dirs,
 
 	for(i = 0; i < len; ++i)
 	{
-		char *full_path;
 		dir_entry_t *entry;
 		int is_dir;
 
@@ -3012,9 +3148,15 @@ flist_list_in(view_t *view, const char path[], int only_dirs,
 			continue;
 		}
 
-		full_path = format_str("%s/%s", path, list[i]);
+		char *full_path = format_str("%s/%s", path, list[i]);
 		entry = entry_list_add(view, &siblings.entries, &siblings.nentries,
 				full_path);
+		free(full_path);
+
+		if(entry == NULL)
+		{
+			continue;
+		}
 
 		is_dir = fentry_is_dir(entry);
 		if((only_dirs && !is_dir) ||
@@ -3023,8 +3165,6 @@ flist_list_in(view_t *view, const char path[], int only_dirs,
 			fentry_free(view, entry);
 			--siblings.nentries;
 		}
-
-		free(full_path);
 	}
 	free_string_array(list, len);
 
@@ -3354,19 +3494,6 @@ clear_marking(view_t *view)
 	}
 }
 
-int
-flist_count_marked(view_t *const view)
-{
-	int i;
-	int count = 0;
-	for(i = 0; i < view->list_rows; ++i)
-	{
-		const dir_entry_t *const entry = &view->dir_entry[i];
-		count += (entry->marked && fentry_is_valid(entry));
-	}
-	return count;
-}
-
 void
 flist_custom_set(view_t *view, const char title[], const char path[],
 		char *lines[], int nlines)
@@ -3392,11 +3519,7 @@ flist_custom_set(view_t *view, const char title[], const char path[],
 void
 flist_custom_add_spec(view_t *view, const char line[])
 {
-	int line_num;
-	/* Skip empty lines. */
-	char *const path = (skip_whitespace(line)[0] == '\0')
-	                 ? NULL
-	                 : parse_file_spec(line, &line_num, flist_get_dir(view));
+	char *const path = parse_line_for_path(line, flist_get_dir(view));
 	if(path != NULL)
 	{
 		flist_custom_add(view, path);
@@ -3413,7 +3536,7 @@ flist_custom_end(view_t *view, int very)
 		return;
 	}
 
-	flist_set_pos(view, 0);
+	fpos_set_pos(view, 0);
 }
 
 void
@@ -3488,7 +3611,7 @@ fentry_is_dir(const dir_entry_t *entry)
 int
 flist_load_tree(view_t *view, const char path[])
 {
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 	get_current_full_path(view, sizeof(full_path), full_path);
 
 	if(flist_load_tree_internal(view, path, 0) != 0)
@@ -3567,7 +3690,7 @@ flist_load_tree_internal(view_t *view, const char path[], int reload)
 static int
 make_tree(view_t *view, const char path[], int reload, trie_t *excluded_paths)
 {
-	char canonic_path[PATH_MAX];
+	char canonic_path[PATH_MAX + 1];
 	int nfiltered;
 	CVType type;
 	const int from_custom = flist_custom_active(view)
