@@ -55,7 +55,7 @@ const char *
 apply_mods(const char path[], const char parent[], const char mod[],
 		int for_shell)
 {
-	static char buf[PATH_MAX];
+	static char buf[PATH_MAX + 1];
 	int napplied = 0;
 
 	copy_str(buf, sizeof(buf), path);
@@ -72,14 +72,12 @@ apply_mods(const char path[], const char parent[], const char mod[],
 		napplied++;
 	}
 
-#ifdef _WIN32
 	/* This is needed to run something like explorer.exe, which isn't smart enough
 	 * to understand forward slashes. */
 	if(for_shell && curr_stats.shell_type != ST_CMD && napplied == 0)
 	{
-		to_back_slash(buf);
+		internal_to_system_slashes(buf);
 	}
-#endif
 
 	return buf;
 }
@@ -89,13 +87,11 @@ static const char *
 apply_mod(const char *path, const char *parent, const char *mod, int *mod_len,
 		int for_shell)
 {
-	char path_buf[PATH_MAX];
-	static char buf[PATH_MAX];
+	char path_buf[PATH_MAX + 1];
+	static char buf[PATH_MAX + 1];
 
 	copy_str(path_buf, sizeof(path_buf), path);
-#ifdef _WIN32
-	to_forward_slash(path_buf);
-#endif
+	system_to_internal_slashes(path_buf);
 
 	*mod_len = 2;
 	if(starts_with_lit(mod, ":p"))
@@ -121,17 +117,15 @@ apply_mod(const char *path, const char *parent, const char *mod, int *mod_len,
 	else
 		return NULL;
 
-#ifdef _WIN32
 	/* This is needed to run something like explorer.exe, which isn't smart enough
 	 * to understand forward slashes. */
 	if(for_shell && curr_stats.shell_type != ST_CMD)
 	{
 		if(!starts_with_lit(mod, ":s") && !starts_with_lit(mod, ":gs"))
 		{
-			to_back_slash(buf);
+			internal_to_system_slashes(buf);
 		}
 	}
-#endif
 
 	return buf;
 }
