@@ -34,6 +34,7 @@
 #include "utils/str.h"
 #include "utils/string_array.h"
 #include "utils/trie.h"
+#include "utils/utils.h"
 #include "filelist.h"
 #include "flist_pos.h"
 #include "running.h"
@@ -101,7 +102,7 @@ save_selection(view_t *view)
 	entry = NULL;
 	while(iter_selected_entries(view, &entry))
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 
 		get_full_path_of(entry, sizeof(full_path), full_path);
 		view->saved_selection[i] = strdup(full_path);
@@ -178,7 +179,7 @@ flist_sel_restore(view_t *view, reg_t *reg)
 
 	for(i = 0; i < view->list_rows; ++i)
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		void *ignored_data;
 		dir_entry_t *const entry = &view->dir_entry[i];
 
@@ -280,10 +281,12 @@ flist_sel_by_filter(view_t *view, const char cmd[], int erase_old, int select)
 	selection_trie = trie_create();
 	for(i = 0; i < nfiles; ++i)
 	{
-		char canonic_path[PATH_MAX];
-		to_canonic_path(files[i], flist_get_dir(view), canonic_path,
-				sizeof(canonic_path));
-		(void)trie_put(selection_trie, canonic_path);
+		char *const path = parse_line_for_path(files[i], flist_get_dir(view));
+		if(path != NULL)
+		{
+			(void)trie_put(selection_trie, path);
+			free(path);
+		}
 	}
 	free_string_array(files, nfiles);
 
@@ -291,7 +294,7 @@ flist_sel_by_filter(view_t *view, const char cmd[], int erase_old, int select)
 	select = (select != 0);
 	for(i = 0; i < view->list_rows; ++i)
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		void *ignored_data;
 		dir_entry_t *const entry = &view->dir_entry[i];
 
@@ -339,7 +342,7 @@ flist_sel_by_pattern(view_t *view, const char pattern[], int erase_old,
 
 	for(i = 0; i < view->list_rows; ++i)
 	{
-		char file_path[PATH_MAX];
+		char file_path[PATH_MAX + 1];
 		dir_entry_t *const entry = &view->dir_entry[i];
 
 		if((entry->selected != 0) == select)
