@@ -65,7 +65,7 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 	int i;
 	char undo_msg[COMMAND_GROUP_INFO_LEN + 1];
 	dir_entry_t *entry;
-	char path[PATH_MAX];
+	char path[PATH_MAX + 1];
 	int from_file;
 	ops_t *ops;
 
@@ -113,16 +113,16 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 
 	nmarked_files = fops_enqueue_marked_files(ops, view, path, 0);
 
-	cmd_group_begin(undo_msg);
+	un_group_open(undo_msg);
 	i = 0;
 	entry = NULL;
 	custom_fnames = (nlines > 0);
 	while(iter_marked_entries(view, &entry) && !ui_cancellation_requested())
 	{
 		/* Must be at this level as dst might point into this buffer. */
-		char src_full[PATH_MAX];
+		char src_full[PATH_MAX + 1];
 
-		char dst_full[PATH_MAX];
+		char dst_full[PATH_MAX + 1];
 		const char *dst = custom_fnames ? list[i] : entry->name;
 		int err, from_trash;
 
@@ -157,7 +157,7 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 					ops);
 			if(err != 0)
 			{
-				view->list_pos = find_file_pos_in_list(view, entry->name);
+				view->list_pos = fpos_find_by_name(view, entry->name);
 			}
 		}
 		else
@@ -169,7 +169,7 @@ fops_cpmv(view_t *view, char *list[], int nlines, CopyMoveLikeOp op, int force)
 
 		++i;
 	}
-	cmd_group_end();
+	un_group_close();
 
 	ui_views_reload_filelists();
 	if(from_file)
@@ -190,8 +190,8 @@ fops_replace(view_t *view, const char dst[], int force)
 {
 	char undo_msg[COMMAND_GROUP_INFO_LEN + 1];
 	dir_entry_t *entry;
-	char dst_dir[PATH_MAX];
-	char src_full[PATH_MAX];
+	char dst_dir[PATH_MAX + 1];
+	char src_full[PATH_MAX + 1];
 	const char *const fname = get_last_path_component(dst);
 	ops_t *ops;
 	void *cp = (void *)(size_t)1;
@@ -213,7 +213,7 @@ fops_replace(view_t *view, const char dst[], int force)
 	snprintf(undo_msg, sizeof(undo_msg), "Copying %s to %s",
 			replace_home_part(src_full), dst_dir);
 
-	cmd_group_begin(undo_msg);
+	un_group_open(undo_msg);
 
 	ui_cancellation_reset();
 
@@ -228,7 +228,7 @@ fops_replace(view_t *view, const char dst[], int force)
 	if(!ui_cancellation_requested() && !is_valid_dir(dst_dir) &&
 			perform_operation(OP_MKDIR, NULL, cp, dst_dir, NULL) == 0)
 	{
-		add_operation(OP_MKDIR, cp, NULL, dst_dir, "");
+		un_group_add_op(OP_MKDIR, cp, NULL, dst_dir, "");
 	}
 
 	if(!ui_cancellation_requested())
@@ -237,7 +237,7 @@ fops_replace(view_t *view, const char dst[], int force)
 				1);
 	}
 
-	cmd_group_end();
+	un_group_close();
 
 	ui_views_reload_filelists();
 
@@ -250,7 +250,7 @@ static int
 cp_file(const char src_dir[], const char dst_dir[], const char src[],
 		const char dst[], CopyMoveLikeOp op, int cancellable, ops_t *ops, int force)
 {
-	char full_src[PATH_MAX], full_dst[PATH_MAX];
+	char full_src[PATH_MAX + 1], full_dst[PATH_MAX + 1];
 
 	to_canonic_path(src, src_dir, full_src, sizeof(full_src));
 	to_canonic_path(dst, dst_dir, full_dst, sizeof(full_dst));
@@ -425,7 +425,7 @@ cpmv_prepare(view_t *view, char ***list, int *nlines, CopyMoveLikeOp op,
 
 	if(op == CMLO_MOVE)
 	{
-		move_cursor_out_of(view, FLS_SELECTION);
+		fpos_move_out_of(view, FLS_SELECTION);
 	}
 
 	return 0;
@@ -465,7 +465,7 @@ check_for_clashes(view_t *view, CopyMoveLikeOp op, const char dst_path[],
 	int i = 0;
 	while(iter_marked_entries(view, &entry))
 	{
-		char src_full[PATH_MAX], dst_full[PATH_MAX];
+		char src_full[PATH_MAX + 1], dst_full[PATH_MAX + 1];
 		const char *const dst_name = (nlines > 0) ? list[i] : marked[i];
 		++i;
 
@@ -474,7 +474,7 @@ check_for_clashes(view_t *view, CopyMoveLikeOp op, const char dst_path[],
 		snprintf(dst_full, sizeof(dst_full), "%s/%s", dst_path, dst_name);
 		chosp(dst_full);
 
-		if(ONE_OF(op, CMLO_MOVE, CMLO_COPY) && is_in_subtree(dst_full, src_full))
+		if(ONE_OF(op, CMLO_MOVE, CMLO_COPY) && is_in_subtree(dst_full, src_full, 0))
 		{
 			ui_sb_errf("Can't move/copy parent inside itself: %s",
 					replace_home_part(src_full));
@@ -482,7 +482,7 @@ check_for_clashes(view_t *view, CopyMoveLikeOp op, const char dst_path[],
 			return 1;
 		}
 
-		if(is_in_subtree(src_full, dst_full))
+		if(is_in_subtree(src_full, dst_full, 0))
 		{
 			ui_sb_errf("Operation would result in loss contents of %s",
 					replace_home_part(src_full));
@@ -554,7 +554,7 @@ static void
 cpmv_file_in_bg(ops_t *ops, const char src[], const char dst[], int move,
 		int force, int from_trash, const char dst_dir[])
 {
-	char dst_full[PATH_MAX];
+	char dst_full[PATH_MAX + 1];
 	snprintf(dst_full, sizeof(dst_full), "%s/%s", dst_dir, dst);
 	if(force && path_exists(dst_full, DEREF) && !from_trash)
 	{
@@ -577,7 +577,7 @@ static int
 cp_file_f(const char src[], const char dst[], CopyMoveLikeOp op, int bg,
 		int cancellable, ops_t *ops, int force)
 {
-	char rel_path[PATH_MAX];
+	char rel_path[PATH_MAX + 1];
 
 	int file_op;
 	int result;
@@ -597,7 +597,7 @@ cp_file_f(const char src[], const char dst[], CopyMoveLikeOp op, int bg,
 
 		if(op == CMLO_LINK_REL)
 		{
-			char dst_dir[PATH_MAX];
+			char dst_dir[PATH_MAX + 1];
 
 			copy_str(dst_dir, sizeof(dst_dir), dst);
 			remove_last_path_component(dst_dir);
@@ -611,7 +611,7 @@ cp_file_f(const char src[], const char dst[], CopyMoveLikeOp op, int bg,
 			dst);
 	if(result == 0 && !bg)
 	{
-		add_operation(file_op, NULL, NULL, src, dst);
+		un_group_add_op(file_op, NULL, NULL, src, dst);
 	}
 	return result;
 }
