@@ -99,6 +99,31 @@ fops_delete(view_t *view, int reg, int use_trash)
 
 	use_trash = use_trash && cfg.use_trash;
 
+	strlist_t marked = {};
+	entry = NULL;
+	while(iter_marked_entries(view, &entry))
+	{
+		if(!flist_custom_active(view))
+		{
+			char name[NAME_MAX + 1];
+			format_entry_name(entry, NF_FULL, sizeof(name), name);
+			marked.nitems = add_to_string_array(&marked.items, marked.nitems, 1,
+					name);
+			continue;
+		}
+
+		char short_path[PATH_MAX + 1];
+		get_short_path_of(view, entry, NF_FULL, 0, sizeof(short_path), short_path);
+		marked.nitems = add_to_string_array(&marked.items, marked.nitems, 1,
+				short_path);
+	}
+	if(!confirm_deletion(marked.items, marked.nitems, use_trash))
+	{
+		free_string_array(marked.items, marked.nitems);
+		return 0;
+	}
+	free_string_array(marked.items, marked.nitems);
+
 	/* This check for the case when we are for sure in the trash. */
 	if(use_trash && top_dir != NULL && is_under_trash(top_dir))
 	{
@@ -115,7 +140,7 @@ fops_delete(view_t *view, int reg, int use_trash)
 	snprintf(undo_msg, sizeof(undo_msg), "%celete in %s: ", use_trash ? 'd' : 'D',
 			replace_home_part(curr_dir));
 	fops_append_marked_files(view, undo_msg, NULL);
-	cmd_group_begin(undo_msg);
+	un_group_open(undo_msg);
 
 	ops = fops_get_ops(OP_REMOVE, use_trash ? "deleting" : "Deleting", curr_dir,
 			curr_dir);
@@ -146,7 +171,7 @@ fops_delete(view_t *view, int reg, int use_trash)
 
 	regs_update_unnamed(reg);
 
-	cmd_group_end();
+	un_group_close();
 
 	ui_view_reset_selection_and_reload(view);
 	ui_view_schedule_reload(view == curr_view ? other_view : curr_view);
@@ -156,6 +181,7 @@ fops_delete(view_t *view, int reg, int use_trash)
 			fops_get_cancellation_suffix());
 
 	fops_free_ops(ops);
+	regs_sync_to_shared_memory();
 	return 1;
 }
 
@@ -182,7 +208,7 @@ fops_delete_current(view_t *view, int use_trash, int nested)
 			replace_home_part(curr_dir));
 	if(!nested)
 	{
-		cmd_group_begin(undo_msg);
+		un_group_open(undo_msg);
 	}
 
 	ops = fops_get_ops(OP_REMOVE, use_trash ? "deleting" : "Deleting", curr_dir,
@@ -197,7 +223,7 @@ fops_delete_current(view_t *view, int use_trash, int nested)
 
 	if(!nested)
 	{
-		cmd_group_end();
+		un_group_close();
 		ui_views_reload_filelists();
 	}
 
@@ -214,7 +240,7 @@ fops_delete_current(view_t *view, int use_trash, int nested)
 static int
 delete_file(dir_entry_t *entry, ops_t *ops, int reg, int use_trash, int nested)
 {
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 	int result;
 
 	get_full_path_of(entry, sizeof(full_path), full_path);
@@ -229,7 +255,7 @@ delete_file(dir_entry_t *entry, ops_t *ops, int reg, int use_trash, int nested)
 		}
 		if(result == 0)
 		{
-			add_operation(OP_REMOVE, NULL, NULL, full_path, "");
+			un_group_add_op(OP_REMOVE, NULL, NULL, full_path, "");
 		}
 	}
 	else if(is_trash_directory(full_path))
@@ -258,7 +284,7 @@ delete_file(dir_entry_t *entry, ops_t *ops, int reg, int use_trash, int nested)
 			}
 			if(result == 0)
 			{
-				add_operation(op, NULL, NULL, full_path, dest);
+				un_group_add_op(op, NULL, NULL, full_path, dest);
 				regs_append(reg, dest);
 			}
 			free(dest);
@@ -331,23 +357,13 @@ fops_delete_bg(view_t *view, int use_trash)
 		}
 	}
 
-	if(cfg_confirm_delete(use_trash))
+	if(!confirm_deletion(args->sel_list, args->sel_list_len, use_trash))
 	{
-		const char *const title = use_trash ? "Deletion" : "Permanent deletion";
-		char perm_del_msg[512];
-
-		snprintf(perm_del_msg, sizeof(perm_del_msg),
-				"Are you sure about removing %ld file%s?",
-				(long)args->sel_list_len, (args->sel_list_len == 1) ? "" : "s");
-
-		if(!prompt_msg(title, perm_del_msg))
-		{
-			fops_free_bg_args(args);
-			return 0;
-		}
+		fops_free_bg_args(args);
+		return 0;
 	}
 
-	move_cursor_out_of(view, FLS_MARKING);
+	fpos_move_out_of(view, FLS_MARKING);
 
 	snprintf(task_desc, sizeof(task_desc), "%celete in %s: ",
 			use_trash ? 'd' : 'D', replace_home_part(curr_dir));
@@ -448,7 +464,7 @@ fops_yank(view_t *view, int reg)
 	entry = NULL;
 	while(iter_marked_entries(view, &entry))
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full_path), full_path);
 
 		if(regs_append(reg, full_path) == 0)
@@ -462,6 +478,8 @@ fops_yank(view_t *view, int reg)
 	ui_sb_msgf("%d file%s yanked", nyanked_files,
 			(nyanked_files == 1) ? "" : "s");
 
+	regs_sync_to_shared_memory();
+
 	return 1;
 }
 
@@ -474,6 +492,9 @@ prepare_register(int reg)
 	if(reg >= 'A' && reg <= 'Z')
 	{
 		reg += 'a' - 'A';
+		/* We're going to append to a register and thus need its contents to be up
+		 * to date. */
+		regs_sync_from_shared_memory();
 	}
 	else
 	{
@@ -485,8 +506,8 @@ prepare_register(int reg)
 int
 fops_retarget(view_t *view)
 {
-	char full_path[PATH_MAX];
-	char linkto[PATH_MAX];
+	char full_path[PATH_MAX + 1];
+	char linkto[PATH_MAX + 1];
 	const dir_entry_t *const entry = get_current_entry(view);
 
 	if(!symlinks_available())
@@ -528,8 +549,8 @@ static void
 change_link_cb(const char new_target[])
 {
 	char undo_msg[COMMAND_GROUP_INFO_LEN];
-	char full_path[PATH_MAX];
-	char linkto[PATH_MAX];
+	char full_path[PATH_MAX + 1];
+	char linkto[PATH_MAX + 1];
 	const char *fname;
 	ops_t *ops;
 	const char *const curr_dir = flist_get_dir(curr_view);
@@ -553,18 +574,18 @@ change_link_cb(const char new_target[])
 	fname = get_last_path_component(full_path);
 	snprintf(undo_msg, sizeof(undo_msg), "cl in %s: on %s from \"%s\" to \"%s\"",
 			replace_home_part(flist_get_dir(curr_view)), fname, linkto, new_target);
-	cmd_group_begin(undo_msg);
+	un_group_open(undo_msg);
 
 	if(perform_operation(OP_REMOVESL, ops, NULL, full_path, NULL) == 0)
 	{
-		add_operation(OP_REMOVESL, NULL, NULL, full_path, linkto);
+		un_group_add_op(OP_REMOVESL, NULL, NULL, full_path, linkto);
 	}
 	if(perform_operation(OP_SYMLINK2, ops, NULL, new_target, full_path) == 0)
 	{
-		add_operation(OP_SYMLINK2, NULL, NULL, new_target, full_path);
+		un_group_add_op(OP_SYMLINK2, NULL, NULL, new_target, full_path);
 	}
 
-	cmd_group_end();
+	un_group_close();
 
 	fops_free_ops(ops);
 }
@@ -583,7 +604,7 @@ fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 {
 	int i;
 	char undo_msg[COMMAND_GROUP_INFO_LEN + 1];
-	char dst_path[PATH_MAX];
+	char dst_path[PATH_MAX + 1];
 	char **marked;
 	size_t nmarked;
 	int custom_fnames;
@@ -674,7 +695,7 @@ fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 
 	custom_fnames = (nlines > 0);
 
-	cmd_group_begin(undo_msg);
+	un_group_open(undo_msg);
 	entry = NULL;
 	i = 0;
 	while(iter_marked_entries(view, &entry) && !ui_cancellation_requested())
@@ -716,7 +737,7 @@ fops_clone(view_t *view, char *list[], int nlines, int force, int copies)
 
 		++i;
 	}
-	cmd_group_end();
+	un_group_close();
 
 	ui_views_reload_filelists();
 	if(from_file)
@@ -794,8 +815,8 @@ static int
 clone_file(const dir_entry_t *entry, const char path[], const char clone[],
 		ops_t *ops)
 {
-	char full_path[PATH_MAX];
-	char clone_name[PATH_MAX];
+	char full_path[PATH_MAX + 1];
+	char clone_name[PATH_MAX + 1];
 
 	to_canonic_path(clone, path, clone_name, sizeof(clone_name));
 	if(path_exists(clone_name, DEREF))
@@ -813,7 +834,7 @@ clone_file(const dir_entry_t *entry, const char path[], const char clone[],
 		return 1;
 	}
 
-	add_operation(OP_COPY, NULL, NULL, full_path, clone_name);
+	un_group_add_op(OP_COPY, NULL, NULL, full_path, clone_name);
 	return 0;
 }
 
@@ -835,7 +856,7 @@ fops_mkdirs(view_t *view, int at, char **names, int count, int create_parent)
 
 	for(i = 0; i < count; ++i)
 	{
-		char full[PATH_MAX];
+		char full[PATH_MAX + 1];
 
 		if(is_in_string_array(names, i, names[i]))
 		{
@@ -861,16 +882,16 @@ fops_mkdirs(view_t *view, int at, char **names, int count, int create_parent)
 	snprintf(buf, sizeof(buf), "mkdir in %s: ", replace_home_part(dst_dir));
 
 	get_group_file_list(names, count, buf);
-	cmd_group_begin(buf);
+	un_group_open(buf);
 	n = 0;
 	for(i = 0; i < count && !ui_cancellation_requested(); ++i)
 	{
-		char full[PATH_MAX];
+		char full[PATH_MAX + 1];
 		to_canonic_path(names[i], dst_dir, full, sizeof(full));
 
 		if(perform_operation(OP_MKDIR, NULL, cp, full, NULL) == 0)
 		{
-			add_operation(OP_MKDIR, cp, NULL, full, "");
+			un_group_add_op(OP_MKDIR, cp, NULL, full, "");
 			++n;
 		}
 		else if(i == 0)
@@ -880,7 +901,7 @@ fops_mkdirs(view_t *view, int at, char **names, int count, int create_parent)
 			--count;
 		}
 	}
-	cmd_group_end();
+	un_group_close();
 
 	if(count > 0)
 	{
@@ -915,7 +936,7 @@ fops_mkfiles(view_t *view, int at, char *names[], int count)
 
 	for(i = 0; i < count; ++i)
 	{
-		char full[PATH_MAX];
+		char full[PATH_MAX + 1];
 
 		if(is_in_string_array(names, i, names[i]))
 		{
@@ -943,20 +964,20 @@ fops_mkfiles(view_t *view, int at, char *names[], int count)
 	snprintf(buf, sizeof(buf), "touch in %s: ", replace_home_part(dst_dir));
 
 	get_group_file_list(names, count, buf);
-	cmd_group_begin(buf);
+	un_group_open(buf);
 	n = 0;
 	for(i = 0; i < count && !ui_cancellation_requested(); ++i)
 	{
-		char full[PATH_MAX];
+		char full[PATH_MAX + 1];
 		to_canonic_path(names[i], dst_dir, full, sizeof(full));
 
 		if(perform_operation(OP_MKFILE, ops, NULL, full, NULL) == 0)
 		{
-			add_operation(OP_MKFILE, NULL, NULL, full, "");
+			un_group_add_op(OP_MKFILE, NULL, NULL, full, "");
 			++n;
 		}
 	}
-	cmd_group_end();
+	un_group_close();
 
 	if(n > 0)
 	{
@@ -1019,19 +1040,19 @@ fops_restore(view_t *view)
 		return 0;
 	}
 
-	move_cursor_out_of(view, FLS_SELECTION);
+	fpos_move_out_of(view, FLS_SELECTION);
 
 	ui_cancellation_reset();
 
-	cmd_group_begin("restore: ");
-	cmd_group_end();
+	un_group_open("restore: ");
+	un_group_close();
 
 	m = 0;
 	n = 0;
 	entry = NULL;
 	while(iter_marked_entries(view, &entry) && !ui_cancellation_requested())
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full_path), full_path);
 
 		if(is_trash_directory(entry->origin) && restore_from_trash(full_path) == 0)
@@ -1073,7 +1094,7 @@ fops_size_bg(const view_t *view, int force)
 static void
 update_dir_entry_size(const view_t *view, int index, int force)
 {
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 	const dir_entry_t *const entry = &view->dir_entry[index];
 
 	if(fentry_is_fake(entry))
@@ -1242,18 +1263,18 @@ fops_chown(int u, int g, uid_t uid, gid_t gid)
 	(void)fops_enqueue_marked_files(ops, view, NULL, 0);
 
 	fops_append_marked_files(view, undo_msg, NULL);
-	cmd_group_begin(undo_msg);
+	un_group_open(undo_msg);
 
 	entry = NULL;
 	while(iter_marked_entries(view, &entry) && !ui_cancellation_requested())
 	{
 		int full_success = (u != 0) + (g != 0);
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full_path), full_path);
 
 		if(u && perform_operation(OP_CHOWN, ops, V(uid), full_path, NULL) == 0)
 		{
-			add_operation(OP_CHOWN, V(uid), V(entry->uid), full_path, "");
+			un_group_add_op(OP_CHOWN, V(uid), V(entry->uid), full_path, "");
 		}
 		else
 		{
@@ -1262,7 +1283,7 @@ fops_chown(int u, int g, uid_t uid, gid_t gid)
 
 		if(g && perform_operation(OP_CHGRP, ops, V(gid), full_path, NULL) == 0)
 		{
-			add_operation(OP_CHGRP, V(gid), V(entry->gid), full_path, "");
+			un_group_add_op(OP_CHGRP, V(gid), V(entry->gid), full_path, "");
 		}
 		else
 		{
@@ -1271,7 +1292,7 @@ fops_chown(int u, int g, uid_t uid, gid_t gid)
 
 		ops_advance(ops, full_success == (u != 0) + (g != 0));
 	}
-	cmd_group_end();
+	un_group_close();
 
 	ui_sb_msgf("%d file%s fully processed%s", ops->succeeded,
 			(ops->succeeded == 1) ? "" : "s", fops_get_cancellation_suffix());
