@@ -40,6 +40,7 @@
 #include "utils/utils.h"
 #include "background.h"
 #include "filelist.h"
+#include "flist_pos.h"
 #include "fops_common.h"
 #include "fops_cpmv.h"
 #include "ops.h"
@@ -56,6 +57,7 @@ static OPS cmlo_to_op(CopyMoveLikeOp op);
 static int path_depth_sort(const void *one, const void *two);
 static int is_dir_clash(const char src_path[], const char dst_dir[]);
 static int put_files_i(view_t *view, int start);
+static void update_cursor_position(view_t *view);
 static int put_next(int force);
 TSTATIC int merge_dirs(const char src[], const char dst[], ops_t *ops);
 static int handle_clashing(int move, const char src[], const char dst[]);
@@ -70,21 +72,23 @@ static void put_continue(int force);
  * the process. */
 static struct
 {
-	reg_t *reg;        /* Register used for the operation. */
-	int *file_order;   /* Defines custom ordering of files in register. */
-	view_t *view;      /* View in which operation takes place. */
-	CopyMoveLikeOp op; /* Type of current operation. */
-	int index;         /* Index of the next file of the register to process. */
-	int processed;     /* Number of successfully processed files. */
-	int skip_all;      /* Skip all conflicting files/directories. */
-	int overwrite_all; /* Overwrite all future conflicting files/directories. */
-	int append;        /* Whether we're appending ending of a file or not. */
-	int allow_merge;   /* Allow merging of files in directories. */
-	int merge;         /* Merge conflicting directory once. */
-	int merge_all;     /* Merge all conflicting directories. */
-	ops_t *ops;        /* Currently running operation. */
-	char *dst_name;    /* Name of destination file. */
-	char *dst_dir;     /* Destination path. */
+	reg_t *reg;          /* Register used for the operation. */
+	int *file_order;     /* Defines custom ordering of files in register. */
+	view_t *view;        /* View in which operation takes place. */
+	CopyMoveLikeOp op;   /* Type of current operation. */
+	int index;           /* Index of the next file of the register to process. */
+	int processed;       /* Number of successfully processed files. */
+	int skip_all;        /* Skip all conflicting files/directories. */
+	int overwrite_all;   /* Overwrite all future conflicting files/directories. */
+	int append;          /* Whether we're appending ending of a file or not. */
+	int allow_merge;     /* Allow merging of files in directories. */
+	int merge;           /* Merge conflicting directory once. */
+	int merge_all;       /* Merge all conflicting directories. */
+	ops_t *ops;          /* Currently running operation. */
+	char *dst_name;      /* Name of destination file. */
+	char *dst_dir;       /* Destination path. */
+	strlist_t put;       /* List of files that were successfully put. */
+	char *last_conflict; /* Path to the file of the last conflict. */
 }
 put_confirm;
 
@@ -113,6 +117,7 @@ fops_put_bg(view_t *view, int at, int reg_name, int move)
 		return 0;
 	}
 
+	regs_sync_from_shared_memory();
 	reg = regs_find(tolower(reg_name));
 	if(reg == NULL || reg->nfiles < 1)
 	{
@@ -268,6 +273,7 @@ initiate_put_files(view_t *view, int at, CopyMoveLikeOp op, const char descr[],
 		return 0;
 	}
 
+	regs_sync_from_shared_memory();
 	reg = regs_find(tolower(reg_name));
 	if(reg == NULL || reg->nfiles < 1)
 	{
@@ -293,16 +299,15 @@ initiate_put_files(view_t *view, int at, CopyMoveLikeOp op, const char descr[],
 	 * and then move all that will clash to the tail of the array. */
 	if(op == CMLO_MOVE || op == CMLO_COPY)
 	{
-		int i, nclashes;
-
-		qsort(put_confirm.file_order, reg->nfiles,
+		safe_qsort(put_confirm.file_order, reg->nfiles,
 				sizeof(put_confirm.file_order[0]), &path_depth_sort);
 
 		/* We basically do partition by "clash" predicate moving all files for which
 		 * it's true to the tail.  Mind that moved files end up in reverse order,
 		 * which is beneficial for us as we can move larger sub-tree and discard
 		 * some of other files. */
-		nclashes = 0;
+		int nclashes = 0;
+		int i;
 		for(i = 0; i < reg->nfiles - nclashes; ++i)
 		{
 			const int id = put_confirm.file_order[i];
@@ -339,6 +344,8 @@ reset_put_confirm(CopyMoveLikeOp main_op, const char descr[],
 	free(put_confirm.dst_name);
 	free(put_confirm.dst_dir);
 	free(put_confirm.file_order);
+	free_string_array(put_confirm.put.items, put_confirm.put.nitems);
+	free(put_confirm.last_conflict);
 
 	memset(&put_confirm, 0, sizeof(put_confirm));
 
@@ -376,7 +383,7 @@ path_depth_sort(const void *one, const void *two)
 	const char *s = put_confirm.reg->files[*(const int *)one];
 	const char *t = put_confirm.reg->files[*(const int *)two];
 
-	char s_real[PATH_MAX], t_real[PATH_MAX];
+	char s_real[PATH_MAX + 1], t_real[PATH_MAX + 1];
 
 	if(os_realpath(s, s_real) != s_real)
 	{
@@ -397,7 +404,7 @@ path_depth_sort(const void *one, const void *two)
 static int
 is_dir_clash(const char src_path[], const char dst_dir[])
 {
-	char dst_path[PATH_MAX];
+	char dst_path[PATH_MAX + 1];
 
 	snprintf(dst_path, sizeof(dst_path), "%s/%s", dst_dir,
 			fops_get_dst_name(src_path, is_under_trash(src_path)));
@@ -432,8 +439,8 @@ put_files_i(view_t *view, int start)
 
 		snprintf(undo_msg, sizeof(undo_msg), "%s in %s: ", descr,
 				replace_home_part(flist_get_dir(view)));
-		cmd_group_begin(undo_msg);
-		cmd_group_end();
+		un_group_open(undo_msg);
+		un_group_close();
 	}
 
 	if(vifm_chdir(put_confirm.dst_dir) != 0)
@@ -458,24 +465,69 @@ put_files_i(view_t *view, int start)
 		}
 		else if(put_result < 0)
 		{
-			ui_sb_msgf("%d file%s inserted%s", put_confirm.processed,
-					(put_confirm.processed == 1) ? "" : "s",
-					fops_get_cancellation_suffix());
-			return 1;
+			break;
 		}
 		++put_confirm.index;
 	}
 
-	regs_pack(put_confirm.reg->name);
+	/* Free ops here when we're done, because as a side-effect it also prints
+	 * error messages. */
+	fops_free_ops(put_confirm.ops);
+	put_confirm.ops = NULL;
 
+	regs_pack(put_confirm.reg->name);
+	update_cursor_position(view);
 	ui_sb_msgf("%d file%s inserted%s", put_confirm.processed,
 			(put_confirm.processed == 1) ? "" : "s", fops_get_cancellation_suffix());
 
-	ui_view_schedule_reload(put_confirm.view);
-
 	return 1;
 }
 
+/* Moves cursor to one of processed or conflicting files, if any. */
+static void
+update_cursor_position(view_t *view)
+{
+	if(put_confirm.last_conflict == NULL && put_confirm.put.nitems == 0)
+	{
+		// Apparently, nothing has changed.
+		return;
+	}
+
+	populate_dir_list(view, 1);
+	ui_view_schedule_redraw(view);
+
+	if(put_confirm.last_conflict != NULL)
+	{
+		dir_entry_t *const entry = entry_from_path(view, view->dir_entry,
+				view->list_rows, put_confirm.last_conflict);
+		if(entry != NULL)
+		{
+			fpos_set_pos(view, entry_to_pos(view, entry));
+		}
+		return;
+	}
+
+	int i;
+	int new_pos = -1;
+	for(i = 0; i < put_confirm.put.nitems; ++i)
+	{
+		dir_entry_t *const entry = entry_from_path(view, view->dir_entry,
+				view->list_rows, put_confirm.put.items[i]);
+		if(entry != NULL)
+		{
+			const int pos = entry_to_pos(view, entry);
+			if(new_pos == -1 || pos < new_pos)
+			{
+				new_pos = pos;
+			}
+		}
+	}
+	if(new_pos != -1)
+	{
+		fpos_set_pos(view, new_pos);
+	}
+}
+
 /* The force argument enables overwriting/replacing/merging.  Returns 0 on
  * success, otherwise non-zero is returned where value greater then zero means
  * error already reported to the user, while negative one means unreported
@@ -487,7 +539,7 @@ put_next(int force)
 	const char *dst_name;
 	const char *const dst_dir = put_confirm.dst_dir;
 	struct stat src_st;
-	char src_buf[PATH_MAX], dst_buf[PATH_MAX];
+	char src_buf[PATH_MAX + 1], dst_buf[PATH_MAX + 1];
 	int from_trash;
 	int op;
 	int move;
@@ -555,7 +607,7 @@ put_next(int force)
 						return 1;
 					}
 
-					if(is_in_subtree(src_buf, dst_buf))
+					if(is_in_subtree(src_buf, dst_buf, 0))
 					{
 						/* Don't delete /a/b before moving /a/b/c to /a/b. */
 						safe_operation = 1;
@@ -618,11 +670,11 @@ put_next(int force)
 	 * move operation itself. */
 	if(move && merge)
 	{
-		char dst_path[PATH_MAX];
+		char dst_path[PATH_MAX + 1];
 
 		success = 1;
 
-		cmd_group_continue();
+		un_group_reopen_last();
 
 		snprintf(dst_path, sizeof(dst_path), "%s/%s", dst_dir, dst_name);
 
@@ -631,7 +683,7 @@ put_next(int force)
 			success = 0;
 		}
 
-		cmd_group_end();
+		un_group_close();
 	}
 	else if(safe_operation)
 	{
@@ -676,9 +728,9 @@ put_next(int force)
 			return -1;
 		}
 
-		cmd_group_continue();
+		un_group_reopen_last();
 
-		msg = replace_group_msg(NULL);
+		msg = un_replace_group_msg(NULL);
 		len = strlen(msg);
 		p = realloc(msg, COMMAND_GROUP_INFO_LEN);
 		if(p == NULL)
@@ -688,15 +740,15 @@ put_next(int force)
 
 		snprintf(msg + len, COMMAND_GROUP_INFO_LEN - len, "%s%s",
 				(msg[len - 2] != ':') ? ", " : "", dst_name);
-		replace_group_msg(msg);
+		un_replace_group_msg(msg);
 		free(msg);
 
 		if(!(move && merge))
 		{
-			add_operation(op, NULL, NULL, src_buf, dst_buf);
+			un_group_add_op(op, NULL, NULL, src_buf, dst_buf);
 		}
 
-		cmd_group_end();
+		un_group_close();
 		++put_confirm.processed;
 		if(move)
 		{
@@ -704,6 +756,11 @@ put_next(int force)
 					&put_confirm.reg->files[put_confirm.file_order[put_confirm.index]],
 					NULL);
 		}
+
+		char dst_path[PATH_MAX + 1];
+		snprintf(dst_path, sizeof(dst_path), "%s/%s", dst_dir, dst_name);
+		put_confirm.put.nitems = add_to_string_array(&put_confirm.put.items,
+				put_confirm.put.nitems, 1, dst_path);
 	}
 
 	return 0;
@@ -737,8 +794,8 @@ merge_dirs(const char src[], const char dst[], ops_t *ops)
 
 	while((d = os_readdir(dir)) != NULL)
 	{
-		char src_path[PATH_MAX];
-		char dst_path[PATH_MAX];
+		char src_path[PATH_MAX + 1];
+		char dst_path[PATH_MAX + 1];
 
 		if(is_builtin_dir(d->d_name))
 		{
@@ -762,7 +819,7 @@ merge_dirs(const char src[], const char dst[], ops_t *ops)
 			{
 				break;
 			}
-			add_operation(OP_MOVEF, put_confirm.ops, NULL, src_path, dst_path);
+			un_group_add_op(OP_MOVEF, put_confirm.ops, NULL, src_path, dst_path);
 		}
 	}
 	os_closedir(dir);
@@ -775,12 +832,12 @@ merge_dirs(const char src[], const char dst[], ops_t *ops)
 	result = perform_operation(OP_RMDIR, put_confirm.ops, NULL, src, NULL);
 	if(result == 0)
 	{
-		add_operation(OP_RMDIR, NULL, NULL, src, "");
+		un_group_add_op(OP_RMDIR, NULL, NULL, src, "");
 	}
 
 	/* Clone file properties as the last step, because modifying directory affects
 	 * timestamps and permissions can affect operations. */
-	clone_timestamps(dst, src, &st);
+	clone_attribs(dst, src, &st);
 	(void)chmod(dst, st.st_mode);
 
 	return result;
@@ -800,8 +857,8 @@ handle_clashing(int move, const char src[], const char dst[])
 	{
 		const char *const another_src =
 			put_confirm.reg->files[put_confirm.file_order[i]];
-		const int sub_path = is_in_subtree(another_src, src);
-		if(is_in_subtree(another_src, dst) && !sub_path)
+		const int sub_path = is_in_subtree(another_src, src, 0);
+		if(is_in_subtree(another_src, dst, 0) && !sub_path)
 		{
 			vle_tb_append_line(lost, replace_home_part(another_src));
 		}
@@ -814,7 +871,7 @@ handle_clashing(int move, const char src[], const char dst[])
 	if(*vle_tb_get_data(lost) != '\0')
 	{
 		int i;
-		char msg[PATH_MAX];
+		char msg[PATH_MAX + 1];
 		response_variant responses[] = {
 			{ .key = 'y', .descr = "[y]es " },
 			{ .key = 'n', .descr = " [n]o\n" },
@@ -846,8 +903,8 @@ handle_clashing(int move, const char src[], const char dst[])
 		{
 			char **const another_src =
 				&put_confirm.reg->files[put_confirm.file_order[i]];
-			const int sub_path = is_in_subtree(*another_src, src);
-			if(is_in_subtree(*another_src, dst) && !sub_path)
+			const int sub_path = is_in_subtree(*another_src, src, 0);
+			if(is_in_subtree(*another_src, dst, 0) && !sub_path)
 			{
 				update_string(another_src, NULL);
 			}
@@ -866,7 +923,7 @@ handle_clashing(int move, const char src[], const char dst[])
 		{
 			char **const another_src =
 				&put_confirm.reg->files[put_confirm.file_order[i]];
-			if(is_in_subtree(*another_src, src))
+			if(is_in_subtree(*another_src, src, 0))
 			{
 				update_string(another_src, NULL);
 			}
@@ -890,7 +947,7 @@ prompt_what_to_do(const char fname[], const char caused_by[])
 		enter         = { .key = '\r', .descr = "" },
 		skip          = { .key = 's', .descr = "[s]kip " },
 		skip_all      = { .key = 'S', .descr = " [S]kip all          \n" },
-		append        = { .key = 'a', .descr = "[a]ppend to the end          \n" },
+		append        = { .key = 'a', .descr = "[a]ppend the tail            \n" },
 		overwrite     = { .key = 'o', .descr = "[o]verwrite " },
 		overwrite_all = { .key = 'O', .descr = " [O]verwrite all\n" },
 		merge         = { .key = 'm', .descr = "[m]erge " },
@@ -906,7 +963,8 @@ prompt_what_to_do(const char fname[], const char caused_by[])
 	responses[i++] = enter;
 	responses[i++] = skip;
 	responses[i++] = skip_all;
-	if(cfg.use_system_calls && !is_dir(fname))
+	if(cfg.use_system_calls && is_regular_file_noderef(fname) &&
+			is_regular_file_noderef(caused_by))
 	{
 		responses[i++] = append;
 	}
@@ -936,6 +994,12 @@ static void
 handle_prompt_response(const char fname[], const char caused_by[],
 		char response)
 {
+	char dst_path[PATH_MAX + 1];
+	snprintf(dst_path, sizeof(dst_path), "%s/%s", put_confirm.dst_dir, fname);
+
+	/* Record last conflict to position cursor at it later. */
+	update_string(&put_confirm.last_conflict, dst_path);
+
 	if(response == '\r' || response == 'r')
 	{
 		prompt_dst_name(fname);
@@ -974,7 +1038,18 @@ handle_prompt_response(const char fname[], const char caused_by[],
 		put_confirm.merge_all = 1;
 		put_continue(1);
 	}
-	else if(response != NC_C_c)
+	else if(response == NC_C_c)
+	{
+		/* After user cancels at conflict resolution, put the cursor at the last
+		 * file that caused the conflict. */
+		dir_entry_t *const entry = entry_from_path(put_confirm.view,
+				put_confirm.view->dir_entry, put_confirm.view->list_rows, dst_path);
+		if(entry != NULL)
+		{
+			fpos_set_pos(put_confirm.view, entry_to_pos(put_confirm.view, entry));
+		}
+	}
+	else
 	{
 		prompt_what_to_do(fname, caused_by);
 	}
@@ -999,6 +1074,11 @@ prompt_dst_name_cb(const char dst_name[])
 		return;
 	}
 
+	/* Record new destination path. */
+	char dst_path[PATH_MAX + 1];
+	snprintf(dst_path, sizeof(dst_path), "%s/%s", put_confirm.dst_dir, dst_name);
+	update_string(&put_confirm.last_conflict, dst_path);
+
 	if(replace_string(&put_confirm.dst_name, dst_name) != 0)
 	{
 		show_error_msg("Memory Error", "Unable to allocate enough memory");
