@@ -21,13 +21,17 @@
 
 #include <curses.h> /* werase() */
 
+#include <sys/types.h> /* pid_t ssize_t */
 #include <sys/stat.h> /* S_IRWXU */
-#include <unistd.h> /* rmdir() unlink() */
+#ifndef _WIN32
+#include <sys/wait.h> /* WEXITSTATUS() WIFEXITED() waitpid() */
+#endif
+#include <unistd.h> /* execve() fork() rmdir() unlink() */
 
 #include <errno.h> /* errno */
 #include <stddef.h> /* NULL size_t */
 #include <stdio.h> /* snprintf() fclose() */
-#include <stdlib.h> /* EXIT_SUCCESS WIFEXITED free() malloc() */
+#include <stdlib.h> /* EXIT_SUCCESS free() malloc() */
 #include <string.h> /* memmove() strcpy() strlen() strcmp() strcat() */
 
 #include "../cfg/config.h"
@@ -54,11 +58,11 @@
 /* Description of existing FUSE mounts. */
 typedef struct fuse_mount_t
 {
-	char source_file_path[PATH_MAX]; /* Full path to source file. */
-	char source_file_dir[PATH_MAX];  /* Full path to directory of source file. */
-	char mount_point[PATH_MAX];      /* Full path to mount point. */
-	int mount_point_id;              /* Identifier of mounts for unique dirs. */
-	struct fuse_mount_t *next;       /* Pointer to the next mount in chain. */
+	char source_file_path[PATH_MAX + 1]; /* Full path to source file. */
+	char source_file_dir[PATH_MAX + 1];  /* Full path to dir of source file. */
+	char mount_point[PATH_MAX + 1];      /* Full path to mount point. */
+	int mount_point_id;                  /* ID of mounts for unique dirs. */
+	struct fuse_mount_t *next;           /* Pointer to the next mount in chain. */
 }
 fuse_mount_t;
 
@@ -73,6 +77,8 @@ TSTATIC void format_mount_command(const char mount_point[],
 static fuse_mount_t * get_mount_by_source(const char source[]);
 static fuse_mount_t * get_mount_by_mount_point(const char dir[]);
 static fuse_mount_t * get_mount_by_path(const char path[]);
+static int run_fuse_command(char cmd[], const cancellation_t *cancellation,
+		int *cancelled);
 static void updir_from_mount(view_t *view, fuse_mount_t *runner);
 
 /* List of active mounts. */
@@ -84,8 +90,8 @@ fuse_try_mount(view_t *view, const char program[])
 	/* TODO: refactor this function fuse_try_mount() */
 
 	fuse_mount_t *runner;
-	char file_full_path[PATH_MAX];
-	char mount_point[PATH_MAX];
+	char file_full_path[PATH_MAX + 1];
+	char mount_point[PATH_MAX + 1];
 
 	if(make_path(cfg.fuse_home, S_IRWXU) != 0)
 	{
@@ -105,7 +111,7 @@ fuse_try_mount(view_t *view, const char program[])
 	}
 	else
 	{
-		char param[PATH_MAX];
+		char param[PATH_MAX + 1];
 		param[0] = '\0';
 
 		/* New file to be mounted. */
@@ -172,7 +178,7 @@ fuse_mount(view_t *view, char file_full_path[], const char param[],
 	int mount_point_id;
 	char buf[2*PATH_MAX];
 	int foreground;
-	char errors_file[PATH_MAX];
+	char errors_file[PATH_MAX + 1];
 	int status;
 	int cancelled;
 
@@ -230,13 +236,13 @@ fuse_mount(view_t *view, char file_full_path[], const char param[],
 	if(foreground)
 	{
 		cancelled = 0;
-		status = bg_and_wait_for_status(buf, &no_cancellation, NULL);
+		status = run_fuse_command(buf, &no_cancellation, NULL);
 	}
 	else
 	{
 		ui_cancellation_reset();
 		ui_cancellation_enable();
-		status = bg_and_wait_for_status(buf, &ui_cancellation_info, &cancelled);
+		status = run_fuse_command(buf, &ui_cancellation_info, &cancelled);
 		ui_cancellation_disable();
 	}
 
@@ -558,7 +564,7 @@ fuse_try_unmount(view_t *view)
 	}
 
 	ui_sb_msg("FUSE unmounting selected file, please stand by..");
-	status = bg_and_wait_for_status(buf, &no_cancellation, NULL);
+	status = run_fuse_command(buf, &no_cancellation, NULL);
 	ui_sb_clear();
 	/* check child status */
 	if(!WIFEXITED(status) || WEXITSTATUS(status))
@@ -586,6 +592,73 @@ fuse_try_unmount(view_t *view)
 	return 1;
 }
 
+/* Runs command in background not redirecting its streams.  To determine an
+ * error uses exit status only.  cancelled can be NULL when operations is not
+ * cancellable.  Returns status on success, otherwise -1 is returned.  Sets
+ * correct value of *cancelled even on error. */
+static int
+run_fuse_command(char cmd[], const cancellation_t *cancellation, int *cancelled)
+{
+#ifndef _WIN32
+	pid_t pid;
+	int status;
+
+	if(cancellation_possible(cancellation))
+	{
+		*cancelled = 0;
+	}
+
+	if(cmd == NULL)
+	{
+		return 1;
+	}
+
+	(void)set_sigchld(1);
+
+	pid = fork();
+	if(pid == (pid_t)-1)
+	{
+		(void)set_sigchld(0);
+		LOG_SERROR_MSG(errno, "Forking has failed.");
+		return -1;
+	}
+
+	if(pid == (pid_t)0)
+	{
+		extern char **environ;
+
+		(void)set_sigchld(0);
+
+		(void)execve(get_execv_path(cfg.shell), make_execv_array(cfg.shell, cmd),
+				environ);
+		_Exit(127);
+	}
+
+	while(waitpid(pid, &status, 0) == -1)
+	{
+		if(errno != EINTR)
+		{
+			LOG_SERROR_MSG(errno, "Failed waiting for process: %" PRINTF_ULL,
+					(unsigned long long)pid);
+			status = -1;
+			break;
+		}
+		process_cancel_request(pid, cancellation);
+	}
+
+	if(cancellation_requested(cancellation))
+	{
+		*cancelled = 1;
+	}
+
+	(void)set_sigchld(0);
+
+	return status;
+#else
+	return -1;
+#endif
+}
+
 static void
 updir_from_mount(view_t *view, fuse_mount_t *runner)
 {
@@ -599,8 +672,8 @@ updir_from_mount(view_t *view, fuse_mount_t *runner)
 
 	file = runner->source_file_path;
 	file += strlen(runner->source_file_dir) + 1;
-	pos = find_file_pos_in_list(view, file);
-	flist_set_pos(view, pos);
+	pos = fpos_find_by_name(view, file);
+	fpos_set_pos(view, pos);
 }
 
 int
