@@ -53,7 +53,7 @@ static VisitResult cp_mv_visitor(const char full_path[], VisitAction action,
 		void *param, int cp);
 
 int
-ior_rm(io_args_t *const args)
+ior_rm(io_args_t *args)
 {
 	const char *const path = args->arg1.path;
 	return traverse(path, &rm_visitor, args);
@@ -114,12 +114,12 @@ rm_visitor(const char full_path[], VisitAction action, void *param)
 }
 
 int
-ior_cp(io_args_t *const args)
+ior_cp(io_args_t *args)
 {
 	const char *const src = args->arg1.src;
 	const char *const dst = args->arg2.dst;
 
-	if(is_in_subtree(dst, src))
+	if(is_in_subtree(dst, src, 0))
 	{
 		(void)ioe_errlst_append(&args->result.errors, src, IO_ERR_UNKNOWN,
 				"Can't copy parent path into subpath");
@@ -162,7 +162,7 @@ cp_visitor(const char full_path[], VisitAction action, void *param)
 }
 
 int
-ior_mv(io_args_t *const args)
+ior_mv(io_args_t *args)
 {
 	const char *const src = args->arg1.src;
 	const char *const dst = args->arg2.dst;
@@ -428,7 +428,7 @@ cp_mv_visitor(const char full_path[], VisitAction action, void *param, int cp)
 						(void)ioe_errlst_append(&cp_args->result.errors, dst_full_path,
 								errno, "Failed to setup directory permissions");
 					}
-					clone_timestamps(dst_full_path, full_path, &st);
+					clone_attribs(dst_full_path, full_path, &st);
 				}
 				else
 				{
