@@ -50,7 +50,7 @@
 #include <errno.h> /* EACCES EEXIST EDQUOT ENOSPC ENXIO errno */
 #include <stddef.h> /* NULL size_t ssize_t */
 #include <stdio.h> /* FILE fclose() fdopen() fread() fwrite() */
-#include <stdlib.h> /* free() malloc() qsort() snprintf() */
+#include <stdlib.h> /* free() malloc() snprintf() */
 #include <string.h> /* strcmp() strcpy() strlen() */
 
 #include "utils/fs.h"
@@ -123,6 +123,8 @@ struct ipc_t
 	ipc_args_cb args_cb;
 	/* Stores callback used for evaluation of expressions. */
 	ipc_eval_cb eval_cb;
+	/* Whether this IPC instance should ignore check requests from outside. */
+	int locked;
 	/* Path to the pipe used by this instance. */
 	char pipe_path[PATH_MAX + 1];
 	/* Opened file of the pipe. */
@@ -178,6 +180,7 @@ ipc_init(const char name[], ipc_args_cb args_cb, ipc_eval_cb eval_cb)
 
 	ipc->args_cb = args_cb;
 	ipc->eval_cb = eval_cb;
+	ipc->locked = 0;
 
 	if(name == NULL)
 	{
@@ -221,7 +224,14 @@ int
 ipc_check(ipc_t *ipc)
 {
 	int len;
-	char *const pkg = receive_pkg(ipc, &len);
+	char *pkg;
+
+	if(ipc->locked)
+	{
+		return 0;
+	}
+
+	pkg = receive_pkg(ipc, &len);
 	if(pkg != NULL)
 	{
 		handle_pkg(ipc, pkg, pkg + len);
@@ -525,7 +535,9 @@ handle_args(ipc_t *ipc, char ***array, int len)
 
 	if(put_into_string_array(array, len, NULL) == len + 1)
 	{
+		ipc->locked = 1;
 		ipc->args_cb(*array);
+		ipc->locked = 0;
 	}
 }
 
@@ -541,7 +553,9 @@ handle_expr(ipc_t *ipc, const char from[], char *array[], int len)
 		return;
 	}
 
+	ipc->locked = 1;
 	result = ipc->eval_cb(array[0]);
+	ipc->locked = 0;
 	if(result == NULL)
 	{
 		char *data[] = { NULL };
@@ -665,7 +679,7 @@ static int
 send_pkg(const char whom[], const char what[], size_t len)
 {
 #ifndef WIN32_PIPE_READ
-	char path[PATH_MAX];
+	char path[PATH_MAX + 1];
 	int fd;
 	FILE *dst;
 	uint32_t size;
@@ -702,7 +716,7 @@ send_pkg(const char whom[], const char what[], size_t len)
 	}
 	return 0;
 #else
-	char path[PATH_MAX];
+	char path[PATH_MAX + 1];
 	HANDLE h;
 	uint32_t size;
 	DWORD nwritten;
@@ -773,7 +787,7 @@ list_servers(const ipc_t *ipc, int *len)
 	}
 #else
 	{
-		char find_pat[PATH_MAX];
+		char find_pat[PATH_MAX + 1];
 		HANDLE hfind;
 		WIN32_FIND_DATAA ffd;
 
@@ -798,10 +812,7 @@ list_servers(const ipc_t *ipc, int *len)
 	}
 #endif
 
-	if(data.len != 0)
-	{
-		qsort(data.lst, data.len, sizeof(*data.lst), &sorter);
-	}
+	safe_qsort(data.lst, data.len, sizeof(*data.lst), &sorter);
 
 	*len = data.len;
 	return data.lst;
@@ -830,7 +841,7 @@ add_to_list(const char name[], const void *data, void *param)
 	/* On Windows it's guaranteed to be a valid pipe. */
 #ifndef WIN32_PIPE_READ
 	{
-		char path[PATH_MAX];
+		char path[PATH_MAX + 1];
 		struct stat statbuf;
 		snprintf(path, sizeof(path), "%s/%s", list_data->ipc_dir, name);
 		if(stat(path, &statbuf) != 0 || !S_ISFIFO(statbuf.st_mode) ||
