@@ -20,10 +20,10 @@
 #ifndef VIFM__MENUS__COLORSCHEME_MENU_H__
 #define VIFM__MENUS__COLORSCHEME_MENU_H__
 
-#include "../ui/ui.h"
+struct view_t;
 
 /* Returns non-zero if status bar message should be saved. */
-int show_colorschemes_menu(FileView *view);
+int show_colorschemes_menu(struct view_t *view);
 
 #endif /* VIFM__MENUS__COLORSCHEME_MENU_H__ */
 
