@@ -325,7 +325,7 @@ menus_full_redraw(menu_state_t *m)
 
 	menus_partial_redraw(m);
 	menus_set_pos(m, m->d->pos);
-	wrefresh(menu_win);
+	ui_refresh_win(menu_win);
 }
 
 int
@@ -399,7 +399,7 @@ navigate_to_selected_file(view_t *view, const char path[])
 
 		load_dir_list(view, 0);
 
-		(void)ensure_file_is_selected(view, name);
+		(void)fpos_ensure_selected(view, name);
 	}
 	else
 	{
@@ -481,7 +481,7 @@ draw_menu_item(menu_state_t *ms, int pos, int line, int clear)
 
 	/* Clear the area. */
 	checked_wmove(menu_win, line, 1);
-	if(curr_stats.load_stage != 0)
+	if(curr_stats.load_stage > 0)
 	{
 		wprintw(menu_win, "%*s", width, "");
 	}
@@ -789,7 +789,7 @@ menus_to_custom_view(menu_state_t *m, view_t *view, int very)
 	 * to use file above cursor position. */
 	if(current == NULL && view->custom.entry_count != 0)
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		get_full_path_of(&view->custom.entries[view->custom.entry_count - 1],
 				sizeof(full_path), full_path);
 
@@ -863,7 +863,7 @@ menus_capture(view_t *view, const char cmd[], int user_sh, menu_data_t *m,
 void
 menus_search_repeat(menu_state_t *m, int backward)
 {
-	if(m->regexp == NULL)
+	if(is_null_or_empty(m->regexp))
 	{
 		ui_sb_err("No search pattern set");
 		curr_stats.save_msg = 1;
@@ -872,7 +872,7 @@ menus_search_repeat(menu_state_t *m, int backward)
 
 	m->backward_search = backward;
 	(void)menus_search(NULL, m->d, 1);
-	wrefresh(menu_win);
+	ui_refresh_win(menu_win);
 
 	if(m->matching_entries > 0)
 	{
