@@ -72,8 +72,6 @@
 #include "visual.h"
 #include "wk.h"
 
-#ifndef TEST
-
 /* History search mode. */
 typedef enum
 {
@@ -122,8 +120,6 @@ typedef struct
 }
 line_stats_t;
 
-#endif
-
 static int prev_mode;
 static CmdLineSubmode sub_mode;
 static line_stats_t input_stat;
@@ -383,6 +379,14 @@ update_cmdline_text(line_stats_t *stat)
 static void
 draw_cmdline_text(line_stats_t *stat)
 {
+	if(curr_stats.silent_ui)
+	{
+		/* XXX: erasing status_bar causes blinking if <silent> mapping that uses
+		 *      command-line is pressed and held down.  Ideally wouldn't need this
+		 *      retur. */
+		return;
+	}
+
 	int pair = -1;
 	int attr = 0;
 
@@ -410,7 +414,7 @@ draw_cmdline_text(line_stats_t *stat)
 	wattroff(status_bar, COLOR_PAIR(cfg.cs.pair[CMD_LINE_COLOR]) | attr);
 
 	update_cursor();
-	wrefresh(status_bar);
+	ui_refresh_win(status_bar);
 }
 
 /* Callback-like function, which is called every time input line is changed. */
@@ -449,6 +453,7 @@ input_line_changed(void)
 
 	if(prev_mode != MENU_MODE && prev_mode != VISUAL_MODE)
 	{
+		fpos_set_pos(curr_view, curr_view->list_pos);
 		redraw_current_view();
 	}
 	else if(prev_mode == MENU_MODE)
@@ -458,7 +463,7 @@ input_line_changed(void)
 
 	/* Hardware cursor is moved on the screen only on refresh, so refresh status
 	 * bar to force cursor moving there before it becomes visible again. */
-	wrefresh(status_bar);
+	ui_refresh_win(status_bar);
 	curs_set(1);
 }
 
@@ -466,17 +471,14 @@ input_line_changed(void)
 static void
 handle_empty_input(void)
 {
-	if(cfg.hl_search)
+	/* Clear selection/highlight. */
+	if(prev_mode == MENU_MODE)
 	{
-		/* Clear selection. */
-		if(prev_mode != MENU_MODE)
-		{
-			flist_sel_stash(curr_view);
-		}
-		else
-		{
-			(void)menus_search("", sub_mode_ptr, 0);
-		}
+		(void)menus_search("", sub_mode_ptr, 0);
+	}
+	else if(cfg.hl_search)
+	{
+		flist_sel_stash(curr_view);
 	}
 
 	if(prev_mode != MENU_MODE)
@@ -611,7 +613,7 @@ enter_cmdline_mode(CmdLineSubmode cl_sub_mode, const char cmd[], void *ptr)
 	if(sub_mode == CLS_COMMAND || sub_mode == CLS_MENU_COMMAND)
 	{
 		wprompt = L":";
-		complete_func = &complete_cmd;
+		complete_func = &vle_cmds_complete;
 	}
 	else if(sub_mode == CLS_FILTER)
 	{
@@ -779,6 +781,11 @@ set_view_port(void)
 		curr_view->top_line = input_stat.old_top;
 		curr_view->list_pos = input_stat.old_pos;
 	}
+	else if(sub_mode == CLS_FILTER)
+	{
+		/* Filtering itself doesn't update status line. */
+		ui_stat_update(curr_view, 1);
+	}
 
 	if(prev_mode == VISUAL_MODE)
 	{
@@ -880,7 +887,7 @@ cmd_ctrl_c(key_info_t key_info, keys_info_t *keys_info)
 		if(!input_stat.search_mode)
 		{
 			leave_visual_mode(curr_stats.save_msg, 1, 1);
-			flist_set_pos(curr_view, check_mark_directory(curr_view, '<'));
+			fpos_set_pos(curr_view, check_mark_directory(curr_view, '<'));
 		}
 	}
 	if(sub_mode == CLS_COMMAND)
@@ -1118,7 +1125,7 @@ draw_wild_menu(int op)
 	}
 	if(op == 0 && len < 2 && i - 1 == pos)
 		last_pos = i;
-	wrefresh(stat_win);
+	ui_refresh_win(stat_win);
 
 	update_cursor();
 }
@@ -1302,7 +1309,18 @@ cmd_return(key_info_t key_info, keys_info_t *keys_info)
 			real_start++;
 		}
 
+		if(sub_mode == CLS_COMMAND)
+		{
+			commands_scope_start();
+		}
 		curr_stats.save_msg = exec_commands(real_start, curr_view, cmd_type);
+		if(sub_mode == CLS_COMMAND)
+		{
+			if(commands_scope_finish() != 0)
+			{
+				curr_stats.save_msg = 1;
+			}
+		}
 	}
 	else if(sub_mode == CLS_PROMPT)
 	{
@@ -1902,9 +1920,9 @@ paste_str(const char str[], int allow_escaping)
 		str = escaped;
 	}
 
-	wide = to_wide(str);
+	wide = to_wide_force(str);
 
-	if(insert_str(wide) == 0)
+	if(insert_str((str[0] != '\0' && wide[0] == L'\0') ? L"?" : wide) == 0)
 	{
 		update_cmdline_text(&input_stat);
 	}
@@ -2072,7 +2090,7 @@ next_dot_completion(void)
 
 	if(input_stat.dot_pos <= curr_stats.cmd_hist.pos)
 	{
-		last = get_last_argument(curr_stats.cmd_hist.items[input_stat.dot_pos++], 1,
+		last = vle_cmds_last_arg(curr_stats.cmd_hist.items[input_stat.dot_pos++], 1,
 				&len);
 	}
 	else
@@ -2524,6 +2542,7 @@ line_completion(line_stats_t *stat)
 			{
 				case CLL_OUT_OF_ARG:
 				case CLL_NO_QUOTING:
+				case CLL_R_QUOTING:
 					vle_compl_set_add_path_hook(&escaped_arg_hook);
 					compl_func_arg = CPP_PERCENT_UNESCAPE;
 					break;
@@ -2537,10 +2556,6 @@ line_completion(line_stats_t *stat)
 					vle_compl_set_add_path_hook(&dquoted_arg_hook);
 					compl_func_arg = CPP_DQUOTES_UNESCAPE;
 					break;
-
-				case CLL_R_QUOTING:
-					assert(0 && "Unexpected completion inside regexp argument.");
-					break;
 			}
 		}
 
