@@ -98,10 +98,9 @@ init_modes(void)
 {
 	LOG_FUNC_ENTER;
 
+	vle_keys_init(MODES_COUNT, mode_flags, &stats_silence_ui);
+
 	int i;
-
-	vle_keys_init(MODES_COUNT, (int*)&mode_flags);
-
 	for(i = 0; i < MODES_COUNT; ++i)
 	{
 		mode_init_funcs[i]();
@@ -118,7 +117,7 @@ modes_pre(void)
 	else if(vle_mode_is(CMDLINE_MODE))
 	{
 		touchwin(status_bar);
-		wrefresh(status_bar);
+		ui_refresh_win(status_bar);
 	}
 	else if(vle_mode_is(VIEW_MODE))
 	{
@@ -131,7 +130,7 @@ modes_pre(void)
 	else if(!curr_stats.save_msg)
 	{
 		ui_sb_clear();
-		wrefresh(status_bar);
+		ui_refresh_win(status_bar);
 	}
 }
 
@@ -188,18 +187,11 @@ modes_statusbar_update(void)
 	{
 		/* Status bar is used for special purposes. */
 	}
-	else if(curr_stats.save_msg)
-	{
-		if(vle_mode_is(VISUAL_MODE))
-		{
-			update_vmode_input();
-		}
-	}
 	else if(curr_view->selected_files || vle_mode_is(VISUAL_MODE))
 	{
 		print_selected_msg();
 	}
-	else
+	else if(!curr_stats.save_msg)
 	{
 		ui_sb_clear();
 	}
@@ -379,7 +371,7 @@ update_vmode_input(void)
 		werase(input_win);
 		checked_wmove(input_win, 0, 0);
 		wprintw(input_win, "%d", curr_view->selected_files);
-		wrefresh(input_win);
+		ui_refresh_win(input_win);
 	}
 }
 
