@@ -46,6 +46,7 @@
 #include "../ui/fileview.h"
 #include "../ui/quickview.h"
 #include "../ui/statusbar.h"
+#include "../ui/tabs.h"
 #include "../ui/ui.h"
 #include "../utils/hist.h"
 #include "../utils/macros.h"
@@ -178,6 +179,8 @@ static void cmd_gh(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_gr(key_info_t key_info, keys_info_t *keys_info);
 #endif
 static void cmd_gs(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_gt(key_info_t key_info, keys_info_t *keys_info);
+static void cmd_gT(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_gU(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_gUgg(key_info_t key_info, keys_info_t *keys_info);
 static void cmd_gu(key_info_t key_info, keys_info_t *keys_info);
@@ -355,6 +358,8 @@ static keys_add_info_t builtin_cmds[] = {
 	{WK_g WK_k,        {{&cmd_k},  .descr = "go to item above"}},
 	{WK_g WK_l,        {{&cmd_return}, .descr = "open current item(s)"}},
 	{WK_g WK_s,        {{&cmd_gs}, .descr = "restore/make selection"}},
+	{WK_g WK_t,        {{&cmd_gt}, .descr = "next or n-th tab"}},
+	{WK_g WK_T,        {{&cmd_gT}, .descr = "n-th previous tab"}},
 	{WK_g WK_U,        {{&cmd_gU}, FOLLOWED_BY_SELECTOR, .descr = "convert to uppercase"}},
 	{WK_g WK_u,        {{&cmd_gu}, FOLLOWED_BY_SELECTOR, .descr = "convert to lowercase"}},
 	{WK_g WK_U WK_U,      {{&cmd_gU}, .descr = "convert to uppercase"}},
@@ -529,7 +534,7 @@ cmd_ctrl_d(key_info_t key_info, keys_info_t *keys_info)
 	if(fpos_can_move_down(curr_view))
 	{
 		curr_view->list_pos = fpos_half_scroll(curr_view, 1);
-		ui_view_schedule_redraw(curr_view);
+		redraw_current_view();
 	}
 }
 
@@ -537,8 +542,7 @@ cmd_ctrl_d(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_e(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(fpos_has_hidden_bottom(curr_view) &&
-			correct_list_pos_on_scroll_down(curr_view, 1))
+	if(fpos_has_hidden_bottom(curr_view) && fpos_scroll_down(curr_view, 1))
 	{
 		scroll_down(curr_view, 1);
 		redraw_current_view();
@@ -568,7 +572,11 @@ page_scroll(int base, int direction)
 	            + old_pos%curr_view->run_size - base%curr_view->run_size;
 	curr_view->list_pos = MAX(0, MIN(curr_view->list_rows - 1, new_pos));
 	scroll_by_files(curr_view, direction*offset);
-	redraw_current_view();
+
+	/* Updating list_pos ourselves doesn't take into account
+	 * synchronization/updates of the other view, so trigger them. */
+	ui_view_schedule_redraw(curr_view);
+	fpos_set_pos(curr_view, curr_view->list_pos);
 }
 
 static void
@@ -675,50 +683,45 @@ cmd_ctrl_o(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_r(key_info_t key_info, keys_info_t *keys_info)
 {
-	int ret;
-
 	curr_stats.confirmed = 0;
 	ui_cancellation_reset();
 
 	ui_sb_msg("Redoing...");
 
-	ret = redo_group();
+	switch(un_group_redo())
+	{
+		case UN_ERR_SUCCESS:
+			ui_views_reload_visible_filelists();
+			ui_sb_msg("Redone one group");
+			break;
+		case UN_ERR_FAIL:
+			ui_views_reload_visible_filelists();
+			ui_sb_err("Redone one group with errors");
+			break;
+		case UN_ERR_NONE:
+			ui_sb_msg("Nothing to redo");
+			break;
+		case UN_ERR_BROKEN:
+			ui_sb_err("Can't redo group, it was skipped");
+			break;
+		case UN_ERR_BALANCE:
+			ui_sb_err("Can't redo what wasn't undone");
+			break;
+		case UN_ERR_SKIPPED:
+			ui_sb_msg("Group redo skipped by user");
+			break;
+		case UN_ERR_CANCELLED:
+			ui_views_reload_visible_filelists();
+			ui_sb_msg("Redoing was cancelled");
+			break;
+		case UN_ERR_ERRORS:
+			ui_sb_err("Redo operation was skipped due to previous errors");
+			break;
+		case UN_ERR_NOUNDO:
+			assert(0 && "un_group_redo() shouldn't return UN_ERR_NOUNDO!");
+			break;
+	}
 
-	if(ret == 0)
-	{
-		ui_views_reload_visible_filelists();
-		ui_sb_msg("Redone one group");
-	}
-	else if(ret == -2)
-	{
-		ui_views_reload_visible_filelists();
-		ui_sb_err("Redone one group with errors");
-	}
-	else if(ret == -1)
-	{
-		ui_sb_msg("Nothing to redo");
-	}
-	else if(ret == -3)
-	{
-		ui_sb_err("Can't redo group, it was skipped");
-	}
-	else if(ret == -4)
-	{
-		ui_sb_err("Can't redo what wasn't undone");
-	}
-	else if(ret == -6)
-	{
-		ui_sb_msg("Group redo skipped by user");
-	}
-	else if(ret == -7)
-	{
-		ui_views_reload_visible_filelists();
-		ui_sb_msg("Redoing was cancelled");
-	}
-	else if(ret == 1)
-	{
-		ui_sb_err("Redo operation was skipped due to previous errors");
-	}
 	curr_stats.save_msg = 1;
 }
 
@@ -729,7 +732,7 @@ cmd_ctrl_u(key_info_t key_info, keys_info_t *keys_info)
 	if(fpos_can_move_up(curr_view))
 	{
 		curr_view->list_pos = fpos_half_scroll(curr_view, 0);
-		ui_view_schedule_redraw(curr_view);
+		redraw_current_view();
 	}
 }
 
@@ -923,8 +926,7 @@ cmd_ctrl_x(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_ctrl_y(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(fpos_has_hidden_top(curr_view) &&
-			correct_list_pos_on_scroll_up(curr_view, 1))
+	if(fpos_has_hidden_top(curr_view) && fpos_scroll_up(curr_view, 1))
 	{
 		scroll_up(curr_view, 1);
 		redraw_current_view();
@@ -993,13 +995,13 @@ find_goto(int ch, int count, int backward, keys_info_t *keys_info)
 	int pos;
 	int old_pos = curr_view->list_pos;
 
-	pos = flist_find_by_ch(curr_view, ch, backward, !keys_info->selector);
+	pos = fpos_find_by_ch(curr_view, ch, backward, !keys_info->selector);
 	if(pos < 0 || pos == curr_view->list_pos)
 		return;
 	while(--count > 0)
 	{
 		curr_view->list_pos = pos;
-		pos = flist_find_by_ch(curr_view, ch, backward, !keys_info->selector);
+		pos = fpos_find_by_ch(curr_view, ch, backward, !keys_info->selector);
 	}
 
 	if(keys_info->selector)
@@ -1010,7 +1012,7 @@ find_goto(int ch, int count, int backward, keys_info_t *keys_info)
 	}
 	else
 	{
-		flist_set_pos(curr_view, pos);
+		fpos_set_pos(curr_view, pos);
 	}
 }
 
@@ -1084,6 +1086,7 @@ cmd_gs(key_info_t key_info, keys_info_t *keys_info)
 		return;
 	}
 
+	regs_sync_from_shared_memory();
 	reg = regs_find(tolower(key_info.reg));
 	if(reg == NULL || reg->nfiles < 1)
 	{
@@ -1095,6 +1098,27 @@ cmd_gs(key_info_t key_info, keys_info_t *keys_info)
 	flist_sel_restore(curr_view, reg);
 }
 
+/* Switches either to the next tab or to tab specified by its number [count]. */
+static void
+cmd_gt(key_info_t key_info, keys_info_t *keys_info)
+{
+	if(key_info.count == NO_COUNT_GIVEN)
+	{
+		tabs_next(1);
+	}
+	else
+	{
+		tabs_goto(key_info.count - 1);
+	}
+}
+
+/* Switches to [count]-th previous tab ([count] is 1 by default). */
+static void
+cmd_gT(key_info_t key_info, keys_info_t *keys_info)
+{
+	tabs_previous(def_count(key_info.count));
+}
+
 /* Handles gU<selector>, gUgU and gUU normal mode commands, which convert file
  * name symbols to uppercase. */
 static void
@@ -1183,7 +1207,7 @@ pick_or_move(keys_info_t *keys_info, int new_pos)
 	}
 	else
 	{
-		flist_set_pos(curr_view, new_pos);
+		fpos_set_pos(curr_view, new_pos);
 	}
 }
 
@@ -1207,16 +1231,19 @@ cmd_V(key_info_t key_info, keys_info_t *keys_info)
 	enter_visual_mode(VS_NORMAL);
 }
 
+/* Possibly exits the application without saving vifminfo file or closes a
+ * tab. */
 static void
 cmd_ZQ(key_info_t key_info, keys_info_t *keys_info)
 {
-	vifm_try_leave(0, 0, 0);
+	ui_quit(0, 0);
 }
 
+/* Possibly exits the application saving vifminfo file or closes a tab. */
 static void
 cmd_ZZ(key_info_t key_info, keys_info_t *keys_info)
 {
-	vifm_try_leave(1, 0, 0);
+	ui_quit(1, 0);
 }
 
 /* Goto mark. */
@@ -1236,7 +1263,7 @@ cmd_quote(key_info_t key_info, keys_info_t *keys_info)
 		curr_stats.save_msg = goto_mark(curr_view, key_info.multi);
 		if(!cfg.auto_ch_pos)
 		{
-			flist_set_pos(curr_view, 0);
+			fpos_set_pos(curr_view, 0);
 		}
 	}
 }
@@ -1265,9 +1292,9 @@ sug_sel_quote(vle_keys_list_cb cb)
 static void
 cmd_dollar(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(!at_last_column(curr_view) || keys_info->selector)
+	if(!fpos_at_last_col(curr_view) || keys_info->selector)
 	{
-		pick_or_move(keys_info, get_end_of_line(curr_view));
+		pick_or_move(keys_info, fpos_line_end(curr_view));
 	}
 }
 
@@ -1315,9 +1342,9 @@ cmd_dot(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_zero(key_info_t key_info, keys_info_t *keys_info)
 {
-	if(!at_first_column(curr_view) || keys_info->selector)
+	if(!fpos_at_first_col(curr_view) || keys_info->selector)
 	{
-		pick_or_move(keys_info, get_start_of_line(curr_view));
+		pick_or_move(keys_info, fpos_line_start(curr_view));
 	}
 }
 
@@ -1341,16 +1368,13 @@ set_count_vars(int count)
 {
 	/* TODO: move this to a better place someday, nowhere to place right now. */
 
-	var_val_t value;
 	var_t var;
 
-	value.integer = (count == NO_COUNT_GIVEN ? 0 : count);
-	var = var_new(VTYPE_INT, value);
+	var = var_from_int(count == NO_COUNT_GIVEN ? 0 : count);
 	setvar("v:count", var);
 	var_free(var);
 
-	value.integer = (count == NO_COUNT_GIVEN ? 1 : count);
-	var = var_new(VTYPE_INT, value);
+	var = var_from_int(count == NO_COUNT_GIVEN ? 1 : count);
 	setvar("v:count1", var);
 	var_free(var);
 }
@@ -1539,17 +1563,10 @@ cmd_d_selector(key_info_t key_info, keys_info_t *keys_info)
 static void
 call_delete(key_info_t key_info, keys_info_t *keys_info, int use_trash)
 {
-	int count;
-
 	check_marking(curr_view, keys_info->count, keys_info->indexes);
-	count = flist_count_marked(curr_view);
-
-	if(count != 0 && confirm_deletion(count, use_trash))
-	{
-		curr_stats.save_msg = fops_delete(curr_view, def_reg(key_info.reg),
-				use_trash);
-		free_list_of_file_indexes(keys_info);
-	}
+	curr_stats.save_msg = (fops_delete(curr_view, def_reg(key_info.reg),
+			use_trash) != 0);
+	free_list_of_file_indexes(keys_info);
 }
 
 /* Applies change from other compare to current one. */
@@ -1842,54 +1859,45 @@ cmd_t(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_u(key_info_t key_info, keys_info_t *keys_info)
 {
-	int ret;
-
 	curr_stats.confirmed = 0;
 	ui_cancellation_reset();
 
 	ui_sb_msg("Undoing...");
 
-	ret = undo_group();
+	switch(un_group_undo())
+	{
+		case UN_ERR_SUCCESS:
+			ui_views_reload_visible_filelists();
+			ui_sb_msg("Undone one group");
+			break;
+		case UN_ERR_FAIL:
+			ui_views_reload_visible_filelists();
+			ui_sb_err("Undone one group with errors");
+			break;
+		case UN_ERR_NONE:
+			ui_sb_msg("Nothing to undo");
+			break;
+		case UN_ERR_BROKEN:
+			ui_sb_err("Can't undo group, it was skipped");
+			break;
+		case UN_ERR_BALANCE:
+			ui_sb_err("Can't undo what wasn't redone");
+			break;
+		case UN_ERR_NOUNDO:
+			ui_sb_err("Operation cannot be undone");
+			break;
+		case UN_ERR_SKIPPED:
+			ui_sb_msg("Group undo skipped by user");
+			break;
+		case UN_ERR_CANCELLED:
+			ui_views_reload_visible_filelists();
+			ui_sb_msg("Undoing was cancelled");
+			break;
+		case UN_ERR_ERRORS:
+			ui_sb_err("Undo operation was skipped due to previous errors");
+			break;
+	}
 
-	if(ret == 0)
-	{
-		ui_views_reload_visible_filelists();
-		ui_sb_msg("Undone one group");
-	}
-	else if(ret == -2)
-	{
-		ui_views_reload_visible_filelists();
-		ui_sb_err("Undone one group with errors");
-	}
-	else if(ret == -1)
-	{
-		ui_sb_msg("Nothing to undo");
-	}
-	else if(ret == -3)
-	{
-		ui_sb_err("Can't undo group, it was skipped");
-	}
-	else if(ret == -4)
-	{
-		ui_sb_err("Can't undo what wasn't redone");
-	}
-	else if(ret == -5)
-	{
-		ui_sb_err("Operation cannot be undone");
-	}
-	else if(ret == -6)
-	{
-		ui_sb_msg("Group undo skipped by user");
-	}
-	else if(ret == -7)
-	{
-		ui_views_reload_visible_filelists();
-		ui_sb_msg("Undoing was cancelled");
-	}
-	else if(ret == 1)
-	{
-		ui_sb_err("Undo operation was skipped due to previous errors");
-	}
 	curr_stats.save_msg = 1;
 }
 
@@ -2044,7 +2052,7 @@ cmd_zr(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_left_paren(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_find_group(curr_view, 0));
+	pick_or_move(keys_info, fpos_find_group(curr_view, 0));
 }
 
 /* Moves cursor to the beginning of the next group of files defined by the
@@ -2052,7 +2060,7 @@ cmd_left_paren(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_right_paren(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_find_group(curr_view, 1));
+	pick_or_move(keys_info, fpos_find_group(curr_view, 1));
 }
 
 /* Go to or pick files until and including previous sibling directory entry or
@@ -2060,7 +2068,7 @@ cmd_right_paren(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_z_k(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_prev_dir_sibling(curr_view));
+	pick_or_move(keys_info, fpos_prev_dir_sibling(curr_view));
 }
 
 /* Go to or pick files until and including next sibling directory entry or do
@@ -2068,7 +2076,7 @@ cmd_z_k(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_z_j(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_next_dir_sibling(curr_view));
+	pick_or_move(keys_info, fpos_next_dir_sibling(curr_view));
 }
 
 /* Go to or pick files until and including previous mismatched entry or do
@@ -2076,7 +2084,7 @@ cmd_z_j(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_lb_c(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_prev_mismatch(curr_view));
+	pick_or_move(keys_info, fpos_prev_mismatch(curr_view));
 }
 
 /* Go to or pick files until and including next mismatched entry or do
@@ -2084,7 +2092,7 @@ cmd_lb_c(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_rb_c(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_next_mismatch(curr_view));
+	pick_or_move(keys_info, fpos_next_mismatch(curr_view));
 }
 
 /* Go to or pick files until and including previous directory entry or do
@@ -2092,7 +2100,7 @@ cmd_rb_c(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_lb_d(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_prev_dir(curr_view));
+	pick_or_move(keys_info, fpos_prev_dir(curr_view));
 }
 
 /* Go to or pick files until and including next directory entry or do
@@ -2100,7 +2108,7 @@ cmd_lb_d(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_rb_d(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_next_dir(curr_view));
+	pick_or_move(keys_info, fpos_next_dir(curr_view));
 }
 
 /* Navigates to previous sibling directory. */
@@ -2140,14 +2148,14 @@ cmd_rb_R(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_lb_s(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_prev_selected(curr_view));
+	pick_or_move(keys_info, fpos_prev_selected(curr_view));
 }
 
 /* Go to or pick files until and including next selected entry or do nothing. */
 static void
 cmd_rb_s(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_next_selected(curr_view));
+	pick_or_move(keys_info, fpos_next_selected(curr_view));
 }
 
 /* Go to or pick files until and including first sibling of the current
@@ -2155,14 +2163,14 @@ cmd_rb_s(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_lb_z(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_first_sibling(curr_view));
+	pick_or_move(keys_info, fpos_first_sibling(curr_view));
 }
 
 /* Go to or pick files until and including last sibling of the current entry. */
 static void
 cmd_rb_z(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_last_sibling(curr_view));
+	pick_or_move(keys_info, fpos_last_sibling(curr_view));
 }
 
 /* Moves cursor to the beginning of the previous group of files defined by them
@@ -2170,7 +2178,7 @@ cmd_rb_z(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_left_curly_bracket(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_find_dir_group(curr_view, 0));
+	pick_or_move(keys_info, fpos_find_dir_group(curr_view, 0));
 }
 
 /* Moves cursor to the beginning of the next group of files defined by them
@@ -2178,7 +2186,7 @@ cmd_left_curly_bracket(key_info_t key_info, keys_info_t *keys_info)
 static void
 cmd_right_curly_bracket(key_info_t key_info, keys_info_t *keys_info)
 {
-	pick_or_move(keys_info, flist_find_dir_group(curr_view, 1));
+	pick_or_move(keys_info, fpos_find_dir_group(curr_view, 1));
 }
 
 /* Redraw with file in top of list. */
@@ -2233,7 +2241,7 @@ pick_files(view_t *view, int end, keys_info_t *keys_info)
 
 	if(end < view->list_pos)
 	{
-		flist_set_pos(view, end);
+		fpos_set_pos(view, end);
 	}
 }
 
