@@ -25,8 +25,6 @@
 #include "utils/utf8.h"
 #endif
 
-#include <curses.h> /* noraw() raw() */
-
 #include <sys/stat.h> /* gid_t uid_t */
 
 #include <assert.h> /* assert() */
@@ -71,11 +69,7 @@
 #define REFLINK_AUTO ""
 #endif
 
-#ifdef GNU_TOOLCHAIN
-#define PRESERVE_FLAGS "--preserve=mode,timestamps"
-#else
 #define PRESERVE_FLAGS "-p"
-#endif
 
 /* Types of conflict resolution actions to perform. */
 typedef enum
@@ -118,8 +112,8 @@ static int op_mkdir(ops_t *ops, void *data, const char *src, const char *dst);
 static int op_rmdir(ops_t *ops, void *data, const char *src, const char *dst);
 static int op_mkfile(ops_t *ops, void *data, const char *src, const char *dst);
 static int ops_uses_syscalls(const ops_t *ops);
-static int exec_io_op(ops_t *ops, int (*func)(io_args_t *const),
-		io_args_t *const args, int cancellable);
+static int exec_io_op(ops_t *ops, int (*func)(io_args_t *), io_args_t *args,
+		int cancellable);
 static int confirm_overwrite(io_args_t *args, const char src[],
 		const char dst[]);
 static char * pretty_dir_path(const char path[]);
@@ -286,12 +280,16 @@ op_none(ops_t *ops, void *data, const char *src, const char *dst)
 static int
 op_remove(ops_t *ops, void *data, const char *src, const char *dst)
 {
-	if(cfg_confirm_delete(0) && !curr_stats.confirmed &&
-			(ops == NULL || !ops->bg))
+	if((ops == NULL || !ops->bg) && cfg_confirm_delete(0) &&
+			!curr_stats.confirmed)
 	{
-		curr_stats.confirmed = prompt_msg("Permanent deletion",
-				"Are you sure?  If you're undoing a command and want to see file "
-				"names, use :undolist! command.");
+		char *msg = format_str("Are you sure?  "
+				"At least the following file is about to be deleted:\n \n%s\n \n"
+				"If you're undoing a command and want to see file names, use "
+				":undolist! command.",
+				replace_home_part(src));
+		curr_stats.confirmed = prompt_msg("Permanent deletion", msg);
+		free(msg);
 		if(!curr_stats.confirmed)
 			return SKIP_UNDO_REDO_OPERATION;
 	}
@@ -326,7 +324,7 @@ op_removesl(ops_t *ops, void *data, const char *src, const char *dst)
 #else
 		char cmd[PATH_MAX*2 + 1];
 		snprintf(cmd, sizeof(cmd), "%s \"%s\"", delete_prg, src);
-		to_back_slash(cmd);
+		internal_to_system_slashes(cmd);
 
 		return os_system(cmd);
 #endif
@@ -353,11 +351,11 @@ op_removesl(ops_t *ops, void *data, const char *src, const char *dst)
 #else
 		if(is_dir(src))
 		{
-			char path[PATH_MAX];
+			char path[PATH_MAX + 1];
 			int err;
 
 			copy_str(path, sizeof(path), src);
-			to_back_slash(path);
+			internal_to_system_slashes(path);
 
 			wchar_t *utf16_path = utf8_to_utf16(path);
 
@@ -478,7 +476,7 @@ op_cp(ops_t *ops, void *data, const char src[], const char dst[],
 		{
 			char cmd[6 + PATH_MAX*2 + 1];
 			snprintf(cmd, sizeof(cmd), "xcopy \"%s\" \"%s\" ", src, dst);
-			to_back_slash(cmd);
+			internal_to_system_slashes(cmd);
 
 			if(is_vista_and_above())
 				strcat(cmd, "/B ");
@@ -932,7 +930,7 @@ ops_uses_syscalls(const ops_t *ops)
 /* Executes i/o operation with some predefined pre/post actions.  Returns exit
  * code of i/o operation. */
 static int
-exec_io_op(ops_t *ops, int (*func)(io_args_t *const), io_args_t *const args,
+exec_io_op(ops_t *ops, int (*func)(io_args_t *), io_args_t *args,
 		int cancellable)
 {
 	int result;
@@ -1054,7 +1052,7 @@ static char *
 pretty_dir_path(const char path[])
 {
 	char dir_only[strlen(path) + 1];
-	char canonic[PATH_MAX];
+	char canonic[PATH_MAX + 1];
 
 	copy_str(dir_only, sizeof(dir_only), path);
 	remove_last_path_component(dir_only);
@@ -1119,19 +1117,11 @@ static char
 prompt_user(const io_args_t *args, const char title[], const char msg[],
 		const response_variant variants[])
 {
-	char response;
-
 	/* Active cancellation conflicts with input processing by putting terminal in
 	 * a cooked mode. */
-	if(args->cancellation.hook != NULL)
-	{
-		raw();
-	}
-	response = prompt_msg_custom(title, msg, variants);
-	if(args->cancellation.hook != NULL)
-	{
-		noraw();
-	}
+	const int cancellation_state = ui_cancellation_pause();
+	const char response = prompt_msg_custom(title, msg, variants);
+	ui_cancellation_resume(cancellation_state);
 
 	return response;
 }
