@@ -38,6 +38,7 @@
 #include "ui/fileview.h"
 #include "ui/quickview.h"
 #include "ui/statusbar.h"
+#include "ui/tabs.h"
 #include "ui/ui.h"
 #include "utils/darray.h"
 #include "utils/log.h"
@@ -49,6 +50,7 @@
 #include "utils/utils.h"
 #include "filelist.h"
 #include "flist_hist.h"
+#include "registers.h"
 #include "search.h"
 #include "sort.h"
 #include "status.h"
@@ -86,13 +88,13 @@ static char * double_commas(const char str[]);
 static void init_cpoptions(optval_t *val);
 static void init_dirsize(optval_t *val);
 static const char * to_endpoint(int i, char buffer[]);
-static void init_timefmt(optval_t *val);
 static void init_trashdir(optval_t *val);
 static void init_dotfiles(optval_t *val);
 static void init_lsoptions(optval_t *val);
 static void init_lsview(optval_t *val);
 static void init_milleroptions(optval_t *val);
 static void init_millerview(optval_t *val);
+static void init_quickview(optval_t *val);
 static void init_shortmess(optval_t *val);
 static void init_sizefmt(optval_t *val);
 static void init_iooptions(optval_t *val);
@@ -132,6 +134,7 @@ static void followlinks_handler(OPT_OP op, optval_t val);
 static void fusehome_handler(OPT_OP op, optval_t val);
 static void gdefault_handler(OPT_OP op, optval_t val);
 static void grepprg_handler(OPT_OP op, optval_t val);
+static void histcursor_handler(OPT_OP op, optval_t val);
 static void history_handler(OPT_OP op, optval_t val);
 static void hlsearch_handler(OPT_OP op, optval_t val);
 static void iec_handler(OPT_OP op, optval_t val);
@@ -141,14 +144,19 @@ static void iooptions_handler(OPT_OP op, optval_t val);
 static void laststatus_handler(OPT_OP op, optval_t val);
 static void lines_handler(OPT_OP op, optval_t val);
 static void locateprg_handler(OPT_OP op, optval_t val);
+#ifndef _WIN32
+static void mediaprg_handler(OPT_OP op, optval_t val);
+#endif
 static void mintimeoutlen_handler(OPT_OP op, optval_t val);
 static void scroll_line_down(view_t *view);
+static void quickview_handler(OPT_OP op, optval_t val);
 static void rulerformat_handler(OPT_OP op, optval_t val);
 static void runexec_handler(OPT_OP op, optval_t val);
 static void scrollbind_handler(OPT_OP op, optval_t val);
 static void scrolloff_handler(OPT_OP op, optval_t val);
 static void shell_handler(OPT_OP op, optval_t val);
 static void shortmess_handler(OPT_OP op, optval_t val);
+static void showtabline_handler(OPT_OP op, optval_t val);
 static void sizefmt_handler(OPT_OP op, optval_t val);
 static optval_t make_sizefmt_value(void);
 #ifndef _WIN32
@@ -201,9 +209,10 @@ static int map_name(const char name[], void *arg);
 static void resort_view(view_t * view);
 static void statusline_handler(OPT_OP op, optval_t val);
 static void suggestoptions_handler(OPT_OP op, optval_t val);
-static int read_int(const char line[], int *i);
 static void reset_suggestoptions(void);
+static void syncregs_handler(OPT_OP op, optval_t val);
 static void syscalls_handler(OPT_OP op, optval_t val);
+static void tabscope_handler(OPT_OP op, optval_t val);
 static void tabstop_handler(OPT_OP op, optval_t val);
 static void timefmt_handler(OPT_OP op, optval_t val);
 static void timeoutlen_handler(OPT_OP op, optval_t val);
@@ -292,11 +301,21 @@ static const char *confirm_vals[][2] = {
 
 /* Possible values of 'dotdirs'. */
 static const char *dotdirs_vals[][2] = {
-	{ "rootparent",    "show .. in FS root" },
-	{ "nonrootparent", "show .. in non-root directories" },
+	[BIT(DD_ROOT_PARENT)]    = { "rootparent",
+	                             "show .. in FS root" },
+	[BIT(DD_NONROOT_PARENT)] = { "nonrootparent",
+	                             "show .. in non-root directories" },
 };
 ARRAY_GUARD(dotdirs_vals, NUM_DOT_DIRS);
 
+/* Possible values of 'histcursor'. */
+static const char *histcursor_vals[][2] = {
+	[BIT(CHPOS_STARTUP)] = { "startup",  "initial view loading on startup" },
+	[BIT(CHPOS_DIRMARK)] = { "dirmark",  "after navigating to a mark w/o file" },
+	[BIT(CHPOS_ENTER)] =   { "direnter", "after picking directory in file list" },
+};
+ARRAY_GUARD(histcursor_vals, NUM_CHPOS);
+
 /* Possible keys of 'lsoptions' option. */
 static const char *lsoptions_enum[][2] = {
 	{ "transposed", "fill grid by column instead of by line" },
@@ -304,15 +323,21 @@ static const char *lsoptions_enum[][2] = {
 
 /* Possible values of 'suggestoptions'. */
 static const char *suggestoptions_vals[][2] = {
-	{ "normal",      "display in normal mode" },
-	{ "visual",      "display in visual mode" },
-	{ "view",        "display in view mode" },
-	{ "otherpane",   "use other pane for suggestions, if available" },
-	{ "delay",       "display suggestions after a short delay" },
-	{ "keys",        "include keys suggestions in results" },
-	{ "foldsubkeys", "fold multiple keys with common prefix" },
-	{ "marks",       "include marks suggestions in results" },
-	{ "registers",   "include registers suggestions in results" },
+	[BIT(SF_NORMAL)]      = { "normal",      "display in normal mode" },
+	[BIT(SF_VISUAL)]      = { "visual",      "display in visual mode" },
+	[BIT(SF_VIEW)]        = { "view",        "display in view mode" },
+	[BIT(SF_OTHERPANE)]   = { "otherpane",
+	                          "use other pane for suggestions, if available" },
+	[BIT(SF_DELAY)]       = { "delay",
+	                          "display suggestions after a short delay" },
+	[BIT(SF_KEYS)]        = { "keys",
+	                          "include keys suggestions in results" },
+	[BIT(SF_MARKS)]       = { "foldsubkeys",
+	                          "fold multiple keys with common prefix" },
+	[BIT(SF_REGISTERS)]   = { "marks",
+	                          "include marks suggestions in results" },
+	[BIT(SF_FOLDSUBKEYS)] = { "registers",
+	                          "include registers suggestions in results" },
 };
 ARRAY_GUARD(suggestoptions_vals, NUM_SUGGESTION_FLAGS);
 
@@ -323,9 +348,26 @@ static const char *iooptions_vals[][2] = {
 
 /* Possible flags of 'shortmess' and their count. */
 static const char *shortmess_vals[][2] = {
-	{ "Tp", "all shortmess values" },
-	{ "T",  "shorten too long status bar messages" },
-	{ "p",  "substitute home path with ~ in view title" },
+	{ "MTp", "all shortmess values" },
+	{ "M",   "use only file name for title in terminal multiplexers" },
+	{ "T",   "shorten too long status bar messages" },
+	{ "p",   "substitute home path with ~ in view title" },
+};
+
+/* Possible values of 'showtabline'. */
+static const char *showtabline_vals[][2] = {
+	{ "never",    "tab line is never displayed" },
+	{ "multiple", "tab line is visible when there are at least two tabs" },
+	{ "always",   "tab line is displayed always" },
+	{ "0",        "tab line is never displayed" },
+	{ "1",        "tab line is visible when there are at least two tabs" },
+	{ "2",        "tab line is displayed always" },
+};
+
+/* Possible values of 'tabscope'. */
+static const char *tabscope_vals[][2] = {
+	{ "global", "single tab contains panes and their layout" },
+	{ "pane",   "tab contains state of just one pane" },
 };
 
 /* Possible flags of 'tuioptions' and their count. */
@@ -464,23 +506,24 @@ static const char *sortorder_enum[][2] = {
 
 /* Possible values of 'vifminfo' option. */
 static const char *vifminfo_set[][2] = {
-	{ "options",   "values of options" },
-	{ "filetypes", "file associations" },
-	{ "commands",  "user-defined :commands" },
-	{ "bookmarks", "marks (e.g. 'a)" },
-	{ "bmarks",    "bookmarks" },
-	{ "tui",       "state of TUI" },
-	{ "dhistory",  "directory history" },
-	{ "state",     "filters and terminal multiplexer" },
-	{ "cs",        "current colorscheme" },
-	{ "savedirs",  "restore last visited directory" },
-	{ "chistory",  "cmdline history" },
-	{ "shistory",  "search history" },
-	{ "dirstack",  "directory stack" },
-	{ "registers", "contents of registers" },
-	{ "phistory",  "prompt history" },
-	{ "fhistory",  "local filter history" },
+	[BIT(VINFO_OPTIONS)]   = { "options",   "values of options" },
+	[BIT(VINFO_FILETYPES)] = { "filetypes", "file associations" },
+	[BIT(VINFO_COMMANDS)]  = { "commands",  "user-defined :commands" },
+	[BIT(VINFO_MARKS)]     = { "bookmarks", "marks (e.g. 'a)" },
+	[BIT(VINFO_BOOKMARKS)] = { "bmarks",    "bookmarks" },
+	[BIT(VINFO_TUI)]       = { "tui",       "state of TUI" },
+	[BIT(VINFO_DHISTORY)]  = { "dhistory",  "directory history" },
+	[BIT(VINFO_STATE)]     = { "state",     "filters and terminal multiplexer" },
+	[BIT(VINFO_CS)]        = { "cs",        "current colorscheme" },
+	[BIT(VINFO_SAVEDIRS)]  = { "savedirs",  "restore last visited directory" },
+	[BIT(VINFO_CHISTORY)]  = { "chistory",  "cmdline history" },
+	[BIT(VINFO_SHISTORY)]  = { "shistory",  "search history" },
+	[BIT(VINFO_DIRSTACK)]  = { "dirstack",  "directory stack" },
+	[BIT(VINFO_REGISTERS)] = { "registers", "contents of registers" },
+	[BIT(VINFO_PHISTORY)]  = { "phistory",  "prompt history" },
+	[BIT(VINFO_FHISTORY)]  = { "fhistory",  "local filter history" },
 };
+ARRAY_GUARD(vifminfo_set, NUM_VINFO);
 
 /* Possible values of 'wildstyle'. */
 static const char *wildstyle_vals[][2] = {
@@ -593,6 +636,11 @@ options[] = {
 	  OPT_STR, 0, NULL, &grepprg_handler, NULL,
 	  { .ref.str_val = &cfg.grep_prg },
 	},
+	{ "histcursor", "", "when to use historical cursor position",
+	  OPT_SET, ARRAY_LEN(histcursor_vals), histcursor_vals,
+		&histcursor_handler, NULL,
+	  { .ref.set_items = &cfg.ch_pos_on },
+	},
 	{ "history", "hi", "length of all histories",
 	  OPT_INT, 0, NULL, &history_handler, NULL,
 	  { .ref.int_val = &cfg.history_len },
@@ -630,10 +678,20 @@ options[] = {
 	  OPT_STR, 0, NULL, &locateprg_handler, NULL,
 	  { .ref.str_val = &cfg.locate_prg },
 	},
+#ifndef _WIN32
+	{ "mediaprg", "", "helper for :media menu",
+	  OPT_STR, 0, NULL, &mediaprg_handler, NULL,
+	  { .ref.str_val = &cfg.media_prg },
+	},
+#endif
 	{ "mintimeoutlen", "", "delay between input polls",
 	  OPT_INT, 0, NULL, &mintimeoutlen_handler, NULL,
 	  { .ref.int_val = &cfg.min_timeout_len },
 	},
+	{ "quickview", "", "whether quick view is active",
+	  OPT_BOOL, 0, NULL, &quickview_handler, NULL,
+	  { .init = &init_quickview },
+	},
 	{ "rulerformat", "ruf", "format of the ruler",
 	  OPT_STR, 0, NULL, &rulerformat_handler, NULL,
 	  { .ref.str_val = &cfg.ruler_format },
@@ -659,6 +717,11 @@ options[] = {
 		NULL,
 	  { .init = &init_shortmess },
 	},
+	{ "showtabline", "stal", "when to display tab line",
+	  OPT_ENUM, ARRAY_LEN(showtabline_vals), showtabline_vals,
+		&showtabline_handler, NULL,
+	  { .ref.enum_item = &cfg.show_tab_line },
+	},
 	{ "sizefmt", "", "human-friendly size format",
 	  OPT_STRLIST, ARRAY_LEN(sizefmt_enum), sizefmt_enum, &sizefmt_handler, NULL,
 	  { .init = &init_sizefmt },
@@ -686,17 +749,24 @@ options[] = {
 		&suggestoptions_handler, NULL,
 	  { .ref.str_val = &empty },
 	},
+	{ "syncregs", "", "defines group of instances that share registers",
+	  OPT_STR, 0, NULL, &syncregs_handler, NULL,
+	  { .ref.str_val = &empty } },
 	{ "syscalls", "", "use system calls for file operations",
 	  OPT_BOOL, 0, NULL, &syscalls_handler, NULL,
 	  { .ref.bool_val = &cfg.use_system_calls },
 	},
+	{ "tabscope", "", "level at which tabs operate",
+	  OPT_ENUM, ARRAY_LEN(tabscope_vals), tabscope_vals, &tabscope_handler, NULL,
+	  { .ref.bool_val = &cfg.pane_tabs },
+	},
 	{ "tabstop", "ts", "widths of one tabulation",
 	  OPT_INT, 0, NULL, &tabstop_handler, NULL,
 	  { .ref.int_val = &cfg.tab_stop },
 	},
 	{ "timefmt", "", "time format",
 	  OPT_STR, 0, NULL, &timefmt_handler, NULL,
-	  { .init = &init_timefmt },
+	  { .ref.str_val = &cfg.time_format },
 	},
 	{ "timeoutlen", "tm", "delay on ambiguous key sequence",
 	  OPT_INT, 0, NULL, &timeoutlen_handler, NULL,
@@ -1011,12 +1081,6 @@ to_endpoint(int i, char buffer[])
 	return buffer;
 }
 
-static void
-init_timefmt(optval_t *val)
-{
-	val->str_val = cfg.time_format + 1;
-}
-
 static void
 init_trashdir(optval_t *val)
 {
@@ -1058,13 +1122,20 @@ init_millerview(optval_t *val)
 	val->bool_val = curr_view->miller_view_g;
 }
 
+/* Initializes 'quickview' option from global state. */
+static void
+init_quickview(optval_t *val)
+{
+	val->bool_val = curr_stats.preview.on;
+}
+
 /* Initializes 'shortmess' from current configuration state. */
 static void
 init_shortmess(optval_t *val)
 {
 	static char buf[32];
-	snprintf(buf, sizeof(buf), "%s%s", cfg.trunc_normal_sb_msgs ? "T" : "",
-			cfg.shorten_title_paths ? "p" : "");
+	snprintf(buf, sizeof(buf), "%s%s%s", cfg.short_term_mux_titles ? "M" : "",
+			cfg.trunc_normal_sb_msgs ? "T" : "", cfg.shorten_title_paths ? "p" : "");
 	val->str_val = buf;
 }
 
@@ -1984,6 +2055,13 @@ grepprg_handler(OPT_OP op, optval_t val)
 	(void)replace_string(&cfg.grep_prg, val.str_val);
 }
 
+/* Handles changes of 'histcursor'.  Updates related configuration value. */
+static void
+histcursor_handler(OPT_OP op, optval_t val)
+{
+	cfg.ch_pos_on = val.set_items;
+}
+
 static void
 history_handler(OPT_OP op, optval_t val)
 {
@@ -2079,12 +2157,24 @@ lines_handler(OPT_OP op, optval_t val)
 	set_option("lines", val, OPT_GLOBAL);
 }
 
+/* Handles updates of the 'locateprg' option. */
 static void
 locateprg_handler(OPT_OP op, optval_t val)
 {
 	(void)replace_string(&cfg.locate_prg, val.str_val);
 }
 
+#ifndef _WIN32
+
+/* Handles updates of the 'mediaprg' option. */
+static void
+mediaprg_handler(OPT_OP op, optval_t val)
+{
+	(void)replace_string(&cfg.media_prg, val.str_val);
+}
+
+#endif
+
 /* Minimum period on waiting for the input.  Works together with timeoutlen. */
 static void
 mintimeoutlen_handler(OPT_OP op, optval_t val)
@@ -2114,6 +2204,14 @@ scroll_line_down(view_t *view)
 	wresize(view->win, view->window_rows, view->window_cols);
 }
 
+/* Handles switch that controls visibility of quick view. */
+static void
+quickview_handler(OPT_OP op, optval_t val)
+{
+	curr_stats.preview.on = val.bool_val;
+	stats_redraw_schedule();
+}
+
 static void
 rulerformat_handler(OPT_OP op, optval_t val)
 {
@@ -2166,11 +2264,16 @@ shortmess_handler(OPT_OP op, optval_t val)
 
 	cfg.trunc_normal_sb_msgs = 0;
 	cfg.shorten_title_paths = 0;
+	cfg.short_term_mux_titles = 0;
 
 	p = val.str_val;
 	while(*p != '\0')
 	{
-		if(*p == 'T')
+		if(*p == 'M')
+		{
+			cfg.short_term_mux_titles = 1;
+		}
+		else if(*p == 'T')
 		{
 			cfg.trunc_normal_sb_msgs = 1;
 		}
@@ -2184,6 +2287,15 @@ shortmess_handler(OPT_OP op, optval_t val)
 	ui_views_update_titles();
 }
 
+/* Handles changes of 'showtabline' option that controls visibility of tab
+ * line. */
+static void
+showtabline_handler(OPT_OP op, optval_t val)
+{
+	cfg.show_tab_line = (val.enum_item > 2 ? val.enum_item - 3 : val.enum_item);
+	stats_redraw_schedule();
+}
+
 /* Handles changes of 'sizefmt' option by parsing string value and updating
  * configuration values. */
 static void
@@ -2888,6 +3000,14 @@ map_name(const char name[], void *arg)
 	return (pos >= 0) ? (pos + 1) : -1;
 }
 
+void
+load_quickview_option(void)
+{
+	optval_t val;
+	val.bool_val = curr_stats.preview.on;
+	set_option("quickview", val, OPT_GLOBAL);
+}
+
 void
 load_geometry(void)
 {
@@ -2988,19 +3108,6 @@ suggestoptions_handler(OPT_OP op, optval_t val)
 	}
 }
 
-/* Converts line to a number.  Handles overflow/underflow by saturating
- * resulting value.  Returns non-zero on success and zero otherwise. */
-static int
-read_int(const char line[], int *i)
-{
-	char *endptr;
-	const long l = strtol(line, &endptr, 10);
-
-	*i = (l > INT_MAX) ? INT_MAX : ((l < INT_MIN) ? INT_MIN : l);
-
-	return *line != '\0' && *endptr == '\0';
-}
-
 /* Sets value of 'suggestoptions' based on current configuration values. */
 static void
 reset_suggestoptions(void)
@@ -3047,6 +3154,23 @@ reset_suggestoptions(void)
 	vle_tb_free(descr);
 }
 
+/* Handles changes of identifier of group of instances that synchronize their
+ * registers. */
+static void
+syncregs_handler(OPT_OP op, optval_t val)
+{
+	if(val.str_val[0] == '\0')
+	{
+		/* Option disabled -> disable functionality. */
+		regs_sync_disable();
+	}
+	else
+	{
+		/* Option enabled -> enable functionality. */
+		regs_sync_enable(val.str_val);
+	}
+}
+
 /* Makes vifm prefer to perform file-system operations with external
  * applications on rather then with system calls.  The option will be eventually
  * removed.  Mostly *nix-like systems are affected. */
@@ -3056,6 +3180,18 @@ syscalls_handler(OPT_OP op, optval_t val)
 	cfg.use_system_calls = val.bool_val;
 }
 
+/* Sets scope of a single tab. */
+static void
+tabscope_handler(OPT_OP op, optval_t val)
+{
+	/* Erase tabs on current scope before switching it. */
+	tabs_only(curr_view);
+	tabs_rename(curr_view, NULL);
+
+	cfg.pane_tabs = val.bool_val;
+	curr_stats.need_update = UT_REDRAW;
+}
+
 static void
 tabstop_handler(OPT_OP op, optval_t val)
 {
@@ -3071,14 +3207,11 @@ tabstop_handler(OPT_OP op, optval_t val)
 	text_option_changed();
 }
 
+/* Sets format according to which time is displayed. */
 static void
 timefmt_handler(OPT_OP op, optval_t val)
 {
-	free(cfg.time_format);
-	cfg.time_format = malloc(1 + strlen(val.str_val) + 1);
-	strcpy(cfg.time_format, " ");
-	strcat(cfg.time_format, val.str_val);
-
+	replace_string(&cfg.time_format, val.str_val);
 	redraw_lists();
 }
 
