@@ -86,8 +86,8 @@ FileHandleLink;
 
 static void handle_file(view_t *view, FileHandleExec exec,
 		FileHandleLink follow);
-static int is_runnable(const view_t *const view, const char full_path[],
-		int type, int force_follow);
+static int is_runnable(const view_t *view, const char full_path[], int type,
+		int force_follow);
 static int is_executable(const char full_path[], const dir_entry_t *curr,
 		int dont_execute, int runnable);
 static int is_dir_entry(const char full_path[], int type);
@@ -161,7 +161,7 @@ follow_file(view_t *view)
 static void
 handle_file(view_t *view, FileHandleExec exec, FileHandleLink follow)
 {
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 	int executable;
 	int runnable;
 	const dir_entry_t *const curr = get_current_entry(view);
@@ -208,7 +208,7 @@ handle_file(view_t *view, FileHandleExec exec, FileHandleLink follow)
 /* Returns non-zero if file can be executed or it's link to a directory (it can
  * be entered), otherwise zero is returned. */
 static int
-is_runnable(const view_t *const view, const char full_path[], int type,
+is_runnable(const view_t *view, const char full_path[], int type,
 		int force_follow)
 {
 	int runnable = !cfg.follow_links && type == FT_LINK &&
@@ -350,7 +350,7 @@ selection_is_consistent(view_t *view)
 	entry = NULL;
 	while(iter_selected_entries(view, &entry))
 	{
-		char full[PATH_MAX];
+		char full[PATH_MAX + 1];
 		get_full_path_of(entry, sizeof(full), full);
 		if(is_dir_entry(full, entry->type))
 		{
@@ -376,7 +376,7 @@ execute_file(const char full_path[], int elevate)
 #else
 	char *const dquoted_full_path = strdup(enclose_in_dquotes(full_path));
 
-	to_back_slash(dquoted_full_path);
+	internal_to_system_slashes(dquoted_full_path);
 	run_win_executable(dquoted_full_path, elevate);
 
 	free(dquoted_full_path);
@@ -659,7 +659,7 @@ run_implicit_prog(view_t *view, const char prog_spec[], int pause, int force_bg)
 static void
 view_current_file(const view_t *view)
 {
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 	get_current_full_path(view, sizeof(full_path), full_path);
 	(void)vim_view_file(full_path, -1, -1, 1);
 }
@@ -671,7 +671,7 @@ static void
 follow_link(view_t *view, int follow_dirs)
 {
 	char *dir, *file;
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 	char linkto[PATH_MAX + NAME_MAX];
 	const dir_entry_t *const curr = get_current_entry(curr_view);
 
@@ -712,10 +712,10 @@ follow_link(view_t *view, int follow_dirs)
 
 	if(file != NULL)
 	{
-		const int pos = find_file_pos_in_list(view, file);
+		const int pos = fpos_find_by_name(view, file);
 		if(pos >= 0)
 		{
-			flist_set_pos(view, pos);
+			fpos_set_pos(view, pos);
 		}
 	}
 
@@ -725,12 +725,9 @@ follow_link(view_t *view, int follow_dirs)
 void
 open_dir(view_t *view)
 {
-	char full_path[PATH_MAX];
-	const char *filename;
+	char full_path[PATH_MAX + 1];
 
-	filename = get_current_file_name(view);
-
-	if(is_parent_dir(filename))
+	if(is_parent_dir(get_current_file_name(view)))
 	{
 		cd_updir(view, 1);
 		return;
@@ -740,7 +737,9 @@ open_dir(view_t *view)
 
 	if(cd_is_possible(full_path))
 	{
+		curr_stats.ch_pos = (cfg_ch_pos_on(CHPOS_ENTER) ? 1 : 0);
 		navigate_to(view, full_path);
+		curr_stats.ch_pos = 1;
 	}
 }
 
@@ -795,7 +794,7 @@ cd_to_parent_dir(view_t *view)
 	if(ret == 0)
 	{
 		load_dir_list(view, 0);
-		flist_set_pos(view, find_file_pos_in_list(view, dir_name));
+		fpos_set_pos(view, fpos_find_by_name(view, dir_name));
 	}
 	return 0;
 }
@@ -825,10 +824,7 @@ shellout(const char command[], ShellPause pause, int use_term_multiplexer)
 
 	cmd = gen_shell_cmd(command, pause == PAUSE_ALWAYS, use_term_multiplexer);
 
-	if(curr_stats.load_stage != 0)
-	{
-		ui_shutdown();
-	}
+	ui_shutdown();
 	ec = vifm_system(cmd);
 	/* No WIFEXITED(ec) check here, since vifm_system(...) shouldn't return until
 	 * subprocess exited. */
@@ -863,7 +859,7 @@ shellout(const char command[], ShellPause pause, int use_term_multiplexer)
 		curr_stats.need_update = UT_REDRAW;
 	}
 
-	if(curr_stats.load_stage != 0)
+	if(curr_stats.load_stage > 0)
 	{
 		curs_set(0);
 	}
@@ -1029,6 +1025,10 @@ gen_term_multiplexer_title_arg(const char cmd[])
 	if(visubcmd != NULL)
 	{
 		title = skip_whitespace(visubcmd + strlen(vicmd) + 1);
+		if(cfg.short_term_mux_titles)
+		{
+			title = get_last_path_component(title);
+		}
 	}
 	else
 	{
