@@ -20,7 +20,6 @@
 #ifndef VIFM__RUNNING_H__
 #define VIFM__RUNNING_H__
 
-#include "ui/ui.h"
 #include "macros.h"
 
 /* Kinds of executable file treatment on file handling. */
@@ -41,31 +40,34 @@ typedef enum
 }
 ShellPause;
 
+struct view_t;
+
 /* Handles opening of current file/selection of the view. */
-void open_file(FileView *view, FileHandleExec exec);
+void open_file(struct view_t *view, FileHandleExec exec);
 
 /* Follows file to find its true location (e.g. target of symbolic link) or just
  * opens it. */
-void follow_file(FileView *view);
+void follow_file(struct view_t *view);
 
 /* Runs current file of the view guided by program specification with additional
  * options. */
-void run_using_prog(FileView *view, const char prog_spec[], int dont_execute,
-		int force_bg);
+void run_using_prog(struct view_t *view, const char prog_spec[],
+		int dont_execute, int force_bg);
 
 /* Handles opening of current file of the view as directory. */
-void open_dir(FileView *view);
+void open_dir(struct view_t *view);
 
 /* Moves the view to levels-th parent directory taking care of special cases
  * like root of FUSE mount. */
-void cd_updir(FileView *view, int levels);
+void cd_updir(struct view_t *view, int levels);
 
 /* Executes command in a shell.  Returns zero on success, otherwise non-zero is
  * returned. */
 int shellout(const char command[], ShellPause pause, int use_term_multiplexer);
 
 /* Returns zero on successful running. */
-int run_with_filetype(FileView *view, const char beginning[], int background);
+int run_with_filetype(struct view_t *view, const char beginning[],
+		int background);
 
 /* Handles most of command handling variants.  Returns:
  *  - > 0 -- handled, good to go;
@@ -76,7 +78,7 @@ int run_ext_command(const char cmd[], MacroFlags flags, int bg, int *save_msg);
 /* Runs the cmd and parses its output as list of paths to compose custom view.
  * Very custom view implies unsorted list.  Returns zero on success, otherwise
  * non-zero is returned. */
-int output_to_custom_flist(FileView *view, const char cmd[], int very,
+int output_to_custom_flist(struct view_t *view, const char cmd[], int very,
 		int interactive);
 
 /* Executes external command capturing its output as list of lines.  Sets *files
