@@ -34,6 +34,7 @@
 #include "int/fuse.h"
 #include "int/term_title.h"
 #include "ui/cancellation.h"
+#include "ui/ui.h"
 #include "utils/log.h"
 #include "utils/macros.h"
 
@@ -59,7 +60,7 @@ received_sigwinch(void)
 
 	if(!isendwin())
 	{
-		schedule_redraw();
+		stats_redraw_schedule();
 	}
 	else
 	{
@@ -71,7 +72,7 @@ static void
 received_sigcont(void)
 {
 	reset_prog_mode();
-	schedule_redraw();
+	stats_redraw_schedule();
 }
 
 static void
@@ -158,7 +159,7 @@ shutdown_nicely(int sig, const char descr[])
 {
   LOG_FUNC_ENTER;
 
-	endwin();
+	ui_shutdown();
 	term_title_update(NULL);
 	fuse_unmount_all();
 	write_info_file();
