@@ -66,11 +66,12 @@ typedef enum
 }
 TermMultiplexer;
 
+/* Types of updates that can be performed by update_screen(). */
 typedef enum
 {
-	UT_NONE, /* no update needed */
-	UT_REDRAW, /* screen redraw requested */
-	UT_FULL, /* file lists reload followed by screen redraw requested */
+	UT_NONE,   /* No update needed. */
+	UT_REDRAW, /* Screen redraw requested. */
+	UT_FULL,   /* File lists reload followed by screen redraw requested. */
 }
 UpdateType;
 
@@ -106,7 +107,8 @@ typedef struct
 {
 	char *cleanup_cmd;           /* Cleanup command. */
 	struct view_info_t *explore; /* State of explored quick view. */
-	unsigned int on : 1;         /* Whether preview mode is active. */
+	unsigned int on : 1;         /* Whether preview mode is active.  Use
+	                                stats_set_quickview() to change the value. */
 	unsigned int graphical : 1;  /* Whether current preview displays graphics. */
 	unsigned int clearing : 1;   /* Whether in process of clearing preview. */
 }
@@ -114,7 +116,7 @@ preview_t;
 
 typedef struct
 {
-	UpdateType need_update;
+	UpdateType need_update; /* Postponed way of doing an update. */
 	int last_char;
 	int save_msg; /* zero - don't save, 2 - save after resize, other - save */
 	int use_register;
@@ -123,7 +125,7 @@ typedef struct
 	int register_saved;
 	int number_of_windows;
 	int drop_new_dir_hist; /* Skip recording of new directory history. */
-	int load_stage; /* 0 - no TUI, 1 - part of TUI, 2 - TUI, 3 - all */
+	int load_stage; /* -1 - test, 0 - no TUI, 1 - part of TUI, 2 - TUI, 3 - all */
 
 	preview_t preview; /* State of preview (quickview). */
 
@@ -132,7 +134,10 @@ typedef struct
 
 	int last_search_backward;
 
-	int ch_pos; /* for :cd, :pushd and 'letter */
+	/* Flag which controls use of historical cursor positions.  On by default, but
+	 * gets temporarily disabled on certain operations possibly depending on value
+	 * of options. */
+	int ch_pos;
 
 	int confirmed;
 
@@ -140,7 +145,7 @@ typedef struct
 	int skip_shellout_redraw;
 
 	col_scheme_t *cs;
-	char color_scheme[NAME_MAX];
+	char color_scheme[NAME_MAX + 1];
 
 	int msg_head, msg_tail;
 	char *msgs[51];
@@ -190,6 +195,8 @@ typedef struct
 
 	int global_local_settings; /* Set local settings globally. */
 
+	int silent_ui; /* Whether UI updates should be "paused". */
+
 	int history_size;   /* Number of elements in histories. */
 	hist_t cmd_hist;    /* History of command-line commands. */
 	hist_t search_hist; /* History of search patterns. */
@@ -255,6 +262,14 @@ int stats_file_choose_action_set(void);
 /* Records status bar message. */
 void stats_save_msg(const char msg[]);
 
+/* Updates curr_stats.preview.on field and performs necessary updates in other
+ * parts of the application. */
+void stats_set_quickview(int on);
+
+/* Non-zero argument makes UI more silent, zero argument makes it less silent.
+ * After calls with non-zero and zero arguments balance out, UI gets updated. */
+void stats_silence_ui(int more);
+
 /* Managing histories. */
 
 /* Changes size of histories stored in status_t.  Zero or negative value
