@@ -491,18 +491,12 @@ is_trash_valid(const char trash_dir[], int allow_empty)
 	    && (allow_empty || !is_dir_empty(trash_dir));
 }
 
-int
-exists_in_trash(const char trash_name[])
-{
-	return path_exists(trash_name, NODEREF);
-}
-
 int
 restore_from_trash(const char trash_name[])
 {
 	int i;
-	char full[PATH_MAX];
-	char path[PATH_MAX];
+	char full[PATH_MAX + 1];
+	char path[PATH_MAX + 1];
 
 	for(i = 0; i < nentries; ++i)
 	{
@@ -523,9 +517,9 @@ restore_from_trash(const char trash_name[])
 		char *msg, *p;
 		size_t len;
 
-		cmd_group_continue();
+		un_group_reopen_last();
 
-		msg = replace_group_msg(NULL);
+		msg = un_replace_group_msg(NULL);
 		len = strlen(msg);
 		p = realloc(msg, COMMAND_GROUP_INFO_LEN);
 		if(p == NULL)
@@ -535,11 +529,11 @@ restore_from_trash(const char trash_name[])
 
 		snprintf(msg + len, COMMAND_GROUP_INFO_LEN - len, "%s%s",
 				(msg[len - 2] != ':') ? ", " : "", strchr(trash_name, '_') + 1);
-		replace_group_msg(msg);
+		un_replace_group_msg(msg);
 		free(msg);
 
-		add_operation(OP_MOVE, NULL, NULL, full, path);
-		cmd_group_end();
+		un_group_add_op(OP_MOVE, NULL, NULL, full, path);
+		un_group_close();
 		remove_from_trash(trash_name);
 		return 0;
 	}
@@ -584,7 +578,7 @@ char *
 gen_trash_name(const char base_path[], const char name[])
 {
 	struct stat st;
-	char buf[PATH_MAX];
+	char buf[PATH_MAX + 1];
 	int i;
 	char *const trash_dir = pick_trash_dir(base_path);
 
@@ -609,7 +603,7 @@ gen_trash_name(const char base_path[], const char name[])
 char *
 pick_trash_dir(const char base_path[])
 {
-	char real_path[PATH_MAX];
+	char real_path[PATH_MAX + 1];
 	char *trash_dir = NULL;
 
 	/* We want all links resolved to do not mistakenly attribute removed files to
@@ -617,9 +611,7 @@ pick_trash_dir(const char base_path[])
 	if(os_realpath(base_path, real_path) == real_path)
 	{
 		/* If realpath() fails, just go with the original path. */
-#ifdef _WIN32
-		to_forward_slash(real_path);
-#endif
+		system_to_internal_slashes(real_path);
 		base_path = real_path;
 	}
 
@@ -866,7 +858,7 @@ expand_uid(const char spec[], int *expanded)
 static char *
 get_rooted_trash_dir(const char base_path[], const char spec[])
 {
-	char full[PATH_MAX];
+	char full[PATH_MAX + 1];
 	if(get_mount_point(base_path, sizeof(full), full) == 0)
 	{
 		return format_root_spec(spec, full);
