@@ -70,6 +70,17 @@ char *HI_GROUPS[] = {
 	[SUGGEST_BOX_COLOR]  = "SuggestBox",
 	[MISMATCH_COLOR]     = "CmpMismatch",
 	[AUX_WIN_COLOR]      = "AuxWin",
+	[TAB_LINE_COLOR]     = "TabLine",
+	[TAB_LINE_SEL_COLOR] = "TabLineSel",
+	[USER1_COLOR]        = "User1",
+	[USER2_COLOR]        = "User2",
+	[USER3_COLOR]        = "User3",
+	[USER4_COLOR]        = "User4",
+	[USER5_COLOR]        = "User5",
+	[USER6_COLOR]        = "User6",
+	[USER7_COLOR]        = "User7",
+	[USER8_COLOR]        = "User8",
+	[USER9_COLOR]        = "User9",
 };
 ARRAY_GUARD(HI_GROUPS, MAXNUM_COLOR);
 
@@ -96,6 +107,17 @@ const char *HI_GROUPS_DESCR[] = {
 	[SUGGEST_BOX_COLOR]  = "suggestion box",
 	[MISMATCH_COLOR]     = "mismatched diff entries",
 	[AUX_WIN_COLOR]      = "auxiliary part of window",
+	[TAB_LINE_COLOR]     = "tab line",
+	[TAB_LINE_SEL_COLOR] = "tip of selected tab",
+	[USER1_COLOR]        = "user color #1",
+	[USER2_COLOR]        = "user color #2",
+	[USER3_COLOR]        = "user color #3",
+	[USER4_COLOR]        = "user color #4",
+	[USER5_COLOR]        = "user color #5",
+	[USER6_COLOR]        = "user color #6",
+	[USER7_COLOR]        = "user color #7",
+	[USER8_COLOR]        = "user color #8",
+	[USER9_COLOR]        = "user color #9",
 };
 ARRAY_GUARD(HI_GROUPS_DESCR, ARRAY_LEN(HI_GROUPS));
 
@@ -394,6 +416,17 @@ static const col_attr_t default_cs[] = {
 	[SUGGEST_BOX_COLOR]  = { -1,            -1,          A_BOLD                  },
 	[MISMATCH_COLOR]     = { COLOR_WHITE,   COLOR_RED,   A_BOLD                  },
 	[AUX_WIN_COLOR]      = { -1,            -1,          -1                      },
+	[TAB_LINE_COLOR]     = { COLOR_WHITE,   COLOR_BLACK, 0                       },
+	[TAB_LINE_SEL_COLOR] = { -1,            -1,          A_BOLD | A_REVERSE      },
+	[USER1_COLOR]        = { -1,            -1,          -1                      },
+	[USER2_COLOR]        = { -1,            -1,          -1                      },
+	[USER3_COLOR]        = { -1,            -1,          -1                      },
+	[USER4_COLOR]        = { -1,            -1,          -1                      },
+	[USER5_COLOR]        = { -1,            -1,          -1                      },
+	[USER6_COLOR]        = { -1,            -1,          -1                      },
+	[USER7_COLOR]        = { -1,            -1,          -1                      },
+	[USER8_COLOR]        = { -1,            -1,          -1                      },
+	[USER9_COLOR]        = { -1,            -1,          -1                      },
 };
 ARRAY_GUARD(default_cs, MAXNUM_COLOR);
 
@@ -524,7 +557,7 @@ list_cs_files(int *len)
 int
 cs_exists(const char name[])
 {
-	char cs_path[PATH_MAX];
+	char cs_path[PATH_MAX + 1];
 	get_cs_path(name, cs_path, sizeof(cs_path));
 	return is_regular_file(cs_path);
 }
@@ -585,6 +618,7 @@ cs_write(void)
 	fprintf(fp, "\" underline\n");
 	fprintf(fp, "\" reverse or inverse\n");
 	fprintf(fp, "\" standout\n");
+	fprintf(fp, "\" italic (on unsupported systems becomes reverse)\n");
 	fprintf(fp, "\" none\n\n");
 
 	fprintf(fp, "\" Vifm supports 256 colors you can use color numbers 0-255\n");
@@ -845,7 +879,7 @@ cs_load_local(int left, const char dir[])
 static int
 source_cs(const char name[])
 {
-	char cs_path[PATH_MAX];
+	char cs_path[PATH_MAX + 1];
 	get_cs_path(name, cs_path, sizeof(cs_path));
 	return cfg_source_file(cs_path);
 }
@@ -885,7 +919,7 @@ get_global_colors_dir(void)
 #ifndef _WIN32
 	return GLOBAL_COLORS_DIR;
 #else
-	static char dir_path[PATH_MAX];
+	static char dir_path[PATH_MAX + 1];
 	if(dir_path[0] == '\0')
 	{
 		snprintf(dir_path, sizeof(dir_path), "%s/colors", get_installed_data_dir());
@@ -960,6 +994,10 @@ cs_attrs_to_str(int attrs)
 		strcat(result, "reverse,");
 	if((attrs & A_STANDOUT) == A_STANDOUT)
 		strcat(result, "standout,");
+#ifdef HAVE_A_ITALIC_DECL
+	if((attrs & A_ITALIC) == A_ITALIC)
+		strcat(result, "italic,");
+#endif
 	if(result[0] != '\0')
 		result[strlen(result) - 1] = '\0';
 	return result;
@@ -1079,6 +1117,7 @@ cs_del_file_hi(const char matchers_expr[])
 	{
 		if(strcmp(matchers_get_expr(cs->file_hi[i].matchers), matchers_expr) == 0)
 		{
+			matchers_free(cs->file_hi[i].matchers);
 			memmove(&cs->file_hi[i], &cs->file_hi[i + 1],
 					sizeof(*cs->file_hi)*((cs->file_hi_count - 1) - i));
 			--cs->file_hi_count;
