@@ -49,7 +49,7 @@ static char * add_highlighted_sym(const char sym[], size_t sym_width,
 		char out[]);
 static size_t get_char_width_esc(const char str[]);
 static void print_char_esc(WINDOW *win, const char str[], esc_state *state);
-static void esc_state_update(esc_state *state, const char str[]);
+TSTATIC void esc_state_update(esc_state *state, const char str[]);
 static void esc_state_process_attr(esc_state *state, int n);
 static void esc_state_set_attr(esc_state *state, int n);
 TSTATIC const char * strchar2str(const char str[], int pos,
@@ -322,7 +322,7 @@ add_highlighted_sym(const char sym[], size_t sym_width, char out[])
 {
 	if(sym[0] != '\033')
 	{
-		strncpy(out, INV_START, sizeof(INV_START) - 1);
+		memcpy(out, INV_START, sizeof(INV_START) - 1);
 		out += sizeof(INV_START) - 1;
 	}
 
@@ -331,7 +331,7 @@ add_highlighted_sym(const char sym[], size_t sym_width, char out[])
 
 	if(sym[0] != '\033')
 	{
-		strncpy(out, INV_END, sizeof(INV_END) - 1);
+		memcpy(out, INV_END, sizeof(INV_END) - 1);
 		out += sizeof(INV_END) - 1;
 	}
 
@@ -425,7 +425,7 @@ print_char_esc(WINDOW *win, const char str[], esc_state *state)
 
 /* Handles escape sequence.  Applies whole escape sequence specified by the str
  * to the state. */
-static void
+TSTATIC void
 esc_state_update(esc_state *state, const char str[])
 {
 	str++;
@@ -476,14 +476,14 @@ esc_state_process_attr(esc_state *state, int n)
 			state->mode = (n == 5) ? ESM_WAIT_BG_COLOR : ESM_SHORT;
 			break;
 		case ESM_WAIT_FG_COLOR:
-			if(n < COLORS)
+			if(n < state->max_colors)
 			{
 				state->fg = n;
 			}
 			state->mode = ESM_SHORT;
 			break;
 		case ESM_WAIT_BG_COLOR:
-			if(n < COLORS)
+			if(n < state->max_colors)
 			{
 				state->bg = n;
 			}
@@ -497,6 +497,13 @@ esc_state_process_attr(esc_state *state, int n)
 static void
 esc_state_set_attr(esc_state *state, int n)
 {
+#ifdef HAVE_A_ITALIC_DECL
+	const int italic_attr = A_ITALIC;
+#else
+	/* If A_ITALIC is missing (it's an extension), use A_REVERSE instead. */
+	const int italic_attr = A_REVERSE;
+#endif
+
 	switch(n)
 	{
 		case 0:
@@ -510,8 +517,8 @@ esc_state_set_attr(esc_state *state, int n)
 		case 2:
 			state->attrs |= A_DIM;
 			break;
-		case 3: case 7:
-			state->attrs |= A_REVERSE;
+		case 3:
+			state->attrs |= italic_attr;
 			break;
 		case 4:
 			state->attrs |= A_UNDERLINE;
@@ -519,9 +526,15 @@ esc_state_set_attr(esc_state *state, int n)
 		case 5: case 6:
 			state->attrs |= A_BLINK;
 			break;
+		case 7:
+			state->attrs |= A_REVERSE;
+			break;
 		case 22:
 			state->attrs &= ~(A_BOLD | A_UNDERLINE | A_BLINK | A_REVERSE | A_DIM);
 			break;
+		case 23:
+			state->attrs &= ~italic_attr;
+			break;
 		case 24:
 			state->attrs &= ~A_UNDERLINE;
 			break;
@@ -547,13 +560,14 @@ esc_state_set_attr(esc_state *state, int n)
 }
 
 void
-esc_state_init(esc_state *state, const col_attr_t *defaults)
+esc_state_init(esc_state *state, const col_attr_t *defaults, int max_colors)
 {
 	state->mode = ESM_SHORT;
 	state->attrs = defaults->attr;
 	state->fg = defaults->fg;
 	state->bg = defaults->bg;
 	state->defaults = *defaults;
+	state->max_colors = max_colors;
 }
 
 /* Converts the leading character of the str string to a printable string.  Puts
@@ -621,6 +635,7 @@ strchar2str(const char str[], int pos, size_t *screen_width)
 	}
 	else
 	{
+		/* XXX: is this code completely unreachable? */
 		buf[0] = str[0];
 		buf[1] = '\0';
 		*screen_width = 1;
