@@ -109,6 +109,8 @@ static void highlight_search(view_t *view, dir_entry_t *entry,
 		int line, int col, int line_attrs);
 static int prepare_col_color(const view_t *view, int primary,
 		const column_data_t *cdt);
+static void mix_in_common_colors(col_attr_t *col, const view_t *view,
+		dir_entry_t *entry, int line_color);
 static void mix_in_file_hi(const view_t *view, dir_entry_t *entry, int type_hi,
 		col_attr_t *col);
 static void mix_in_file_name_hi(const view_t *view, dir_entry_t *entry,
@@ -305,7 +307,7 @@ draw_dir_list_only(view_t *view)
 			.view = view,
 			.entry = &view->dir_entry[x],
 			.line_pos = x,
-			.current_pos = (view == curr_view) ? view->list_pos : -1,
+			.current_pos = view->list_pos,
 		};
 
 		compute_and_draw_cell(&cdt, cell, col_width);
@@ -745,11 +747,11 @@ consider_scroll_bind(view_t *view)
 
 		if(can_scroll_up(other))
 		{
-			(void)correct_list_pos_on_scroll_down(other, 0);
+			(void)fpos_scroll_down(other, 0);
 		}
 		if(can_scroll_down(other))
 		{
-			(void)correct_list_pos_on_scroll_up(other, 0);
+			(void)fpos_scroll_up(other, 0);
 		}
 
 		other->curr_line = other->list_pos - other->top_line;
@@ -788,7 +790,7 @@ fview_cursor_redraw(view_t *view)
 	{
 		/* Call file list function, which will also ensure that current position in
 		 * list is correct. */
-		flist_set_pos(view, view->list_pos);
+		fpos_set_pos(view, view->list_pos);
 	}
 	else
 	{
@@ -842,12 +844,7 @@ prepare_inactive_color(view_t *view, dir_entry_t *entry, int line_color)
 	const col_scheme_t *cs = ui_view_get_cs(view);
 	col_attr_t col = cs->color[WIN_COLOR];
 
-	mix_in_file_hi(view, entry, line_color, &col);
-
-	if(entry->selected)
-	{
-		cs_mix_colors(&col, &cs->color[SELECTED_COLOR]);
-	}
+	mix_in_common_colors(&col, view, entry, line_color);
 
 	if(cs_is_color_set(&cs->color[OTHER_LINE_COLOR]))
 	{
@@ -1228,7 +1225,6 @@ highlight_search(view_t *view, dir_entry_t *entry, const char full_column[],
 static int
 prepare_col_color(const view_t *view, int primary, const column_data_t *cdt)
 {
-	view_t *const other = (view == &lwin) ? &rwin : &lwin;
 	const col_scheme_t *const cs = ui_view_get_cs(view);
 	col_attr_t col = cs->color[WIN_COLOR];
 
@@ -1239,23 +1235,9 @@ prepare_col_color(const view_t *view, int primary, const column_data_t *cdt)
 
 	/* File-specific highlight affects only primary field for non-current lines
 	 * and whole line for the current line. */
-	if(primary || cdt->line_pos == cdt->current_pos)
-	{
-		mix_in_file_hi(view, cdt->entry, cdt->line_hi_group, &col);
-	}
-
-	/* If two files on the same line in side-by-side comparison have different
-	 * ids, that's a mismatch. */
-	if(view->custom.type == CV_DIFF &&
-			other->dir_entry[entry_to_pos(view, cdt->entry)].id != cdt->entry->id)
-	{
-		cs_mix_colors(&col, &cs->color[MISMATCH_COLOR]);
-	}
-
-	if(cdt->entry->selected)
-	{
-		cs_mix_colors(&col, &cs->color[SELECTED_COLOR]);
-	}
+	const int with_line_hi = (primary || cdt->line_pos == cdt->current_pos);
+	const int line_color = with_line_hi ? cdt->line_hi_group : -1;
+	mix_in_common_colors(&col, view, cdt->entry, line_color);
 
 	if(cdt->line_pos == cdt->current_pos)
 	{
@@ -1272,6 +1254,33 @@ prepare_col_color(const view_t *view, int primary, const column_data_t *cdt)
 	return COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) | col.attr;
 }
 
+/* Mixes in colors of current entry, mismatch and selection. */
+static void
+mix_in_common_colors(col_attr_t *col, const view_t *view, dir_entry_t *entry,
+		int line_color)
+{
+	if(line_color >= 0)
+	{
+		mix_in_file_hi(view, entry, line_color, col);
+	}
+
+	const col_scheme_t *const cs = ui_view_get_cs(view);
+	view_t *const other = (view == &lwin) ? &rwin : &lwin;
+
+	/* If two files on the same line in side-by-side comparison have different
+	 * ids, that's a mismatch. */
+	if(view->custom.type == CV_DIFF &&
+			other->dir_entry[entry_to_pos(view, entry)].id != entry->id)
+	{
+		cs_mix_colors(col, &cs->color[MISMATCH_COLOR]);
+	}
+
+	if(entry->selected)
+	{
+		cs_mix_colors(col, &cs->color[SELECTED_COLOR]);
+	}
+}
+
 /* Applies file name and file type specific highlights for the entry. */
 static void
 mix_in_file_hi(const view_t *view, dir_entry_t *entry, int type_hi,
@@ -1393,10 +1402,10 @@ format_size(int id, const void *data, size_t buf_len, char buf[])
 	if(fentry_is_dir(cdt->entry))
 	{
 		uint64_t nitems;
-		fentry_get_dir_info(view, cdt->entry, &size, &nitems);
+		uint64_t *nitems_ptr = (cfg.view_dir_size == VDS_NITEMS ? &nitems : NULL);
+		fentry_get_dir_info(view, cdt->entry, &size, nitems_ptr);
 
-		if(size == DCACHE_UNKNOWN && cfg.view_dir_size == VDS_NITEMS &&
-				nitems != DCACHE_UNKNOWN)
+		if(size == DCACHE_UNKNOWN && nitems_ptr != NULL)
 		{
 			snprintf(buf, buf_len + 1, " %d", (int)nitems);
 			return;
@@ -1449,7 +1458,7 @@ static void
 format_target(int id, const void *data, size_t buf_len, char buf[])
 {
 	const column_data_t *cdt = data;
-	char full_path[PATH_MAX];
+	char full_path[PATH_MAX + 1];
 
 	buf[0] = '\0';
 
@@ -1514,7 +1523,8 @@ format_time(int id, const void *data, size_t buf_len, char buf[])
 
 	if(tm_ptr != NULL)
 	{
-		strftime(buf, buf_len + 1, cfg.time_format, tm_ptr);
+		buf[0] = ' ';
+		strftime(buf + 1, buf_len, cfg.time_format, tm_ptr);
 	}
 	else
 	{
