@@ -102,7 +102,7 @@ qv_ensure_is_shown(void)
 	{
 		return 1;
 	}
-	curr_stats.preview.on = 1;
+	stats_set_quickview(1);
 	return 0;
 }
 
@@ -127,7 +127,7 @@ qv_toggle(void)
 {
 	if(curr_stats.preview.on)
 	{
-		curr_stats.preview.on = 0;
+		stats_set_quickview(0);
 
 		if(ui_view_is_visible(other_view))
 		{
@@ -148,7 +148,7 @@ qv_toggle(void)
 	}
 	else
 	{
-		curr_stats.preview.on = 1;
+		stats_set_quickview(1);
 		qv_draw(curr_view);
 	}
 }
@@ -159,11 +159,21 @@ qv_draw(view_t *view)
 	const dir_entry_t *curr;
 
 	if(curr_stats.load_stage < 2 || curr_stats.number_of_windows == 1 ||
-	   vle_mode_is(VIEW_MODE) || view_detached_draw())
+	   vle_mode_is(VIEW_MODE))
 	{
 		return;
 	}
 
+	/* Viewers might use relative path, so try to make sure that we're at correct
+	 * location. */
+	(void)vifm_chdir(flist_get_dir(view));
+
+	if(view_detached_draw())
+	{
+		/* View mode handled the drawing. */
+		return;
+	}
+
 	ui_view_erase(other_view);
 
 	curr = get_current_entry(view);
@@ -180,7 +190,7 @@ qv_draw(view_t *view)
 static void
 view_entry(const dir_entry_t *entry)
 {
-	char path[PATH_MAX];
+	char path[PATH_MAX + 1];
 	qv_get_path_to_explore(entry, path, sizeof(path));
 
 	switch(entry->type)
@@ -362,7 +372,7 @@ print_dir_tree(tree_print_state_t *s, const char path[], int last)
 	reached_limit = 0;
 	for(i = 0; i < len && !reached_limit && !ui_cancellation_requested(); ++i)
 	{
-		char link_target[PATH_MAX];
+		char link_target[PATH_MAX + 1];
 		const int last_entry = (i == len - 1);
 		char *const full_path = format_str("%s/%s", path, lst[i]);
 
@@ -537,7 +547,7 @@ view_stream(FILE *fp, int wrapped)
 	const char *res;
 	esc_state state;
 
-	esc_state_init(&state, &cs->color[WIN_COLOR]);
+	esc_state_init(&state, &cs->color[WIN_COLOR], COLORS);
 
 	skip_bom(fp);
 	res = get_line(fp, line, sizeof(line));
@@ -646,7 +656,7 @@ qv_execute_viewer(const char viewer[])
 	char *expanded;
 
 	expanded = expand_viewer_command(viewer);
-	fp = read_cmd_output(expanded);
+	fp = read_cmd_output(expanded, 0);
 	free(expanded);
 
 	return fp;
