@@ -33,6 +33,8 @@
 #include "../compat/pthread.h"
 #include "../compat/reallocarray.h"
 #include "../engine/mode.h"
+#include "../engine/parsing.h"
+#include "../engine/var.h"
 #include "../modes/modes.h"
 #include "../utils/fs.h"
 #include "../utils/log.h"
@@ -45,13 +47,29 @@
 #include "../utils/utils.h"
 #include "../background.h"
 #include "../filelist.h"
+#include "color_manager.h"
+#include "color_scheme.h"
 #include "ui.h"
 
+/* Line paired with parallel character of arrays that specify user colors. */
+typedef struct
+{
+	char *line;       /* Text of the line. */
+	size_t line_len;  /* Length of line field. */
+	char *attrs;      /* Specifies when to enable which user highlight group. */
+	size_t attrs_len; /* Length of attrs field. */
+}
+LineWithAttrs;
+
+static void print_with_attrs(WINDOW *win, const char line[], const char attrs[],
+		int default_attr);
 static void update_stat_window_old(view_t *view, int lazy_redraw);
 static void refresh_window(WINDOW *win, int lazily);
-TSTATIC char * expand_status_line_macros(view_t *view, const char format[]);
-static char * parse_view_macros(view_t *view, const char **format,
+TSTATIC LineWithAttrs expand_status_line_macros(view_t *view,
+		const char format[]);
+static LineWithAttrs parse_view_macros(view_t *view, const char **format,
 		const char macros[], int opt);
+static int sync_attrs(LineWithAttrs *result, int extra_width);
 static int expand_num(char buf[], size_t buf_len, int val);
 static const char * get_tip(void);
 static void check_expanded_str(const char buf[], int skip, int *nexpansions);
@@ -73,9 +91,6 @@ static pthread_spinlock_t job_bar_changed_lock;
 void
 ui_stat_update(view_t *view, int lazy_redraw)
 {
-	int x;
-	char *buf;
-
 	if(!cfg.display_statusline || view != curr_view)
 	{
 		return;
@@ -95,22 +110,57 @@ ui_stat_update(view_t *view, int lazy_redraw)
 		return;
 	}
 
-	x = getmaxx(stdscr);
-	wresize(stat_win, 1, x);
-	wbkgdset(stat_win, COLOR_PAIR(cfg.cs.pair[STATUS_LINE_COLOR]) |
-			cfg.cs.color[STATUS_LINE_COLOR].attr);
-
-	buf = expand_status_line_macros(view, cfg.status_line);
-	buf = break_in_two(buf, getmaxx(stdscr));
+	const int width = getmaxx(stdscr);
+	const int default_attr = COLOR_PAIR(cfg.cs.pair[STATUS_LINE_COLOR])
+	                       | cfg.cs.color[STATUS_LINE_COLOR].attr;
 
+	wresize(stat_win, 1, width);
+	wbkgdset(stat_win, default_attr);
 	werase(stat_win);
 	checked_wmove(stat_win, 0, 0);
-	wprint(stat_win, buf);
-	free(buf);
+
+	LineWithAttrs result = expand_status_line_macros(view, cfg.status_line);
+	assert(strlen(result.attrs) == utf8_strsw(result.line) && "Broken attrs!");
+	result.line = break_in_two(result.line, width, "%=");
+	result.attrs = break_in_two(result.attrs, width, "=");
+	print_with_attrs(stat_win, result.line, result.attrs, default_attr);
+	free(result.line);
+	free(result.attrs);
 
 	refresh_window(stat_win, lazy_redraw);
 }
 
+/* Prints line onto a window highlighting it according to attrs, which should
+ * specify 0-9 color groups for every character in line. */
+static void
+print_with_attrs(WINDOW *win, const char line[], const char attrs[],
+		int default_attr)
+{
+	int attr = default_attr;
+	while(*line != '\0')
+	{
+		if(*attrs == '0')
+		{
+			attr = default_attr;
+		}
+		else if(*attrs != ' ')
+		{
+			const int color = (USER1_COLOR + (*attrs - '1'));
+			col_attr_t col = cfg.cs.color[STATUS_LINE_COLOR];
+			cs_mix_colors(&col, &cfg.cs.color[color]);
+			attr = COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) | col.attr;
+		}
+
+		const size_t len = utf8_chrw(line);
+		char char_buf[len + 1];
+		copy_str(char_buf, sizeof(char_buf), line);
+		wprinta(win, char_buf, attr);
+
+		line += len;
+		attrs += utf8_chrsw(char_buf);
+	}
+}
+
 /* Formats status line in the "old way" (before introduction of 'statusline'
  * option). */
 static void
@@ -191,17 +241,27 @@ refresh_window(WINDOW *win, int lazily)
 	}
 	else
 	{
-		wrefresh(stat_win);
+		ui_refresh_win(stat_win);
 	}
 }
 
 /* Expands view macros to be displayed on the status line according to the
- * format string.  Returns newly allocated string, which should be freed by the
- * caller, or NULL if there is not enough memory. */
-TSTATIC char *
+ * format string.  Returns expanded line and attribute line, the latter one
+ * contains a character in the set [0-9 ] (space included) per utf-8 character
+ * of the former that specifies which user highlight group should be used
+ * starting with that character. */
+TSTATIC LineWithAttrs
 expand_status_line_macros(view_t *view, const char format[])
 {
-	return expand_view_macros(view, format, "tTfaAugsEdD-xlLSz%[]");
+	const dir_entry_t *const curr = get_current_entry(view);
+	if(curr == NULL || fentry_is_fake(curr))
+	{
+		/* Fake entries don't have valid information. */
+		LineWithAttrs result = { .line = strdup(""), .attrs = strdup("") };
+		return result;
+	}
+
+	return parse_view_macros(view, &format, "tTfaAugsEdD-xlLSz%[]{*", 0);
 }
 
 /* Expands possibly limited set of view macros.  Returns newly allocated string,
@@ -209,24 +269,26 @@ expand_status_line_macros(view_t *view, const char format[])
 char *
 expand_view_macros(view_t *view, const char format[], const char macros[])
 {
-	return parse_view_macros(view, &format, macros, 0);
+	LineWithAttrs result = parse_view_macros(view, &format, macros, 0);
+	free(result.attrs);
+	return result.line;
 }
 
 /* Expands macros in the *format string advancing the pointer as it goes.  The
  * opt represents conditional expression state, should be zero for non-recursive
  * calls.  Returns newly allocated string, which should be freed by the
  * caller. */
-static char *
+static LineWithAttrs
 parse_view_macros(view_t *view, const char **format, const char macros[],
 		int opt)
 {
 	const dir_entry_t *const curr = get_current_entry(view);
-	char *result = strdup("");
-	size_t len = 0;
+	LineWithAttrs result = { .line = strdup(""), .attrs = strdup("") };
 	char c;
 	int nexpansions = 0;
+	int has_expander = 0;
 
-	if(curr == NULL || fentry_is_fake(curr))
+	if(curr == NULL)
 	{
 		return result;
 	}
@@ -239,15 +301,31 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 		const char *const next = ++*format;
 		int skip, ok;
 
-		if(c != '%' || (!char_is_one_of(macros, *next) && !isdigit(*next)))
+		if(c != '%' ||
+				(!char_is_one_of(macros, *next) && !isdigit(*next) &&
+				 (*next != '=' || has_expander)))
 		{
-			if(strappendch(&result, &len, c) != 0)
+			if(strappendch(&result.line, &result.line_len, c) != 0)
 			{
 				break;
 			}
 			continue;
 		}
 
+		if(*next == '=')
+		{
+			(void)sync_attrs(&result, 0);
+
+			if(strappend(&result.line, &result.line_len, "%=") != 0 ||
+					strappendch(&result.attrs, &result.attrs_len, '=') != 0)
+			{
+				break;
+			}
+			++*format;
+			has_expander = 1;
+			continue;
+		}
+
 		if(*next == '-')
 		{
 			left_align = 1;
@@ -355,9 +433,21 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 				break;
 			case '[':
 				{
-					char *const opt_str = parse_view_macros(view, format, macros, 1);
-					copy_str(buf, sizeof(buf), opt_str);
-					free(opt_str);
+					LineWithAttrs opt = parse_view_macros(view, format, macros, 1);
+					copy_str(buf, sizeof(buf), opt.line);
+					free(opt.line);
+
+					char *attrs = opt.attrs;
+					if(sync_attrs(&result, 0) && opt.attrs_len > 0U)
+					{
+						if(*attrs != ' ')
+						{
+							result.attrs[result.attrs_len - 1U] = *attrs;
+						}
+						++attrs;
+					}
+					strappend(&result.attrs, &result.attrs_len, attrs);
+					free(opt.attrs);
 					break;
 				}
 			case ']':
@@ -365,7 +455,14 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 				{
 					if(nexpansions == 0)
 					{
-						replace_string(&result, "");
+						replace_string(&result.line, "");
+						replace_string(&result.attrs, "");
+						result.line_len = 0U;
+						result.attrs_len = 0U;
+					}
+					if(sync_attrs(&result, 0))
+					{
+						result.attrs[--result.attrs_len] = '\0';
 					}
 					return result;
 				}
@@ -373,6 +470,64 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 				LOG_INFO_MSG("Unmatched %%]");
 				ok = 0;
 				break;
+			case '{':
+				{
+					/* Try to find matching closing bracket
+					 * TODO: implement the way to escape it, so that the expr may contain
+					 * closing brackets */
+					const char *e = strchr(*format, '}');
+					char *expr = NULL, *resstr = NULL;
+					var_t res = var_false();
+					ParsingErrors parsing_error;
+
+					/* If there's no matching closing bracket, just add the opening one
+					 * literally */
+					if(e == NULL)
+					{
+						ok = 0;
+						break;
+					}
+
+					/* Create a NULL-terminated copy of the given expr.
+					 * TODO: we could temporarily use buf for that, to avoid extra
+					 * allocation, but explicitly named variable reads better. */
+					expr = calloc(e - (*format) + 1 /* NUL-term */, 1);
+					memcpy(expr, *format, e - (*format));
+
+					/* Try to parse expr, and convert the res to string if succeed. */
+					parsing_error = parse(expr, 0, &res);
+					if(parsing_error == PE_NO_ERROR)
+					{
+						resstr = var_to_str(res);
+					}
+
+					if(resstr != NULL)
+					{
+						copy_str(buf, sizeof(buf), resstr);
+					}
+					else
+					{
+						copy_str(buf, sizeof(buf), "<Invalid expr>");
+					}
+
+					var_free(res);
+					free(resstr);
+					free(expr);
+
+					*format = e + 1 /* closing bracket */;
+				}
+				break;
+			case '*':
+				if(width > 9)
+				{
+					snprintf(buf, sizeof(buf), "%%%d*", (int)width);
+					width = 0;
+					break;
+				}
+				(void)sync_attrs(&result, 1);
+				result.attrs[result.attrs_len - 1] = '0' + width;
+				width = 0;
+				break;
 
 			default:
 				LOG_INFO_MSG("Unexpected %%-sequence: %%%c", c);
@@ -388,7 +543,7 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 		if(!ok)
 		{
 			*format = next;
-			if(strappendch(&result, &len, '%') != 0)
+			if(strappendch(&result.line, &result.line_len, '%') != 0)
 			{
 				break;
 			}
@@ -398,7 +553,7 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 		check_expanded_str(buf, skip, &nexpansions);
 		stralign(buf, width, ' ', left_align);
 
-		if(strappend(&result, &len, buf) != 0)
+		if(strappend(&result.line, &result.line_len, buf) != 0)
 		{
 			break;
 		}
@@ -407,12 +562,34 @@ parse_view_macros(view_t *view, const char **format, const char macros[],
 	/* Unmatched %[. */
 	if(opt)
 	{
-		(void)strprepend(&result, &len, "%[");
+		(void)strprepend(&result.line, &result.line_len, "%[");
 	}
 
+	if(sync_attrs(&result, 0))
+	{
+		result.attrs[--result.attrs_len] = '\0';
+	}
 	return result;
 }
 
+/* Makes sure that result->attrs has at least as many elements as result->line
+ * contains characters + extra_width.  Returns non-zero if result->attrs has
+ * extra characters compared to result->line. */
+static int
+sync_attrs(LineWithAttrs *result, int extra_width)
+{
+	const size_t nchars = utf8_strsw(result->line) + extra_width;
+	if(result->attrs_len < nchars)
+	{
+		char *const new_attrs = format_str("%s%*s", result->attrs,
+				(int)(nchars - result->attrs_len), "");
+		free(result->attrs);
+		result->attrs = new_attrs;
+		result->attrs_len = nchars;
+	}
+	return (result->attrs_len > nchars);
+}
+
 /* Prints number into the buffer.  Returns non-zero if numeric value is
  * "empty" (zero). */
 static int
@@ -517,8 +694,8 @@ ui_stat_refresh(void)
 {
 	ui_stat_job_bar_check_for_updates();
 
-	wrefresh(job_bar);
-	wrefresh(stat_win);
+	ui_refresh_win(job_bar);
+	ui_refresh_win(stat_win);
 }
 
 int
