@@ -17,6 +17,10 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+/* Save wrefresh identifier behind the macro before it's poisoned in the
+ * header. */
+#define use_wrefresh wrefresh
+
 #include "ui.h"
 
 #include <curses.h> /* mvwin() wbkgdset() werase() */
@@ -73,6 +77,7 @@
 #include "quickview.h"
 #include "statusbar.h"
 #include "statusline.h"
+#include "tabs.h"
 
 /* Type of path transformation function for format_view_title(). */
 typedef char * (*path_func)(const char[]);
@@ -90,6 +95,8 @@ static pthread_mutex_t rwin_timestamps_mutex = PTHREAD_MUTEX_INITIALIZER;
 view_t lwin = { .timestamps_mutex = &lwin_timestamps_mutex };
 view_t rwin = { .timestamps_mutex = &rwin_timestamps_mutex };
 
+static int pair_in_use(short int pair);
+static void move_pair(short int from, short int to);
 static void create_windows(void);
 static void update_geometry(void);
 static int get_working_area_height(void);
@@ -106,13 +113,20 @@ static int are_statusbar_widgets_visible(void);
 static int get_ruler_width(view_t *view);
 static char * expand_ruler_macros(view_t *view, const char format[]);
 static void switch_panes_content(void);
-static void update_origins(view_t *view, const char *old_main_origin);
 static void set_splitter(int pos);
+static FileType ui_view_entry_target_type(const dir_entry_t *entry);
 static void refresh_bottom_lines(void);
 static char * path_identity(const char path[]);
+static int view_shows_tabline(const view_t *view);
+static int get_tabline_height(void);
+static void print_tab_title(WINDOW *win, view_t *view, col_attr_t base_col,
+		path_func pf);
+static void compute_avg_width(int *avg_width, int *spare_width, int max_width,
+		view_t *view, path_func pf);
+static char * make_tab_title(const tab_info_t *tab_info, path_func pf);
 static char * format_view_title(const view_t *view, path_func pf);
 static void print_view_title(const view_t *view, int active_view, char title[]);
-static void fixup_titles_attributes(const view_t *view, int active_view);
+static col_attr_t fixup_titles_attributes(const view_t *view, int active_view);
 static int is_in_miller_view(const view_t *view);
 static int is_forced_list_mode(const view_t *view);
 static uint64_t get_updated_time(uint64_t prev);
@@ -132,12 +146,12 @@ ui_ruler_update(view_t *view, int lazy_redraw)
 	update_statusbar_layout();
 
 	expanded = expand_ruler_macros(view, cfg.ruler_format);
-	expanded = break_in_two(expanded, getmaxx(ruler_win));
+	expanded = break_in_two(expanded, getmaxx(ruler_win), "%=");
 
 	ui_ruler_set(expanded);
 	if(!lazy_redraw)
 	{
-		wrefresh(ruler_win);
+		ui_refresh_win(ruler_win);
 	}
 
 	free(expanded);
@@ -166,18 +180,30 @@ setup_ncurses_interface(void)
 	curs_set(0);
 
 	getmaxyx(stdscr, screen_y, screen_x);
-	/* screen is too small to be useful*/
+	/* Screen is too small to be useful. */
 	if(screen_y < MIN_TERM_HEIGHT || screen_x < MIN_TERM_WIDTH)
 	{
 		vifm_finish("Terminal is too small to run vifm.");
 	}
 
 	if(!has_colors())
+	{
 		vifm_finish("Vifm requires a console that can support color.");
+	}
 
 	start_color();
 	use_default_colors();
 
+	const colmgr_conf_t colmgr_conf = {
+		.max_color_pairs = COLOR_PAIRS,
+		.max_colors = COLORS,
+		.init_pair = &init_pair,
+		.pair_content = &pair_content,
+		.pair_in_use = &pair_in_use,
+		.move_pair = &move_pair,
+	};
+	colmgr_init(&colmgr_conf);
+
 	cs_load_defaults();
 
 	create_windows();
@@ -203,6 +229,47 @@ setup_ncurses_interface(void)
 	return 1;
 }
 
+/* Checks whether pair is being used at the moment.  Returns non-zero if so and
+ * zero otherwise. */
+static int
+pair_in_use(short int pair)
+{
+	int i;
+
+	for(i = 0; i < MAXNUM_COLOR; ++i)
+	{
+		if(cfg.cs.pair[i] == pair || lwin.cs.pair[i] == pair ||
+				rwin.cs.pair[i] == pair)
+		{
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
+/* Substitutes old pair number with the new one. */
+static void
+move_pair(short int from, short int to)
+{
+	int i;
+	for(i = 0; i < MAXNUM_COLOR; ++i)
+	{
+		if(cfg.cs.pair[i] == from)
+		{
+			cfg.cs.pair[i] = to;
+		}
+		if(lwin.cs.pair[i] == from)
+		{
+			lwin.cs.pair[i] = to;
+		}
+		if(rwin.cs.pair[i] == from)
+		{
+			rwin.cs.pair[i] = to;
+		}
+	}
+}
+
 /* Initializes all WINDOW variables by calling newwin() to create ncurses
  * windows and configures hardware cursor. */
 static void
@@ -224,6 +291,7 @@ create_windows(void)
 	ltop_line2 = newwin(1, 1, 0, 0);
 
 	top_line = newwin(1, 1, 0, 0);
+	tab_line = newwin(1, 1, 0, 0);
 
 	rtop_line1 = newwin(1, 1, 0, 0);
 	rtop_line2 = newwin(1, 1, 0, 0);
@@ -253,6 +321,7 @@ create_windows(void)
 	leaveok(ltop_line1, TRUE);
 	leaveok(ltop_line2, TRUE);
 	leaveok(top_line, TRUE);
+	leaveok(tab_line, TRUE);
 	leaveok(rtop_line1, TRUE);
 	leaveok(rtop_line2, TRUE);
 	leaveok(rwin.title, TRUE);
@@ -313,13 +382,23 @@ only_layout(view_t *view, int screen_x)
 {
 	const int vborder_pos_correction = cfg.side_borders_visible ? 1 : 0;
 	const int vborder_size_correction = cfg.side_borders_visible ? -2 : 0;
+	const int y = get_tabline_height();
 
 	wresize(view->title, 1, screen_x - 2);
-	mvwin(view->title, 0, 1);
+	mvwin(view->title, y, 1);
+
+	mvwin(ltop_line1, y, 0);
+	mvwin(ltop_line2, y, 0);
+
+	mvwin(rtop_line1, y, screen_x - 1);
+	mvwin(rtop_line2, y, screen_x - 1);
+
+	wresize(tab_line, 1, screen_x);
+	mvwin(tab_line, 0, 0);
 
 	wresize(view->win, get_working_area_height(),
 			screen_x + vborder_size_correction);
-	mvwin(view->win, 1, vborder_pos_correction);
+	mvwin(view->win, y + 1, vborder_pos_correction);
 }
 
 /* Updates TUI elements sizes and coordinates for vertical configuration of
@@ -330,6 +409,7 @@ vertical_layout(int screen_x)
 	const int vborder_pos_correction = cfg.side_borders_visible ? 1 : 0;
 	const int vborder_size_correction = cfg.side_borders_visible ? -1 : 0;
 	const int border_height = get_working_area_height();
+	const int y = get_tabline_height();
 
 	int splitter_pos;
 	int splitter_width;
@@ -348,31 +428,34 @@ vertical_layout(int screen_x)
 		curr_stats.splitter_pos = splitter_pos;
 
 	wresize(lwin.title, 1, splitter_pos - 1);
-	mvwin(lwin.title, 0, 1);
+	mvwin(lwin.title, y, 1);
 
 	wresize(lwin.win, border_height, splitter_pos + vborder_size_correction);
-	mvwin(lwin.win, 1, vborder_pos_correction);
+	mvwin(lwin.win, y + 1, vborder_pos_correction);
 
 	wbkgdset(mborder, COLOR_PAIR(cfg.cs.pair[BORDER_COLOR]) |
 			cfg.cs.color[BORDER_COLOR].attr);
 	wresize(mborder, border_height, splitter_width);
-	mvwin(mborder, 1, splitter_pos);
+	mvwin(mborder, y + 1, splitter_pos);
 
-	mvwin(ltop_line1, 0, 0);
-	mvwin(ltop_line2, 0, 0);
+	mvwin(ltop_line1, y, 0);
+	mvwin(ltop_line2, y, 0);
+
+	wresize(tab_line, 1, screen_x);
+	mvwin(tab_line, 0, 0);
 
 	wresize(top_line, 1, splitter_width);
-	mvwin(top_line, 0, splitter_pos);
+	mvwin(top_line, y, splitter_pos);
 
-	mvwin(rtop_line1, 0, screen_x - 1);
-	mvwin(rtop_line2, 0, screen_x - 1);
+	mvwin(rtop_line1, y, screen_x - 1);
+	mvwin(rtop_line2, y, screen_x - 1);
 
 	wresize(rwin.title, 1, screen_x - (splitter_pos + splitter_width + 1));
-	mvwin(rwin.title, 0, splitter_pos + splitter_width);
+	mvwin(rwin.title, y, splitter_pos + splitter_width);
 
 	wresize(rwin.win, border_height,
 			screen_x - (splitter_pos + splitter_width) + vborder_size_correction);
-	mvwin(rwin.win, 1, splitter_pos + splitter_width);
+	mvwin(rwin.win, y + 1, splitter_pos + splitter_width);
 }
 
 /* Updates TUI elements sizes and coordinates for horizontal configuration of
@@ -382,6 +465,7 @@ horizontal_layout(int screen_x, int screen_y)
 {
 	const int vborder_pos_correction = cfg.side_borders_visible ? 1 : 0;
 	const int vborder_size_correction = cfg.side_borders_visible ? -2 : 0;
+	const int y = get_tabline_height();
 
 	int splitter_pos;
 
@@ -397,15 +481,15 @@ horizontal_layout(int screen_x, int screen_y)
 		curr_stats.splitter_pos = splitter_pos;
 
 	wresize(lwin.title, 1, screen_x - 2);
-	mvwin(lwin.title, 0, 1);
+	mvwin(lwin.title, y, 1);
 
 	wresize(rwin.title, 1, screen_x - 2);
 	mvwin(rwin.title, splitter_pos, 1);
 
-	wresize(lwin.win, splitter_pos - 1, screen_x + vborder_size_correction);
-	mvwin(lwin.win, 1, vborder_pos_correction);
+	wresize(lwin.win, splitter_pos - (y + 1), screen_x + vborder_size_correction);
+	mvwin(lwin.win, y + 1, vborder_pos_correction);
 
-	wresize(rwin.win, get_working_area_height() - splitter_pos,
+	wresize(rwin.win, get_working_area_height() - splitter_pos + y,
 			screen_x + vborder_size_correction);
 	mvwin(rwin.win, splitter_pos + 1, vborder_pos_correction);
 
@@ -414,20 +498,23 @@ horizontal_layout(int screen_x, int screen_y)
 	wresize(mborder, 1, screen_x);
 	mvwin(mborder, splitter_pos, 0);
 
-	mvwin(ltop_line1, 0, 0);
+	mvwin(ltop_line1, y, 0);
 	mvwin(ltop_line2, splitter_pos, 0);
 
-	wresize(top_line, 1, 2 - screen_x%2);
-	mvwin(top_line, 0, screen_x/2 - 1 + screen_x%2);
+	wresize(tab_line, 1, screen_x);
+	mvwin(tab_line, 0, 0);
 
-	mvwin(rtop_line1, 0, screen_x - 1);
+	wresize(top_line, 1, 2 - screen_x%2);
+	mvwin(top_line, y, screen_x/2 - 1 + screen_x%2);
+
+	mvwin(rtop_line1, y, screen_x - 1);
 	mvwin(rtop_line2, splitter_pos, screen_x - 1);
 
 	wresize(lborder, screen_y - 1, 1);
-	mvwin(lborder, 0, 0);
+	mvwin(lborder, y, 0);
 
 	wresize(rborder, screen_y - 1, 1);
-	mvwin(rborder, 0, screen_x - 1);
+	mvwin(rborder, y, screen_x - 1);
 }
 
 static void
@@ -491,11 +578,6 @@ resize_all(void)
 	{
 		only_layout(&lwin, screen_x);
 		only_layout(&rwin, screen_x);
-
-		mvwin(ltop_line1, 0, 0);
-		mvwin(ltop_line2, 0, 0);
-		mvwin(rtop_line1, 0, screen_x - 1);
-		mvwin(rtop_line2, 0, screen_x - 1);
 	}
 	else
 	{
@@ -527,7 +609,8 @@ get_working_area_height(void)
 	     - 1                                /* Top line. */
 	     - (cfg.display_statusline ? 1 : 0) /* Status line. */
 	     - ui_stat_job_bar_height()         /* Job bar. */
-	     - 1;                               /* Status bar line. */
+	     - 1                                /* Status bar line. */
+	     - get_tabline_height();            /* Tab line. */
 }
 
 /* Updates internal data structures to reflect actual terminal geometry. */
@@ -538,11 +621,6 @@ update_geometry(void)
 
 	update_term_size();
 
-#ifdef _WIN32
-	/* This resizes vifm to window size. */
-	resize_term(0, 0);
-#endif
-
 	getmaxyx(stdscr, screen_y, screen_x);
 	cfg.lines = screen_y;
 	cfg.columns = screen_x;
@@ -558,6 +636,19 @@ update_geometry(void)
 	load_geometry();
 }
 
+void
+ui_quit(int write_info, int force)
+{
+	if(tabs_quit_on_close())
+	{
+		vifm_try_leave(write_info, 0, force);
+	}
+	else
+	{
+		tabs_close();
+	}
+}
+
 int
 cv_unsorted(CVType type)
 {
@@ -729,15 +820,21 @@ reload_lists(void)
 	}
 }
 
-/* reloads view on window_reload() call */
+/* Reloads view handling special case of loading it for the first time during
+ * startup. */
 static void
 reload_list(view_t *view)
 {
-	if(curr_stats.load_stage >= 3)
-		load_saving_pos(view);
-	else
-		load_dir_list(view,
-				!(cfg.vifm_info&VIFMINFO_SAVEDIRS) || view->list_pos != 0);
+	if(curr_stats.load_stage < 3)
+	{
+		const int keep_position = cfg_ch_pos_on(CHPOS_STARTUP)
+		                        ? 0
+		                        : !is_dir_list_loaded(view);
+		load_dir_list(view, keep_position);
+		return;
+	}
+
+	load_saving_pos(view);
 }
 
 void
@@ -807,6 +904,8 @@ touch_all_windows(void)
 
 	if(!in_menu)
 	{
+		update_window_lazy(tab_line);
+
 		if(curr_stats.number_of_windows == 1)
 		{
 			/* In one window view. */
@@ -899,7 +998,7 @@ update_input_bar(const wchar_t *str)
 
 	werase(input_win);
 	compat_waddwstr(input_win, str);
-	wrefresh(input_win);
+	ui_refresh_win(input_win);
 }
 
 void
@@ -908,7 +1007,7 @@ clear_num_window(void)
 	if(curr_stats.use_input_bar)
 	{
 		werase(input_win);
-		wrefresh(input_win);
+		ui_refresh_win(input_win);
 	}
 }
 
@@ -1009,6 +1108,11 @@ update_attributes(void)
 	wattrset(ltop_line2, cfg.cs.color[TOP_LINE_COLOR].attr & ~A_REVERSE);
 	werase(ltop_line2);
 
+	wbkgdset(tab_line, COLOR_PAIR(cfg.cs.pair[TAB_LINE_COLOR]) |
+			(cfg.cs.color[TAB_LINE_COLOR].attr & A_REVERSE));
+	wattrset(tab_line, cfg.cs.color[TAB_LINE_COLOR].attr & ~A_REVERSE);
+	werase(tab_line);
+
 	wbkgdset(top_line, COLOR_PAIR(cfg.cs.pair[TOP_LINE_COLOR]) |
 			(cfg.cs.color[TOP_LINE_COLOR].attr & A_REVERSE));
 	wattrset(top_line, cfg.cs.color[TOP_LINE_COLOR].attr & ~A_REVERSE);
@@ -1046,6 +1150,15 @@ update_attributes(void)
 	wbkgdset(input_win, COLOR_PAIR(cfg.cs.pair[CMD_LINE_COLOR]));
 }
 
+void
+ui_refresh_win(WINDOW *win)
+{
+	if(!curr_stats.silent_ui)
+	{
+		use_wrefresh(win);
+	}
+}
+
 void
 wprint(WINDOW *win, const char str[])
 {
@@ -1096,9 +1209,9 @@ resize_for_menu_like(void)
 
 	update_statusbar_layout();
 
-	wrefresh(status_bar);
-	wrefresh(ruler_win);
-	wrefresh(input_win);
+	ui_refresh_win(status_bar);
+	ui_refresh_win(ruler_win);
+	ui_refresh_win(input_win);
 
 	return 0;
 }
@@ -1106,13 +1219,16 @@ resize_for_menu_like(void)
 void
 ui_setup_for_menu_like(void)
 {
-	scrollok(menu_win, FALSE);
-	curs_set(0);
-	werase(menu_win);
-	werase(status_bar);
-	werase(ruler_win);
-	wrefresh(status_bar);
-	wrefresh(ruler_win);
+	if(curr_stats.load_stage > 0)
+	{
+		scrollok(menu_win, FALSE);
+		curs_set(0);
+		werase(menu_win);
+		werase(status_bar);
+		werase(ruler_win);
+		ui_refresh_win(status_bar);
+		ui_refresh_win(ruler_win);
+	}
 }
 
 /* Query terminal size from the "device" and pass it to curses library. */
@@ -1139,6 +1255,12 @@ update_term_size(void)
 	{
 		resizeterm(ws.ws_row, ws.ws_col);
 	}
+#elif defined(__PDCURSES__)
+	if(is_termresized())
+	{
+		/* This resizes vifm to window size. */
+		resize_term(0, 0);
+	}
 #endif
 }
 
@@ -1221,7 +1343,7 @@ refresh_view_win(view_t *view)
 		return;
 	}
 
-	wrefresh(view->win);
+	ui_refresh_win(view->win);
 	refresh_bottom_lines();
 }
 
@@ -1237,6 +1359,7 @@ move_window(view_t *view, int horizontally, int first)
 		 * right/bottom). */
 		switch_panes_content();
 		go_to_other_pane();
+		tabs_switch_panes();
 	}
 }
 
@@ -1282,56 +1405,46 @@ ui_view_unpick(view_t *view, view_t *old_curr, view_t *old_other)
 static void
 switch_panes_content(void)
 {
-	view_t tmp_view;
-	WINDOW* tmp;
-	int t;
+	ui_swap_view_data(&lwin, &rwin);
 
-	tmp = lwin.win;
-	lwin.win = rwin.win;
-	rwin.win = tmp;
-
-	t = lwin.window_rows;
-	lwin.window_rows = rwin.window_rows;
-	rwin.window_rows = t;
-
-	t = lwin.window_cols;
-	lwin.window_cols = rwin.window_cols;
-	rwin.window_cols = t;
-
-	t = lwin.local_cs;
-	lwin.local_cs = rwin.local_cs;
-	rwin.local_cs = t;
-
-	tmp = lwin.title;
-	lwin.title = rwin.title;
-	rwin.title = tmp;
-
-	tmp_view = lwin;
-	lwin = rwin;
-	rwin = tmp_view;
-
-	update_origins(&lwin, &rwin.curr_dir[0]);
-	update_origins(&rwin, &lwin.curr_dir[0]);
+	flist_update_origins(&lwin, &rwin.curr_dir[0], &lwin.curr_dir[0]);
+	flist_update_origins(&rwin, &lwin.curr_dir[0], &rwin.curr_dir[0]);
 
 	view_panes_swapped();
 
 	curr_stats.need_update = UT_REDRAW;
 }
 
-/* Updates pointers to main (default) origins in file list entries. */
-static void
-update_origins(view_t *view, const char *old_main_origin)
+void
+ui_swap_view_data(view_t *left, view_t *right)
 {
-	char *const new_origin = &view->curr_dir[0];
-	int i;
-	for(i = 0; i < view->list_rows; ++i)
-	{
-		dir_entry_t *const entry = &view->dir_entry[i];
-		if(entry->origin == old_main_origin)
-		{
-			entry->origin = new_origin;
-		}
-	}
+	view_t tmp_view;
+	WINDOW *tmp;
+	int t;
+
+	tmp = left->win;
+	left->win = right->win;
+	right->win = tmp;
+
+	t = left->window_rows;
+	left->window_rows = right->window_rows;
+	right->window_rows = t;
+
+	t = left->window_cols;
+	left->window_cols = right->window_cols;
+	right->window_cols = t;
+
+	t = left->local_cs;
+	left->local_cs = right->local_cs;
+	right->local_cs = t;
+
+	tmp = left->title;
+	left->title = right->title;
+	right->title = tmp;
+
+	tmp_view = *left;
+	*left = *right;
+	*right = tmp_view;
 }
 
 void
@@ -1461,7 +1574,7 @@ ui_get_decors(const dir_entry_t *entry, const char **prefix,
 		((dir_entry_t *)entry)->name_dec_num = 0;
 		if(cfg.name_dec_count != 0)
 		{
-			char full_path[PATH_MAX];
+			char full_path[PATH_MAX + 1];
 			int i;
 
 			get_full_path_of(entry, sizeof(full_path) - 1U, full_path);
@@ -1499,6 +1612,24 @@ ui_get_decors(const dir_entry_t *entry, const char **prefix,
 	}
 }
 
+/* Gets real type of file view entry.  Returns type of entry, resolving symbolic
+ * link if needed. */
+static FileType
+ui_view_entry_target_type(const dir_entry_t *entry)
+{
+	if(entry->type == FT_LINK)
+	{
+		char *const full_path = format_str("%s/%s", entry->origin, entry->name);
+		const FileType type = (get_symlink_type(full_path) != SLT_UNKNOWN)
+		                    ? FT_DIR
+		                    : FT_LINK;
+		free(full_path);
+		return type;
+	}
+
+	return entry->type;
+}
+
 void
 checked_wmove(WINDOW *win, int y, int x)
 {
@@ -1512,14 +1643,14 @@ void
 ui_display_too_small_term_msg(void)
 {
 	touchwin(stdscr);
-	wrefresh(stdscr);
+	ui_refresh_win(stdscr);
 
 	mvwin(status_bar, 0, 0);
 	wresize(status_bar, getmaxy(stdscr), getmaxx(stdscr));
 	werase(status_bar);
 	waddstr(status_bar, "Terminal is too small for vifm");
 	touchwin(status_bar);
-	wrefresh(status_bar);
+	ui_refresh_win(status_bar);
 }
 
 void
@@ -1592,28 +1723,41 @@ ui_views_update_titles(void)
 void
 ui_view_title_update(view_t *view)
 {
-	char *title;
 	const int gen_view = vle_mode_is(VIEW_MODE) && !curr_view->explore_mode;
 	view_t *selected = gen_view ? other_view : curr_view;
+	path_func pf = cfg.shorten_title_paths ? &replace_home_part : &path_identity;
+	col_attr_t title_col;
 
-	if(curr_stats.load_stage < 2)
+	if(curr_stats.load_stage < 2 || !ui_view_is_visible(view))
 	{
 		return;
 	}
 
-	title = format_view_title(view,
-			cfg.shorten_title_paths ? &replace_home_part : &path_identity);
-
-	if(view == selected && cfg.set_title)
+	if(view == selected && cfg.set_title && !curr_stats.silent_ui)
 	{
-		term_title_update(title);
+		char *const term_title = format_view_title(view, pf);
+		term_title_update(term_title);
+		free(term_title);
 	}
 
-	print_view_title(view, view == selected, title);
+	title_col = fixup_titles_attributes(view, view == selected);
 
-	wnoutrefresh(view->title);
+	if(view_shows_tabline(view))
+	{
+		print_tab_title(view->title, view, title_col, pf);
+	}
+	else
+	{
+		char *const title = format_view_title(view, pf);
+		print_view_title(view, view == selected, title);
+		wnoutrefresh(view->title);
+		free(title);
+	}
 
-	free(title);
+	if(view == curr_view && get_tabline_height() > 0)
+	{
+		print_tab_title(tab_line, view, cfg.cs.color[TAB_LINE_COLOR], pf);
+	}
 }
 
 /* Identity path function.  Returns its argument. */
@@ -1623,6 +1767,172 @@ path_identity(const char path[])
 	return (char *)path;
 }
 
+/* Checks whether view displays list of pane tabs at the moment.  Returns
+ * non-zero if so, otherwise zero is returned. */
+static int
+view_shows_tabline(const view_t *view)
+{
+	return cfg.pane_tabs
+	    && !(curr_stats.preview.on && view == other_view)
+	    && cfg.show_tab_line != STL_NEVER
+	    && !(cfg.show_tab_line == STL_MULTIPLE && tabs_count(view) == 1);
+}
+
+/* Retrieves height of the tab line in lines.  Returns the height. */
+static int
+get_tabline_height(void)
+{
+	if(cfg.pane_tabs || cfg.show_tab_line == STL_NEVER)
+	{
+		return 0;
+	}
+
+	return (cfg.show_tab_line == STL_MULTIPLE && tabs_count(curr_view) == 1)
+	     ? 0
+	     : 1;
+}
+
+/* Prints title of the tab on specified curses window. */
+static void
+print_tab_title(WINDOW *win, view_t *view, col_attr_t base_col, path_func pf)
+{
+	int i;
+	tab_info_t tab_info;
+
+	const int max_width = getmaxx(win);
+	int width_used = 0;
+	int avg_width, spare_width;
+
+	wbkgdset(win, COLOR_PAIR(colmgr_get_pair(base_col.fg, base_col.bg)) |
+			(base_col.attr & A_REVERSE));
+	wattrset(win, base_col.attr & ~A_REVERSE);
+
+	werase(win);
+	checked_wmove(win, 0, 0);
+
+	compute_avg_width(&avg_width, &spare_width, max_width, view, pf);
+
+	for(i = 0; tabs_get(view, i, &tab_info); ++i)
+	{
+		char *title = make_tab_title(&tab_info, pf);
+		const int width_needed = utf8_strsw(title);
+		char buffer_dummy[1];
+		const int extra_width = snprintf(buffer_dummy, 0U, "[%d:]", i + 1);
+		int width = max_width;
+
+		col_attr_t col = base_col;
+		if(tab_info.view == view)
+		{
+			cs_mix_colors(&col, &cfg.cs.color[TAB_LINE_SEL_COLOR]);
+		}
+
+		if(tab_info.view != view)
+		{
+			width = tab_info.last ? (max_width - width_used)
+			                      : MIN(avg_width, extra_width + width_needed);
+		}
+
+		if(width < width_needed + extra_width)
+		{
+			char *ellipsed;
+
+			if(spare_width > 0)
+			{
+				width += 1;
+				--spare_width;
+			}
+
+			ellipsed = left_ellipsis(title, MAX(0, width - extra_width),
+					curr_stats.ellipsis);
+			free(title);
+			title = ellipsed;
+		}
+
+		wbkgdset(win, COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) |
+				(col.attr & A_REVERSE));
+		wattrset(win, col.attr & ~A_REVERSE);
+
+		if(width > extra_width)
+		{
+			wprintw(win, "[%d:%s]", i + 1, title);
+		}
+
+		/* Here result of `utf8_strsw(title)` might be different from one computed
+		 * above. */
+		width_used += extra_width + utf8_strsw(title);
+
+		free(title);
+	}
+
+	wnoutrefresh(win);
+}
+
+/* Computes average width of tab tips as well as number of spare character
+ * positions. */
+static void
+compute_avg_width(int *avg_width, int *spare_width, int max_width, view_t *view,
+		path_func pf)
+{
+	int left = max_width;
+	int widths[tabs_count(view)];
+	int i;
+	tab_info_t tab_info;
+
+	*avg_width = 0;
+	*spare_width = 0;
+
+	for(i = 0; tabs_get(view, i, &tab_info); ++i)
+	{
+		char *const title = make_tab_title(&tab_info, pf);
+		widths[i] = utf8_strsw(title) + snprintf(title, 0U, "[%d:]", i + 1);
+		free(title);
+
+		if(tab_info.view == view && tabs_count(view) != 1)
+		{
+			left = MAX(max_width - widths[i], 0);
+			*avg_width = left/(tabs_count(view) - 1);
+			*spare_width = left%(tabs_count(view) - 1);
+		}
+	}
+
+	int new_avg_width = *avg_width;
+	do
+	{
+		int well_used_width = 0;
+		int truncated_count = 0;
+		*avg_width = new_avg_width;
+		for(i = 0; tabs_get(view, i, &tab_info); ++i)
+		{
+			if(tab_info.view != view)
+			{
+				if(widths[i] <= *avg_width)
+				{
+					well_used_width += widths[i];
+				}
+				else
+				{
+					++truncated_count;
+				}
+			}
+		}
+		if(truncated_count == 0)
+		{
+			break;
+		}
+		new_avg_width = (left - well_used_width)/truncated_count;
+		*spare_width = (left - well_used_width)%truncated_count;
+	}
+	while(new_avg_width != *avg_width);
+}
+
+/* Gets title of the tab.  Returns newly allocated string. */
+static char *
+make_tab_title(const tab_info_t *tab_info, path_func pf)
+{
+	return (tab_info->name == NULL) ? format_view_title(tab_info->view, pf)
+	                                : strdup(tab_info->name);
+}
+
 /* Formats title for the view.  The pf function will be applied to full paths.
  * Returns newly allocated string, which should be freed by the caller, or NULL
  * if there is not enough memory. */
@@ -1631,7 +1941,7 @@ format_view_title(const view_t *view, path_func pf)
 {
 	if(view->explore_mode)
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		get_current_full_path(view, sizeof(full_path), full_path);
 		return strdup(pf(full_path));
 	}
@@ -1668,7 +1978,6 @@ print_view_title(const view_t *view, int active_view, char title[])
 		return;
 	}
 
-	fixup_titles_attributes(view, active_view);
 	werase(view->title);
 
 	ellipsis = active_view
@@ -1679,13 +1988,21 @@ print_view_title(const view_t *view, int active_view, char title[])
 	free(ellipsis);
 }
 
-/* Updates attributes for view titles and top line. */
-static void
+/* Updates attributes for view titles and top line.  Returns base color used for
+ * the title. */
+static col_attr_t
 fixup_titles_attributes(const view_t *view, int active_view)
 {
+	col_attr_t col = cfg.cs.color[TOP_LINE_COLOR];
+
+	if(view->title == NULL)
+	{
+		/* Workaround for tests. */
+		return col;
+	}
+
 	if(active_view)
 	{
-		col_attr_t col = cfg.cs.color[TOP_LINE_COLOR];
 		cs_mix_colors(&col, &cfg.cs.color[TOP_LINE_SEL_COLOR]);
 
 		wbkgdset(view->title, COLOR_PAIR(colmgr_get_pair(col.fg, col.bg)) |
@@ -1694,7 +2011,6 @@ fixup_titles_attributes(const view_t *view, int active_view)
 	}
 	else
 	{
-		col_attr_t col = cfg.cs.color[TOP_LINE_COLOR];
 		const int bg_attr = COLOR_PAIR(cfg.cs.pair[TOP_LINE_COLOR])
 		                  | (col.attr & A_REVERSE);
 
@@ -1704,6 +2020,8 @@ fixup_titles_attributes(const view_t *view, int active_view)
 		wattrset(top_line, col.attr & ~A_REVERSE);
 		werase(top_line);
 	}
+
+	return col;
 }
 
 int
@@ -1785,22 +2103,6 @@ ui_view_displays_columns(const view_t *view)
 	    || is_forced_list_mode(view);
 }
 
-FileType
-ui_view_entry_target_type(const dir_entry_t *entry)
-{
-	if(entry->type == FT_LINK)
-	{
-		char *const full_path = format_str("%s/%s", entry->origin, entry->name);
-		const FileType type = (get_symlink_type(full_path) != SLT_UNKNOWN)
-		                    ? FT_DIR
-		                    : FT_LINK;
-		free(full_path);
-		return type;
-	}
-
-	return entry->type;
-}
-
 int
 ui_view_available_width(const view_t *view)
 {
@@ -1909,7 +2211,7 @@ ui_view_wipe(view_t *view)
 		mvwaddstr(view->win, i, 0, line_filler);
 	}
 	redrawwin(view->win);
-	wrefresh(view->win);
+	ui_refresh_win(view->win);
 }
 
 int
