@@ -326,7 +326,7 @@ struct view_t
 	WINDOW *title;
 
 	/* Directory we're currently in. */
-	char curr_dir[PATH_MAX];
+	char curr_dir[PATH_MAX + 1];
 
 	/* Data related to custom filling. */
 	struct cv_data_t custom;
@@ -344,9 +344,9 @@ struct view_t
 
 	/* Monitor that checks for directory changes. */
 	fswatch_t *watch;
-	char watched_dir[PATH_MAX];
+	char watched_dir[PATH_MAX + 1];
 
-	char last_dir[PATH_MAX];
+	char last_dir[PATH_MAX + 1];
 
 	/* Number of files that match current search pattern. */
 	int matches;
@@ -444,6 +444,8 @@ struct view_t
 	uint64_t last_reload; /* Time of last [full] reload. */
 
 	int on_slow_fs; /* Whether current directory has access penalties. */
+	int has_dups;   /* Whether current directory has duplicated file entries (FS
+	                   issue). */
 };
 
 extern view_t lwin;
@@ -461,6 +463,7 @@ WINDOW *sort_win;
 WINDOW *change_win;
 WINDOW *error_win;
 WINDOW *top_line;
+WINDOW *tab_line;
 
 WINDOW *lborder;
 WINDOW *mborder;
@@ -483,6 +486,11 @@ int ui_char_pressed(wint_t c);
 
 int setup_ncurses_interface(void);
 
+/* Closes current tab if it's not the last one, closes whole application
+ * otherwise.  Might also fail if background tasks are present and user chooses
+ * not to stop them. */
+void ui_quit(int write_info, int force);
+
 /* Checks whether custom view of specified type is unsorted.  Returns non-zero
  * if so, otherwise zero is returned. */
 int cv_unsorted(CVType type);
@@ -527,6 +535,10 @@ void redraw_lists(void);
 /* Forces immediate update of attributes for most of windows. */
 void update_attributes(void);
 
+/* Refreshes the window, should be used instead of wrefresh(). */
+#pragma GCC poison wrefresh
+void ui_refresh_win(WINDOW *win);
+
 /* Prints str in current window position. */
 void wprint(WINDOW *win, const char str[]);
 
@@ -538,7 +550,7 @@ void wprinta(WINDOW *win, const char str[], int line_attrs);
  * on success, and non-zero otherwise. */
 int resize_for_menu_like(void);
 
-/* Performs updates of layout for manu like modes. */
+/* Performs updates of layout for menu like modes. */
 void ui_setup_for_menu_like(void);
 
 /* Performs real pane redraw in the TUI and maybe some related operations. */
@@ -551,8 +563,12 @@ void move_window(view_t *view, int horizontally, int first);
 /* Swaps current and other views. */
 void switch_panes(void);
 
-/* Setups view to the the curr_view.  Saving previous state in supplied
- * buffers.  Use ui_view_unpick() to revert the effect. */
+/* Swaps data fields of two panes.  Doesn't correct origins of directory
+ * entries. */
+void ui_swap_view_data(view_t *left, view_t *right);
+
+/* Setups view to the the curr_view.  Saving previous state in supplied buffers.
+ * Use ui_view_unpick() to revert the effect. */
 void ui_view_pick(view_t *view, view_t **old_curr, view_t **old_other);
 
 /* Restores what has been done by ui_view_pick(). */
@@ -638,10 +654,6 @@ int ui_view_is_visible(const view_t *view);
  * zero is returned. */
 int ui_view_displays_columns(const view_t *view);
 
-/* Gets real type of file view entry.  Returns type of entry, resolving symbolic
- * link if needed. */
-FileType ui_view_entry_target_type(const dir_entry_t *entry);
-
 /* Gets width of part of the view that is available for file list.  Returns the
  * width. */
 int ui_view_available_width(const view_t *view);
