@@ -136,6 +136,10 @@ char ** list_sorted_files(const char path[], int *len);
  * regular file. */
 int is_regular_file(const char path[]);
 
+/* Checks whether file path points to a regular file (not a symbolic link or
+ * anything else).  Returns non-zero if so, otherwise zero is returned. */
+int is_regular_file_noderef(const char path[]);
+
 /* Renames file specified by the src argument to the path specified by the dst
  * argument.  Overwrites destination file if it exists.  Returns non-zero on
  * error, otherwise zero is returned. */
@@ -161,8 +165,10 @@ int entry_is_dir(const char full_path[], const struct dirent* dentry);
 int is_dirent_targets_dir(const char full_path[], const struct dirent *d);
 
 /* Checks that entity pointed to by the path is located under the root
- * directory.  Returns non-zero if so, otherwise zero is returned. */
-int is_in_subtree(const char path[], const char root[]);
+ * directory.  Non-zero include_root parameter makes root being considered to be
+ * part of the subtree that it defines.  Returns non-zero if so, otherwise zero
+ * is returned. */
+int is_in_subtree(const char path[], const char root[], int include_root);
 
 /* Checks whether to paths belong to the same file system.  Returns non-zero if
  * so, otherwise zero is returned. */
