@@ -143,7 +143,7 @@ canonicalize_path(const char directory[], char buf[], size_t buf_size)
 		}
 		else if(prev_dir_present &&
 				(strncmp(p, "../", 3) == 0 || strcmp(p, "..") == 0) &&
-				!ends_with(buf, "../"))
+				!(ends_with(buf, "/../") || strcmp(buf, "../") == 0))
 		{
 			/* Remove the last path component added. */
 #ifdef _WIN32
@@ -243,7 +243,7 @@ skip_dotdir_if_any(const char *path[], int has_parent)
 const char *
 make_rel_path(const char path[], const char base[])
 {
-	static char buf[PATH_MAX];
+	static char buf[PATH_MAX + 1];
 
 	const char *p = path, *b = base;
 	int i;
@@ -343,7 +343,7 @@ is_unc_root(const char *path)
 }
 
 char *
-shell_like_escape(const char string[], int internal)
+shell_like_escape(const char string[], int type)
 {
 	size_t len;
 	size_t i;
@@ -364,7 +364,7 @@ shell_like_escape(const char string[], int internal)
 		switch(*string)
 		{
 			case '%':
-				if(internal)
+				if(type == 1)
 				{
 					*dup++ = '%';
 				}
@@ -398,7 +398,7 @@ shell_like_escape(const char string[], int internal)
 				break;
 
 			case '\n':
-				if(internal)
+				if(type != 0)
 				{
 					break;
 				}
@@ -410,6 +410,7 @@ shell_like_escape(const char string[], int internal)
 
 			/* Escape the following characters only at the beginning of the line. */
 			case '~':
+			case '=': /* Command-path expansion in zsh. */
 				if(dup == ret)
 					*dup++ = '\\';
 				break;
@@ -434,7 +435,7 @@ replace_home_part(const char path[])
 char *
 replace_home_part_strict(const char path[])
 {
-	static char buf[PATH_MAX];
+	static char buf[PATH_MAX + 1];
 	size_t len;
 
 	len = strlen(cfg.home_dir) - 1;
@@ -612,7 +613,7 @@ to_canonic_path(const char path[], const char base[], char buf[],
 {
 	if(!is_path_absolute(path))
 	{
-		char full_path[PATH_MAX];
+		char full_path[PATH_MAX + 1];
 		/* Assert is not level above to keep "." in curr_dir in tests, but this
 		 * should be possible to change. */
 		assert(is_path_absolute(base) && "Base path has to be absolute.");
@@ -754,7 +755,7 @@ find_cmd_in_path(const char cmd[], size_t path_len, char path[])
 	paths = get_paths(&paths_count);
 	for(i = 0; i < paths_count; i++)
 	{
-		char tmp_path[PATH_MAX];
+		char tmp_path[PATH_MAX + 1];
 		snprintf(tmp_path, sizeof(tmp_path), "%s/%s", paths[i], cmd);
 
 		/* Need to check for executable, not just a file, as this additionally
@@ -775,9 +776,7 @@ void
 generate_tmp_file_name(const char prefix[], char buf[], size_t buf_len)
 {
 	snprintf(buf, buf_len, "%s/%s", get_tmpdir(), prefix);
-#ifdef _WIN32
-	to_forward_slash(buf);
-#endif
+	system_to_internal_slashes(buf);
 	copy_str(buf, buf_len, make_name_unique(buf));
 }
 
@@ -803,13 +802,13 @@ is_unc_path(const char path[])
 }
 
 void
-to_forward_slash(char path[])
+system_to_internal_slashes(char path[])
 {
 	replace_char(path, '\\', '/');
 }
 
 void
-to_back_slash(char path[])
+internal_to_system_slashes(char path[])
 {
 	replace_char(path, '/', '\\');
 }
