@@ -26,8 +26,8 @@
 #include <stdio.h> /* snprintf() */
 #include <stdlib.h> /* free() malloc() mbstowcs() memmove() memset() realloc()
                        strtol() wcstombs() */
-#include <string.h> /* strdup() strncmp() strlen() strcmp() strchr() strrchr()
-                       strncpy() */
+#include <string.h> /* memcpy() strdup() strncmp() strlen() strcmp() strchr()
+                       strrchr() strncpy() strcspn() strspn() */
 #include <wchar.h> /* wint_t vswprintf() */
 #include <wctype.h> /* iswprint() iswupper() towlower() towupper() */
 
@@ -44,6 +44,8 @@ static int transform_wide_str(const char str[], wint_t (*f)(wint_t), char buf[],
 TSTATIC void squash_double_commas(char str[]);
 static char * ellipsis(const char str[], size_t max_width, const char ell[],
 		int right);
+static size_t copy_substr(char dst[], size_t dst_len, const char src[],
+		char terminator);
 
 void
 chomp(char str[])
@@ -364,6 +366,12 @@ strnoscmp(const char *s, const char *t, size_t n)
 #endif
 }
 
+int
+strossorter(const void *s, const void *t)
+{
+	return stroscmp(*(const char **)s, *(const char **)t);
+}
+
 char *
 after_last(const char *str, char c)
 {
@@ -457,7 +465,7 @@ strprepend(char **str, size_t *len, const char prefix[])
 	{
 		memmove(new + prefix_len, new, *len + 1);
 	}
-	strncpy(new, prefix, prefix_len);
+	memcpy(new, prefix, prefix_len);
 	*str = new;
 	*len += prefix_len;
 
@@ -584,16 +592,17 @@ ellipsis(const char str[], size_t max_width, const char ell[], int right)
 }
 
 char *
-break_in_two(char str[], size_t max)
+break_in_two(char str[], size_t max, const char separator[])
 {
 	int i;
 	size_t len, size;
 	char *result;
-	char *break_point = strstr(str, "%=");
+	char *break_point = strstr(str, separator);
 	if(break_point == NULL)
 		return str;
 
-	len = utf8_strsw(str) - 2;
+	const size_t separator_len = strlen(separator);
+	len = utf8_strsw(str) - separator_len;
 	size = strlen(str);
 	size = MAX(size, max);
 	result = malloc(size*4 + 2);
@@ -616,8 +625,8 @@ break_in_two(char str[], size_t max)
 	result[i] = '\0';
 
 	if(len > max)
-		break_point = strstr(str, "%=");
-	strcat(result, break_point + 2);
+		break_point = strstr(str, separator);
+	strcat(result, break_point + separator_len);
 
 	free(str);
 	return result;
@@ -643,17 +652,17 @@ vifm_swprintf(wchar_t str[], size_t len, const wchar_t format[], ...)
 }
 
 const char *
-extract_part(const char str[], char separator, char part_buf[])
+extract_part(const char str[], const char separators[], char part_buf[])
 {
 	const char *end = NULL;
-	str = skip_char(str, separator);
+	str += strspn(str, separators);
 	if(str[0] != '\0')
 	{
-		end = until_first(str, separator);
+		end = str + strcspn(str, separators);
 		copy_str(part_buf, end - str + 1, str);
 		if(*end != '\0')
 		{
-			end++;
+			++end;
 		}
 	}
 	return end;
@@ -833,7 +842,12 @@ copy_str(char dst[], size_t dst_len, const char src[])
 	return (dst == src) ? 0U : copy_substr(dst, dst_len, src, '\0');
 }
 
-size_t
+/* Copies characters from the string pointed to by src and terminated by the
+ * terminator to piece of memory of size dst_len pointed to by dst.  Ensures
+ * that copied string ends with null character.  Does nothing for zero
+ * dst_len.  Returns number of characters written, including terminating null
+ * character. */
+static size_t
 copy_substr(char dst[], size_t dst_len, const char src[], char terminator)
 {
 	char *past_end;
@@ -866,6 +880,17 @@ str_to_int(const char str[])
 	     : ((number == LONG_MIN) ? INT_MIN : number);
 }
 
+int
+read_int(const char line[], int *i)
+{
+	char *endptr;
+	const long l = strtol(line, &endptr, 10);
+
+	*i = (l > INT_MAX) ? INT_MAX : ((l < INT_MIN) ? INT_MIN : l);
+
+	return *line != '\0' && *endptr == '\0';
+}
+
 void
 replace_char(char str[], char from, char to)
 {
