@@ -35,8 +35,8 @@
 #include <math.h> /* modf() pow() */
 #include <stddef.h> /* size_t */
 #include <stdio.h> /* snprintf() */
-#include <stdlib.h> /* free() malloc() */
-#include <string.h> /* strdup() strchr() strlen() strpbrk() strtol() */
+#include <stdlib.h> /* free() malloc() qsort() */
+#include <string.h> /* memcpy() strdup() strchr() strlen() strpbrk() strtol() */
 #include <wchar.h> /* wcwidth() */
 
 #include "../cfg/config.h"
@@ -149,7 +149,7 @@ show_progress_cb(const void *descr)
 int
 vifm_chdir(const char path[])
 {
-	char curr_path[PATH_MAX];
+	char curr_path[PATH_MAX + 1];
 	if(get_cwd(curr_path, sizeof(curr_path)) == curr_path)
 	{
 		if(stroscmp(curr_path, path) == 0)
@@ -196,7 +196,7 @@ expand_envvars(const char str[], int escape_vals)
 				char *escaped_var_value = NULL;
 				if(escape_vals)
 				{
-					escaped_var_value = shell_like_escape(var_value, 1);
+					escaped_var_value = shell_like_escape(var_value, 2);
 					var_value = escaped_var_value;
 				}
 
@@ -341,7 +341,7 @@ enclose_in_dquotes(const char str[])
 const char *
 make_name_unique(const char filename[])
 {
-	static char unique[PATH_MAX];
+	static char unique[PATH_MAX + 1];
 	size_t len;
 	int i;
 
@@ -460,7 +460,7 @@ escape_for_squotes(const char string[], size_t offset)
 
 	/* Copy prefix not escaping it. */
 	offset = MIN(len, offset);
-	strncpy(out, string, offset);
+	memcpy(out, string, offset);
 	out += offset;
 	string += offset;
 
@@ -492,7 +492,7 @@ escape_for_dquotes(const char string[], size_t offset)
 
 	/* Copy prefix not escaping it. */
 	offset = MIN(len, offset);
-	strncpy(out, string, offset);
+	memcpy(out, string, offset);
 	out += offset;
 	string += offset;
 
@@ -615,13 +615,23 @@ def_count(int count)
 	return (count == NO_COUNT_GIVEN) ? 1 : count;
 }
 
+char *
+parse_line_for_path(const char line[], const char cwd[])
+{
+	int line_num;
+	/* Skip empty lines. */
+	return (skip_whitespace(line)[0] == '\0')
+	     ? NULL
+	     : parse_file_spec(line, &line_num, cwd);
+}
+
 char *
 parse_file_spec(const char spec[], int *line_num, const char cwd[])
 {
 	char *path_buf;
 	const char *colon;
 	const size_t bufs_len = strlen(cwd) + 1U + strlen(spec) + 1U + 1U;
-	char canonicalized[PATH_MAX];
+	char canonicalized[PATH_MAX + 1];
 
 	path_buf = malloc(bufs_len);
 	if(path_buf == NULL)
@@ -682,9 +692,7 @@ parse_file_spec(const char spec[], int *line_num, const char cwd[])
 	chomp(path_buf);
 	canonicalize_path(path_buf, canonicalized, sizeof(canonicalized));
 
-#ifdef _WIN32
-	to_forward_slash(canonicalized);
-#endif
+	system_to_internal_slashes(canonicalized);
 
 	if(!ends_with_slash(path_buf) && !is_root_dir(canonicalized) &&
 			strcmp(canonicalized, "./") != 0)
@@ -716,5 +724,17 @@ is_graphical_viewer(const char viewer[])
 	return (strstr(viewer, "%px") != NULL && strstr(viewer, "%py") != NULL);
 }
 
+void
+safe_qsort(void *base, size_t nmemb, size_t size,
+		int (*compar)(const void *, const void *))
+{
+	if(nmemb != 0U)
+	{
+		/* Even when there are no entries, qsort() shouldn't be called with a NULL
+		 * parameter.  It isn't allowed by the standard. */
+		qsort(base, nmemb, size, compar);
+	}
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
