@@ -58,8 +58,9 @@ void recover_after_shellout(void);
 /* Invokes handler for each line read from stdout of the command specified via
  * cmd.  Error stream is displayed separately.  Implements heuristic according
  * to which if command output includes null character, it's taken as a separator
- * instead of regular newline characters.  Supports cancellation.  Returns zero
- * on success, otherwise non-zero is returned. */
+ * instead of regular newline characters.  Supports cancellation.  Ignores exit
+ * code of the command and succeeds even if it doesn't exist.  Returns zero on
+ * success, otherwise non-zero is returned. */
 int process_cmd_output(const char descr[], const char cmd[], int user_sh,
 		int interactive, cmd_output_handler handler, void *arg);
 
@@ -151,6 +152,16 @@ int def_count(int count);
  * non-zero if it's likely, otherwise zero is returned. */
 int is_graphical_viewer(const char viewer[]);
 
+/* Wrapper around qsort() that allows base to be NULL when nmemb is 0 by
+ * skipping call to qsort(). */
+void safe_qsort(void *base, size_t nmemb, size_t size,
+		int (*compar)(const void *, const void *));
+
+/* Checks line for path in it.  Ignores empty lines and attempts to parse it as
+ * location line (path followed by a colon and optional line and column
+ * numbers).  Returns canonicalized path as a newly allocated string or NULL. */
+char * parse_line_for_path(const char line[], const char cwd[]);
+
 /* Extracts path and line number from the spec (default line number is 1).
  * Returns path in as newly allocated string and sets *line_num to line number,
  * otherwise NULL is returned. */
@@ -238,16 +249,15 @@ int reopen_term_stdin(void);
 
 /* Executes the command via shell and opens its output for reading.  Returns
  * NULL on error, otherwise stream valid for reading is returned. */
-FILE * read_cmd_output(const char cmd[]);
+FILE * read_cmd_output(const char cmd[], int preserve_stdin);
 
 /* Gets path to directory where files bundled with Vifm are stored.  Returns
  * pointer to a statically allocated buffer. */
 const char * get_installed_data_dir(void);
 
-/* Clones timestamps from file specified by from to file at path.  st is a hint
+/* Clones attributes from file specified by from to file at path.  st is a hint
  * to omit extra file system requests if possible, can be NULL on Windows. */
-void clone_timestamps(const char path[], const char from[],
-		const struct stat *st);
+void clone_attribs(const char path[], const char from[], const struct stat *st);
 
 /* Retrives amount of free space at location specified by the path.  Returns the
  * amount in bytes. */
