@@ -36,8 +36,8 @@
 #include <grp.h> /* getgrnam() getgrgid_r() */
 #include <pthread.h> /* pthread_sigmask() */
 #include <pwd.h> /* getpwnam() getpwuid_r() */
-#include <unistd.h> /* X_OK dup() dup2() getpid() isatty() pause() sysconf()
-                       ttyname() */
+#include <unistd.h> /* X_OK chown() dup() dup2() getpid() isatty() pause()
+                       sysconf() ttyname() */
 
 #include <assert.h> /* assert() */
 #include <ctype.h> /* isdigit() */
@@ -89,7 +89,7 @@ get_mount_point_traverser_state;
 
 static int get_mount_info_traverser(struct mntent *entry, void *arg);
 static void free_mnt_entries(struct mntent *entries, unsigned int nentries);
-struct mntent * read_mnt_entries(unsigned int *nentries);
+static struct mntent * read_mnt_entries(unsigned int *nentries);
 static int clone_mnt_entry(struct mntent *lhs, const struct mntent *rhs);
 static void free_mnt_entry(struct mntent *entry);
 static int starts_with_list_item(const char str[], const char list[]);
@@ -233,10 +233,8 @@ get_proc_exit_status(pid_t pid)
 }
 
 void _gnuc_noreturn
-run_from_fork(int pipe[2], int err_only, char cmd[])
+run_from_fork(int pipe[2], int err_only, int preserve_stdin, char cmd[])
 {
-	int nullfd;
-
 	/* Close read end of the pipe. */
 	(void)close(pipe[0]);
 
@@ -250,24 +248,31 @@ run_from_fork(int pipe[2], int err_only, char cmd[])
 		_Exit(EXIT_FAILURE);
 	}
 
-	/* Close write end of the pipe (already duplicated it). */
-	(void)close(pipe[1]);
+	if(pipe[1] != STDERR_FILENO && pipe[1] != STDOUT_FILENO)
+	{
+		/* Close write end of the pipe after it was duplicated. */
+		(void)close(pipe[1]);
+	}
 
-	/* Send stdin and maybe stdout to /dev/null */
-	nullfd = open("/dev/null", O_RDWR);
-	if(nullfd == -1)
+	const int null_fd = open("/dev/null", O_RDWR);
+	if(null_fd == -1)
 	{
 		_Exit(EXIT_FAILURE);
 	}
 
-	/* if(dup2(nullfd, STDIN_FILENO) == -1) */
-	/* { */
-	/* 	_Exit(EXIT_FAILURE); */
-	/* } */
-	if(err_only && dup2(nullfd, STDOUT_FILENO) == -1)
+	if(!preserve_stdin && dup2(null_fd, STDIN_FILENO) == -1)
 	{
 		_Exit(EXIT_FAILURE);
 	}
+	if(err_only && dup2(null_fd, STDOUT_FILENO) == -1)
+	{
+		_Exit(EXIT_FAILURE);
+	}
+
+	if(null_fd != STDIN_FILENO && null_fd != STDOUT_FILENO)
+	{
+		(void)close(null_fd);
+	}
 
 	execvp(get_execv_path(cfg.shell), make_execv_array(cfg.shell, cmd));
 	_Exit(127);
@@ -412,7 +417,7 @@ refers_to_slower_fs(const char from[], const char to[])
 int
 is_on_slow_fs(const char full_path[], const char slowfs_specs[])
 {
-	char fs_name[PATH_MAX];
+	char fs_name[PATH_MAX + 1];
 	get_mount_point_traverser_state state = {
 		.type = MI_FS_TYPE,
 		.path = full_path,
@@ -518,7 +523,10 @@ traverse_mount_points(mptraverser client, void *arg)
 
 	for(i = 0; i < nentries; ++i)
 	{
-		client(&entries[i], arg);
+		if(client(&entries[i], arg))
+		{
+			break;
+		}
 	}
 
 	return 0;
@@ -540,7 +548,7 @@ free_mnt_entries(struct mntent *entries, unsigned int nentries)
 
 /* Reads in array of mount entries.  Always sets *nentries.  Returns the array,
  * which might be NULL if empty.  On memory allocation error, skips entries. */
-struct mntent *
+static struct mntent *
 read_mnt_entries(unsigned int *nentries)
 {
 	FILE *f;
@@ -961,7 +969,7 @@ open_tty(void)
 }
 
 FILE *
-read_cmd_output(const char cmd[])
+read_cmd_output(const char cmd[], int preserve_stdin)
 {
 	FILE *fp;
 	pid_t pid;
@@ -980,7 +988,7 @@ read_cmd_output(const char cmd[])
 
 	if(pid == 0)
 	{
-		run_from_fork(out_pipe, 0, (char *)cmd);
+		run_from_fork(out_pipe, 0, preserve_stdin, (char *)cmd);
 		return NULL;
 	}
 
@@ -1002,9 +1010,9 @@ get_installed_data_dir(void)
 }
 
 void
-clone_timestamps(const char path[], const char from[], const struct stat *st)
+clone_attribs(const char path[], const char from[], const struct stat *st)
 {
-	struct timeval tv[2];
+	chown(path, st->st_uid, st->st_gid);
 
 #if defined(HAVE_STRUCT_STAT_ST_MTIM) && defined(HAVE_FUTIMENS)
 	const int fd = open(path, O_WRONLY);
@@ -1021,6 +1029,7 @@ clone_timestamps(const char path[], const char from[], const struct stat *st)
 	}
 #endif
 
+	struct timeval tv[2];
 #ifdef HAVE_STRUCT_STAT_ST_MTIM
 	tv[0].tv_sec = st->st_atim.tv_sec;
 	tv[0].tv_usec = st->st_atim.tv_nsec/1000;
