@@ -49,6 +49,7 @@
 #include "macros.h"
 #include "path.h"
 #include "str.h"
+#include "test_helpers.h"
 #include "utf8.h"
 
 #define PE_HDR_SIGNATURE 0x00004550U
@@ -58,11 +59,13 @@
 
 static const char PATHEXT_EXT_DEF[] = ".bat;.exe;.com";
 
-static int should_wait_for_program(const char cmd[]);
+TSTATIC int should_wait_for_program(const char cmd[]);
 static DWORD handle_process(const char cmd[], HANDLE proc, int *got_exit_code);
 static int get_subsystem(const char filename[]);
 static int get_stream_subsystem(FILE *fp);
-static FILE * read_cmd_output_internal(const char cmd[], int out_pipe[2]);
+static FILE * read_cmd_output_internal(const char cmd[], int out_pipe[2],
+		int preserve_stdin);
+static int get_set_owner_privilege(void);
 static char * get_root_path(const char path[]);
 static BOOL CALLBACK close_app_enum(HWND hwnd, LPARAM lParam);
 
@@ -104,7 +107,7 @@ run_in_shell_no_cls(char command[])
 void
 recover_after_shellout(void)
 {
-	if(curr_stats.load_stage != 0)
+	if(curr_stats.load_stage > 0)
 	{
 		reset_prog_mode();
 		resize_term(cfg.lines, cfg.columns);
@@ -155,7 +158,7 @@ wcwidth(wchar_t c)
 }
 
 int
-win_exec_cmd(char cmd[], int *const returned_exit_code)
+win_exec_cmd(char cmd[], int *returned_exit_code)
 {
 	wchar_t *utf16_cmd;
 	BOOL ret;
@@ -254,13 +257,14 @@ handle_process(const char cmd[], HANDLE proc, int *got_exit_code)
 /* Checks whether execution should be paused until command is finished.  Returns
  * non-zero when such synchronization is required, otherwise zero is
  * returned. */
-static int
+TSTATIC int
 should_wait_for_program(const char cmd[])
 {
 	char name[NAME_MAX + 1];
-	char path[PATH_MAX];
+	char path[PATH_MAX + 1];
 
 	(void)extract_cmd_name(cmd, 0, sizeof(name), name);
+	system_to_internal_slashes(name);
 
 	if(get_cmd_path(name, sizeof(path), path) == 0)
 	{
@@ -359,7 +363,7 @@ is_win_executable(const char name[])
 	strtoupper(name_buf);
 
 	p = env_get_def("PATHEXT", PATHEXT_EXT_DEF);
-	while((p = extract_part(p, ';', ext_buf)) != NULL)
+	while((p = extract_part(p, ";", ext_buf)) != NULL)
 	{
 		strtoupper(ext_buf);
 		if(ends_with(name_buf, ext_buf))
@@ -457,7 +461,10 @@ traverse_mount_points(mptraverser client, void *arg)
 				.mnt_dir = drive,
 				.mnt_type = "",
 			};
-			client(&entry, arg);
+			if(client(&entry, arg))
+			{
+				break;
+			}
 		}
 	}
 
@@ -480,7 +487,7 @@ executable_exists(const char path[])
 	pos = strlen(path_buf);
 
 	p = env_get_def("PATHEXT", PATHEXT_EXT_DEF);
-	while((p = extract_part(p, ';', path_buf + pos)) != NULL)
+	while((p = extract_part(p, ";", path_buf + pos)) != NULL)
 	{
 		if(path_exists(path_buf, DEREF) && !is_dir(path_buf))
 		{
@@ -498,7 +505,7 @@ get_exe_dir(char dir_buf[], size_t dir_buf_len)
 		return 1;
 	}
 
-	to_forward_slash(dir_buf);
+	system_to_internal_slashes(dir_buf);
 	break_atr(dir_buf, '/');
 	return 0;
 }
@@ -668,9 +675,9 @@ reopen_term_stdin(void)
 }
 
 FILE *
-read_cmd_output(const char cmd[])
+read_cmd_output(const char cmd[], int preserve_stdin)
 {
-	int out_fd, err_fd;
+	int in_fd, out_fd, err_fd;
 	int out_pipe[2];
 	FILE *result;
 
@@ -679,39 +686,58 @@ read_cmd_output(const char cmd[])
 		return NULL;
 	}
 
+	in_fd = dup(_fileno(stdin));
 	out_fd = dup(_fileno(stdout));
 	err_fd = dup(_fileno(stderr));
 
-	result = read_cmd_output_internal(cmd, out_pipe);
+	result = read_cmd_output_internal(cmd, out_pipe, preserve_stdin);
 
+	_dup2(in_fd, _fileno(stdin));
 	_dup2(out_fd, _fileno(stdout));
 	_dup2(err_fd, _fileno(stderr));
+	close(in_fd);
+	close(out_fd);
+	close(err_fd);
 
 	if(result == NULL)
+	{
 		close(out_pipe[0]);
+	}
 	close(out_pipe[1]);
 
 	return result;
 }
 
-const char *
-get_installed_data_dir(void)
-{
-	static char data_dir[PATH_MAX];
-	if(data_dir[0] == '\0')
-	{
-		char exe_dir[PATH_MAX];
-		(void)get_exe_dir(exe_dir, sizeof(exe_dir));
-		snprintf(data_dir, sizeof(data_dir), "%s/data", exe_dir);
-	}
-	return data_dir;
-}
-
+/* Performs redirection and execution of the command.  Returns file descriptor
+ * bound to stdout of the command. */
 static FILE *
-read_cmd_output_internal(const char cmd[], int out_pipe[2])
+read_cmd_output_internal(const char cmd[], int out_pipe[2], int preserve_stdin)
 {
-	char *args[4];
-	int retcode;
+	if(!preserve_stdin)
+	{
+		HANDLE h = CreateFileA("\\\\.\\NUL", GENERIC_READ, 0, NULL, OPEN_EXISTING,
+				FILE_ATTRIBUTE_NORMAL, NULL);
+		if(h == INVALID_HANDLE_VALUE)
+		{
+			return NULL;
+		}
+
+		const int fd = _open_osfhandle((intptr_t)h, _O_RDWR);
+		if(fd == -1)
+		{
+			CloseHandle(h);
+			return NULL;
+		}
+
+		if(_dup2(fd, _fileno(stdin)) != 0)
+		{
+			return NULL;
+		}
+		if(fd != _fileno(stdin))
+		{
+			close(fd);
+		}
+	}
 
 	if(_dup2(out_pipe[1], _fileno(stdout)) != 0)
 	{
@@ -722,21 +748,30 @@ read_cmd_output_internal(const char cmd[], int out_pipe[2])
 		return NULL;
 	}
 
-	args[0] = "cmd";
-	args[1] = "/C";
-	args[2] = (char *)cmd;
-	args[3] = NULL;
+	const char *args[] = { "cmd", "/C", cmd, NULL };
+	const int retcode = _spawnvp(P_NOWAIT, args[0], (const char **)args);
 
-	retcode = _spawnvp(P_NOWAIT, args[0], (const char **)args);
+	return (retcode == 0 ? NULL : _fdopen(out_pipe[0], "r"));
+}
 
-	return (retcode == 0) ? NULL : _fdopen(out_pipe[0], "r");
+const char *
+get_installed_data_dir(void)
+{
+	static char data_dir[PATH_MAX + 1];
+	if(data_dir[0] == '\0')
+	{
+		char exe_dir[PATH_MAX + 1];
+		(void)get_exe_dir(exe_dir, sizeof(exe_dir));
+		snprintf(data_dir, sizeof(data_dir), "%s/data", exe_dir);
+	}
+	return data_dir;
 }
 
 FILE *
 win_tmpfile()
 {
-	char dir[PATH_MAX];
-	char file[PATH_MAX];
+	char dir[PATH_MAX + 1];
+	char file[PATH_MAX + 1];
 	HANDLE h;
 	int fd;
 	FILE *f;
@@ -775,11 +810,29 @@ win_tmpfile()
 }
 
 void
-clone_timestamps(const char path[], const char from[], const struct stat *st)
+clone_attribs(const char path[], const char from[], const struct stat *st)
 {
 	wchar_t *const utf16_path = utf8_to_utf16(path);
 	wchar_t *const utf16_from = utf8_to_utf16(from);
 
+	if(get_set_owner_privilege())
+	{
+		SECURITY_INFORMATION info = GROUP_SECURITY_INFORMATION
+		                          | OWNER_SECURITY_INFORMATION;
+		DWORD size_needed = 0U;
+		(void)GetFileSecurityW(utf16_from, info, NULL, 0, &size_needed);
+		if(size_needed != 0U)
+		{
+			char sec_descr[size_needed];
+			PSECURITY_DESCRIPTOR descr = (void *)&sec_descr;
+			if(GetFileSecurityW(utf16_from, info, descr, size_needed,
+						&size_needed) != FALSE)
+			{
+				SetFileSecurityW(utf16_path, info, descr);
+			}
+		}
+	}
+
 	HANDLE hfrom = CreateFileW(utf16_from, GENERIC_READ,
 			FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
 			FILE_ATTRIBUTE_NORMAL, NULL);
@@ -810,6 +863,46 @@ clone_timestamps(const char path[], const char from[], const struct stat *st)
 	free(utf16_path);
 }
 
+/* Requests privilege required to set file owner/group.  Returns non-zero if it
+ * was granted and zero otherwise. */
+static int
+get_set_owner_privilege(void)
+{
+	static int granted = -1;
+	if(granted >= 0)
+	{
+		return granted;
+	}
+
+	HANDLE token;
+	if(!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES, &token))
+	{
+		return (granted = 0);
+	}
+
+	LUID luid;
+	if(!LookupPrivilegeValue(NULL, "SeRestorePrivilege", &luid))
+	{
+		CloseHandle(token);
+		return (granted = 0);
+	}
+
+	TOKEN_PRIVILEGES tp;
+	tp.PrivilegeCount = 1;
+	tp.Privileges[0].Luid = luid;
+	tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
+
+	if(!AdjustTokenPrivileges(token, FALSE, &tp, sizeof(tp), NULL, NULL))
+	{
+		CloseHandle(token);
+		return (granted = 0);
+	}
+
+	CloseHandle(token);
+	granted = (GetLastError() != ERROR_NOT_ALL_ASSIGNED);
+	return granted;
+}
+
 uint64_t
 get_free_space(const char at[])
 {
@@ -844,7 +937,7 @@ get_root_path(const char path[])
 	slash = until_first(after_first(path + 2, '/'), '/');
 	root = format_str("%.*s\\", (int)(slash - path), path);
 
-	to_forward_slash(root);
+	system_to_internal_slashes(root);
 	return root;
 }
 
