@@ -28,6 +28,7 @@
 #include <wchar.h> /* wchar_t */
 
 #include "macros.h"
+#include "test_helpers.h"
 
 #ifndef WEXITSTATUS
 #define WEXITSTATUS(a) (a)
@@ -48,7 +49,7 @@ int wcwidth(wchar_t c);
  * that contain spaces to be enclosed in double quotes.  On internal error
  * returns last error code and sets *returned_exit_code to zero, otherwise sets
  * *returned_exit_code to non-zero and returns exit code of a process. */
-int win_exec_cmd(char cmd[], int *const returned_exit_code);
+int win_exec_cmd(char cmd[], int *returned_exit_code);
 
 /* Turns command into the one suitable to be run by a shell.  Returns the
  * string. */
@@ -82,6 +83,10 @@ int win_cancel_process(DWORD pid, HANDLE hprocess);
 /* Converts FILETIME to time_t.  Returns converted time. */
 time_t win_to_unix_time(FILETIME ft);
 
+TSTATIC_DEFS(
+	int should_wait_for_program(const char cmd[]);
+)
+
 #endif /* VIFM__UTILS__UTILS_WIN_H__ */
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
