@@ -4,7 +4,7 @@
 
 #include <stdlib.h>
 
-#include "../../src/escape.h"
+#include "../../src/ui/escape.h"
 
 #define INV_START "\033[7,1m"
 #define INV_END "\033[27,22m"
@@ -13,6 +13,7 @@ static regex_t re1c;
 static regex_t re2c;
 static regex_t re3c;
 static regex_t re4c;
+static regex_t re5c;
 
 SETUP()
 {
@@ -20,10 +21,12 @@ SETUP()
 	assert_success(regcomp(&re2c, "ab", 0));
 	assert_success(regcomp(&re3c, "$", 0));
 	assert_success(regcomp(&re4c, "a?", REG_EXTENDED));
+	assert_success(regcomp(&re5c, "^", 0));
 }
 
 TEARDOWN()
 {
+	regfree(&re5c);
 	regfree(&re4c);
 	regfree(&re3c);
 	regfree(&re2c);
@@ -130,5 +133,32 @@ TEST(non_empty_match_after_empty_matche_with_esc)
 	free(actual);
 }
 
+TEST(empty_match_is_fine)
+{
+	const char *const input = "foo \033[31m bar vifm";
+	const char *const expected = input;
+	char *const actual = esc_highlight_pattern(input, &re5c);
+	assert_string_equal(expected, actual);
+	free(actual);
+}
+
+TEST(trailing_empty_match_is_fine)
+{
+	const char *const input = "data \033(B\033[m";
+	const char *const expected = input;
+	char *const actual = esc_highlight_pattern(input, &re3c);
+	assert_string_equal(expected, actual);
+	free(actual);
+}
+
+TEST(empty_match_with_utf8_character)
+{
+	const char *const input = "б";
+	const char *const expected = input;
+	char *const actual = esc_highlight_pattern(input, &re5c);
+	assert_string_equal(expected, actual);
+	free(actual);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
