@@ -1,10 +1,13 @@
 #include <stic.h>
 
+#include <unistd.h> /* symlink() */
+
 #include <stddef.h> /* NULL */
 #include <stdlib.h> /* free() remove() */
 #include <string.h> /* strdup() */
 
 #include "../../src/cfg/config.h"
+#include "../../src/compat/fs_limits.h"
 #include "../../src/engine/functions.h"
 #include "../../src/engine/parsing.h"
 #include "../../src/engine/variables.h"
@@ -222,11 +225,13 @@ TEST(getpanetype_for_compare_view)
 {
 	strcpy(lwin.curr_dir, TEST_DATA_PATH "/rename");
 
+	curr_view = &lwin;
+	other_view = &rwin;
+
 	opt_handlers_setup();
 	compare_one_pane(&lwin, CT_CONTENTS, LT_DUPS, 0);
 	opt_handlers_teardown();
 
-	curr_view = &lwin;
 	ASSERT_OK("getpanetype()", "compare");
 }
 
@@ -277,5 +282,93 @@ TEST(chooseopt_options_are_set)
 	args_free(&args);
 }
 
+TEST(tabpagenr)
+{
+	ASSERT_OK("tabpagenr()", "1");
+	ASSERT_OK("tabpagenr('$')", "1");
+
+	ASSERT_FAIL("tabpagenr(1)", PE_INVALID_EXPRESSION);
+	ASSERT_FAIL("tabpagenr('a')", PE_INVALID_EXPRESSION);
+}
+
+TEST(fnameescape)
+{
+	ASSERT_FAIL("fnameescape()", PE_INVALID_EXPRESSION);
+
+	ASSERT_OK("fnameescape(' ')", "\\ ");
+	ASSERT_OK("fnameescape('%')", "%%");
+	ASSERT_OK("fnameescape(\"'\")", "\\'");
+	ASSERT_OK("fnameescape('\"')", "\\\"");
+}
+
+TEST(filetype)
+{
+	opt_handlers_setup();
+
+	/* symlink() is not available on Windows, but the rest of the code is fine. */
+#ifndef _WIN32
+	char cwd[PATH_MAX + 1];
+	assert_non_null(get_cwd(cwd, sizeof(cwd)));
+	char test_data[PATH_MAX + 1];
+	make_abs_path(test_data, sizeof(test_data), TEST_DATA_PATH, "", cwd);
+
+	char path[PATH_MAX + 1];
+	snprintf(path, sizeof(path), "%s/existing-files", test_data);
+	assert_success(symlink(path, SANDBOX_PATH "/dir-link"));
+	snprintf(path, sizeof(path), "%s/existing-files/b", test_data);
+	assert_success(symlink(path, SANDBOX_PATH "/file-link"));
+	snprintf(path, sizeof(path), "%s/no-such-file", test_data);
+	assert_success(symlink(path, SANDBOX_PATH "/broken-link"));
+#endif
+
+	flist_custom_start(&lwin, "test");
+	flist_custom_add(&lwin, TEST_DATA_PATH "/existing-files/a");
+	flist_custom_add(&lwin, TEST_DATA_PATH "/compare");
+#ifndef _WIN32
+	flist_custom_add(&lwin, SANDBOX_PATH "/dir-link");
+	flist_custom_add(&lwin, SANDBOX_PATH "/file-link");
+	flist_custom_add(&lwin, SANDBOX_PATH "/broken-link");
+#endif
+	assert_true(flist_custom_finish(&lwin, CV_VERY, 0) == 0);
+
+	curr_view = &lwin;
+
+	ASSERT_OK("filetype('')", "");
+	lwin.list_pos = 0;
+	ASSERT_OK("filetype('.')", "reg");
+	ASSERT_OK("filetype(1)", "reg");
+	ASSERT_OK("filetype(2)", "dir");
+#ifndef _WIN32
+	ASSERT_OK("filetype('3')", "link");
+	ASSERT_OK("filetype('4')", "link");
+	ASSERT_OK("filetype('3', 1)", "dir");
+	ASSERT_OK("filetype('4', 1)", "reg");
+	ASSERT_OK("filetype(5, 1)", "broken");
+#endif
+
+	opt_handlers_teardown();
+
+#ifndef _WIN32
+	assert_success(remove(SANDBOX_PATH "/dir-link"));
+	assert_success(remove(SANDBOX_PATH "/file-link"));
+	assert_success(remove(SANDBOX_PATH "/broken-link"));
+#endif
+}
+
+TEST(has)
+{
+#ifndef _WIN32
+	ASSERT_OK("has('unix')", "1");
+	ASSERT_OK("has('win')", "0");
+#else
+	ASSERT_OK("has('unix')", "0");
+	ASSERT_OK("has('win')", "1");
+#endif
+
+	ASSERT_OK("has('anythingelse')", "0");
+	ASSERT_OK("has('nix')", "0");
+	ASSERT_OK("has('windows')", "0");
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
