@@ -37,6 +37,9 @@ SETUP()
 
 	opt_handlers_setup();
 
+	columns_setup_column(SK_BY_NAME);
+	columns_setup_column(SK_BY_SIZE);
+
 	saved_cwd = save_cwd();
 }
 
@@ -44,6 +47,8 @@ TEARDOWN()
 {
 	restore_cwd(saved_cwd);
 
+	columns_teardown();
+
 	view_teardown(&lwin);
 	view_teardown(&rwin);
 
@@ -528,7 +533,7 @@ TEST(sorting_is_not_changed)
 	init_commands();
 	assert_success(exec_commands("set sort=ext", &lwin, CIT_COMMAND));
 	assert_int_equal(SK_NONE, lwin.sort[0]);
-	reset_cmds();
+	vle_cmds_reset();
 }
 
 TEST(cursor_moves_in_both_views_synchronously)
@@ -540,11 +545,11 @@ TEST(cursor_moves_in_both_views_synchronously)
 	assert_int_equal(0, lwin.list_pos);
 	assert_int_equal(lwin.list_pos, rwin.list_pos);
 
-	flist_set_pos(&lwin, 2);
+	fpos_set_pos(&lwin, 2);
 	assert_int_equal(2, lwin.list_pos);
 	assert_int_equal(lwin.list_pos, rwin.list_pos);
 
-	flist_set_pos(&rwin, 3);
+	fpos_set_pos(&rwin, 3);
 	assert_int_equal(3, rwin.list_pos);
 	assert_int_equal(lwin.list_pos, rwin.list_pos);
 }
@@ -559,8 +564,8 @@ TEST(two_independent_compare_views_are_not_bound)
 	assert_int_equal(0, lwin.list_pos);
 	assert_int_equal(lwin.list_pos, rwin.list_pos);
 
-	flist_set_pos(&lwin, 2);
-	flist_set_pos(&rwin, 3);
+	fpos_set_pos(&lwin, 2);
+	fpos_set_pos(&rwin, 3);
 	assert_int_equal(2, lwin.list_pos);
 	assert_int_equal(3, rwin.list_pos);
 
