@@ -261,6 +261,7 @@ TEST(global_local_nature_of_normal_zo)
 	load_view_options(curr_view);
 
 	init_modes();
+	init_commands();
 
 	curr_stats.global_local_settings = 1;
 
@@ -273,7 +274,7 @@ TEST(global_local_nature_of_normal_zo)
 	curr_stats.global_local_settings = 0;
 
 	vle_keys_reset();
-	reset_cmds();
+	vle_cmds_reset();
 	opt_handlers_teardown();
 }
 
