@@ -1,5 +1,7 @@
 #include <stic.h>
 
+#include <sys/stat.h> /* chmod() */
+
 #include <string.h> /* strcpy() */
 #include <time.h> /* time() */
 
@@ -8,6 +10,7 @@
 #include "../../src/compat/os.h"
 #include "../../src/ui/ui.h"
 #include "../../src/utils/cancellation.h"
+#include "../../src/utils/dynarray.h"
 #include "../../src/utils/fs.h"
 #include "../../src/utils/str.h"
 #include "../../src/compare.h"
@@ -53,9 +56,9 @@ TEST(compare_view_defines_id_grouping)
 	assert_int_equal(3, lwin.list_rows);
 
 	assert_int_equal(0, lwin.list_pos);
-	assert_int_equal(1, lwin.list_pos = flist_find_group(&lwin, 1));
-	assert_int_equal(2, lwin.list_pos = flist_find_group(&lwin, 1));
-	assert_int_equal(1, lwin.list_pos = flist_find_group(&lwin, 0));
+	assert_int_equal(1, lwin.list_pos = fpos_find_group(&lwin, 1));
+	assert_int_equal(2, lwin.list_pos = fpos_find_group(&lwin, 1));
+	assert_int_equal(1, lwin.list_pos = fpos_find_group(&lwin, 0));
 }
 
 TEST(goto_file_nagivates_to_files)
@@ -66,14 +69,14 @@ TEST(goto_file_nagivates_to_files)
 
 	cfg.ignore_case = 1;
 
-	assert_int_equal(-1, flist_find_by_ch(&lwin, 'a', 0, 0));
-	assert_int_equal(-1, flist_find_by_ch(&lwin, 'A', 1, 0));
-	assert_int_equal(0, flist_find_by_ch(&lwin, 'A', 0, 1));
-	assert_int_equal(0, flist_find_by_ch(&lwin, 'a', 1, 1));
-	assert_int_equal(1, flist_find_by_ch(&lwin, 'b', 0, 0));
-	assert_int_equal(-1, flist_find_by_ch(&lwin, 'B', 1, 0));
-	assert_int_equal(1, flist_find_by_ch(&lwin, 'B', 0, 1));
-	assert_int_equal(1, flist_find_by_ch(&lwin, 'b', 1, 1));
+	assert_int_equal(-1, fpos_find_by_ch(&lwin, 'a', 0, 0));
+	assert_int_equal(-1, fpos_find_by_ch(&lwin, 'A', 1, 0));
+	assert_int_equal(0, fpos_find_by_ch(&lwin, 'A', 0, 1));
+	assert_int_equal(0, fpos_find_by_ch(&lwin, 'a', 1, 1));
+	assert_int_equal(1, fpos_find_by_ch(&lwin, 'b', 0, 0));
+	assert_int_equal(-1, fpos_find_by_ch(&lwin, 'B', 1, 0));
+	assert_int_equal(1, fpos_find_by_ch(&lwin, 'B', 0, 1));
+	assert_int_equal(1, fpos_find_by_ch(&lwin, 'b', 1, 1));
 }
 
 TEST(goto_file_nagivates_to_files_with_case_override)
@@ -86,14 +89,14 @@ TEST(goto_file_nagivates_to_files_with_case_override)
 	cfg.case_override = CO_GOTO_FILE;
 	cfg.case_ignore = 0;
 
-	assert_int_equal(-1, flist_find_by_ch(&lwin, 'a', 0, 0));
-	assert_int_equal(-1, flist_find_by_ch(&lwin, 'A', 1, 0));
-	assert_int_equal(0, flist_find_by_ch(&lwin, 'A', 0, 1));
-	assert_int_equal(0, flist_find_by_ch(&lwin, 'a', 1, 1));
-	assert_int_equal(1, flist_find_by_ch(&lwin, 'b', 0, 0));
-	assert_int_equal(-1, flist_find_by_ch(&lwin, 'B', 1, 0));
-	assert_int_equal(0, flist_find_by_ch(&lwin, 'B', 0, 1));
-	assert_int_equal(1, flist_find_by_ch(&lwin, 'b', 1, 1));
+	assert_int_equal(-1, fpos_find_by_ch(&lwin, 'a', 0, 0));
+	assert_int_equal(-1, fpos_find_by_ch(&lwin, 'A', 1, 0));
+	assert_int_equal(0, fpos_find_by_ch(&lwin, 'A', 0, 1));
+	assert_int_equal(0, fpos_find_by_ch(&lwin, 'a', 1, 1));
+	assert_int_equal(1, fpos_find_by_ch(&lwin, 'b', 0, 0));
+	assert_int_equal(-1, fpos_find_by_ch(&lwin, 'B', 1, 0));
+	assert_int_equal(0, fpos_find_by_ch(&lwin, 'B', 0, 1));
+	assert_int_equal(1, fpos_find_by_ch(&lwin, 'b', 1, 1));
 
 	cfg.case_override = 0;
 }
@@ -106,13 +109,13 @@ TEST(find_directory)
 
 	assert_int_equal(2, lwin.list_rows);
 
-	assert_int_equal(0, flist_next_dir(&lwin));
-	assert_int_equal(0, flist_prev_dir(&lwin));
+	assert_int_equal(0, fpos_next_dir(&lwin));
+	assert_int_equal(0, fpos_prev_dir(&lwin));
 
 	lwin.list_pos = 1;
 
-	assert_int_equal(1, flist_next_dir(&lwin));
-	assert_int_equal(0, flist_prev_dir(&lwin));
+	assert_int_equal(1, fpos_next_dir(&lwin));
+	assert_int_equal(0, fpos_prev_dir(&lwin));
 }
 
 TEST(find_selected)
@@ -126,13 +129,13 @@ TEST(find_selected)
 	lwin.dir_entry[2].selected = 1;
 	lwin.selected_files = 2;
 
-	assert_int_equal(2, flist_next_selected(&lwin));
-	assert_int_equal(0, flist_prev_selected(&lwin));
+	assert_int_equal(2, fpos_next_selected(&lwin));
+	assert_int_equal(0, fpos_prev_selected(&lwin));
 
 	lwin.list_pos = 1;
 
-	assert_int_equal(2, flist_next_selected(&lwin));
-	assert_int_equal(0, flist_prev_selected(&lwin));
+	assert_int_equal(2, fpos_next_selected(&lwin));
+	assert_int_equal(0, fpos_prev_selected(&lwin));
 }
 
 TEST(find_first_and_last_siblings)
@@ -142,17 +145,17 @@ TEST(find_first_and_last_siblings)
 	assert_success(flist_load_tree(&lwin, lwin.curr_dir));
 	assert_int_equal(12, lwin.list_rows);
 
-	assert_int_equal(0, flist_first_sibling(&lwin));
-	assert_int_equal(11, flist_last_sibling(&lwin));
+	assert_int_equal(0, fpos_first_sibling(&lwin));
+	assert_int_equal(11, fpos_last_sibling(&lwin));
 
 	lwin.list_pos = 8;
 
-	assert_int_equal(0, flist_first_sibling(&lwin));
-	assert_int_equal(11, flist_last_sibling(&lwin));
+	assert_int_equal(0, fpos_first_sibling(&lwin));
+	assert_int_equal(11, fpos_last_sibling(&lwin));
 
 	lwin.list_pos = 11;
-	assert_int_equal(0, flist_first_sibling(&lwin));
-	assert_int_equal(11, flist_last_sibling(&lwin));
+	assert_int_equal(0, fpos_first_sibling(&lwin));
+	assert_int_equal(11, fpos_last_sibling(&lwin));
 }
 
 TEST(find_next_and_prev_dir_sibling)
@@ -162,18 +165,18 @@ TEST(find_next_and_prev_dir_sibling)
 	assert_success(flist_load_tree(&lwin, lwin.curr_dir));
 	assert_int_equal(12, lwin.list_rows);
 
-	assert_int_equal(0, flist_prev_dir_sibling(&lwin));
-	assert_int_equal(8, flist_next_dir_sibling(&lwin));
+	assert_int_equal(0, fpos_prev_dir_sibling(&lwin));
+	assert_int_equal(8, fpos_next_dir_sibling(&lwin));
 
 	lwin.list_pos = 8;
 
-	assert_int_equal(0, flist_prev_dir_sibling(&lwin));
-	assert_int_equal(8, flist_next_dir_sibling(&lwin));
+	assert_int_equal(0, fpos_prev_dir_sibling(&lwin));
+	assert_int_equal(8, fpos_next_dir_sibling(&lwin));
 
 	lwin.list_pos = 11;
 
-	assert_int_equal(8, flist_prev_dir_sibling(&lwin));
-	assert_int_equal(11, flist_next_dir_sibling(&lwin));
+	assert_int_equal(8, fpos_prev_dir_sibling(&lwin));
+	assert_int_equal(11, fpos_next_dir_sibling(&lwin));
 }
 
 TEST(find_next_and_prev_mismatches)
@@ -192,18 +195,18 @@ TEST(find_next_and_prev_mismatches)
 
 	lwin.list_pos = 0;
 
-	assert_int_equal(0, flist_prev_mismatch(&lwin));
-	assert_int_equal(2, flist_next_mismatch(&lwin));
+	assert_int_equal(0, fpos_prev_mismatch(&lwin));
+	assert_int_equal(2, fpos_next_mismatch(&lwin));
 
 	lwin.list_pos = 2;
 
-	assert_int_equal(2, flist_prev_mismatch(&lwin));
-	assert_int_equal(2, flist_next_mismatch(&lwin));
+	assert_int_equal(2, fpos_prev_mismatch(&lwin));
+	assert_int_equal(2, fpos_next_mismatch(&lwin));
 
 	lwin.list_pos = 3;
 
-	assert_int_equal(2, flist_prev_mismatch(&lwin));
-	assert_int_equal(3, flist_next_mismatch(&lwin));
+	assert_int_equal(2, fpos_prev_mismatch(&lwin));
+	assert_int_equal(3, fpos_next_mismatch(&lwin));
 
 	view_teardown(&rwin);
 }
@@ -344,6 +347,42 @@ TEST(fentry_get_recalculates_size)
 	update_string(&cfg.shell, NULL);
 }
 
+TEST(fentry_get_nitems_calculates_number_of_items)
+{
+	char origin[] = TEST_DATA_PATH;
+	const dir_entry_t entry = {
+		.name = "existing-files", .origin = origin, .type = FT_DIR
+	};
+
+	update_string(&cfg.shell, "");
+	assert_success(stats_init(&cfg));
+
+	assert_ulong_equal(3, fentry_get_nitems(&lwin, &entry));
+
+	update_string(&cfg.shell, NULL);
+}
+
+TEST(fentry_get_nitems_recalculates_number_of_items)
+{
+	char origin[] = SANDBOX_PATH;
+	dir_entry_t entry = {
+		.name = ".", .origin = origin, .type = FT_DIR
+	};
+
+	update_string(&cfg.shell, "");
+	assert_success(stats_init(&cfg));
+
+	assert_ulong_equal(0, fentry_get_nitems(&lwin, &entry));
+
+	create_file(SANDBOX_PATH "/a");
+	entry.mtime = time(NULL) + 10;
+	assert_ulong_equal(1, fentry_get_nitems(&lwin, &entry));
+
+	update_string(&cfg.shell, NULL);
+
+	assert_success(remove(SANDBOX_PATH "/a"));
+}
+
 TEST(root_path_does_not_get_more_than_one_slash, IF(not_windows))
 {
 	const char type_decs_slash[FT_COUNT][2][9] = {
@@ -370,5 +409,39 @@ TEST(root_path_does_not_get_more_than_one_slash, IF(not_windows))
 	memcpy(&cfg.type_decs, &type_decs_brackets, sizeof(cfg.type_decs));
 }
 
+TEST(duplicated_entries_detected)
+{
+	lwin.list_rows = 2;
+	lwin.list_pos = 0;
+	lwin.top_line = 0;
+	lwin.dir_entry = dynarray_cextend(NULL,
+			lwin.list_rows*sizeof(*lwin.dir_entry));
+	lwin.dir_entry[0].name = strdup("lfile0");
+	lwin.dir_entry[0].origin = &lwin.curr_dir[0];
+	lwin.dir_entry[1].name = strdup("lfile0");
+	lwin.dir_entry[1].origin = &lwin.curr_dir[0];
+
+	check_file_uniqueness(&lwin);
+
+	assert_int_equal(1, lwin.list_rows);
+	assert_true(lwin.has_dups);
+}
+
+TEST(cache_handles_noexec_dirs, IF(not_windows))
+{
+	assert_success(os_mkdir(SANDBOX_PATH "/dir", 0700));
+	create_file(SANDBOX_PATH "/dir/file");
+	assert_success(chmod(SANDBOX_PATH "/dir", 0666));
+
+	cached_entries_t cache = {};
+	assert_true(flist_update_cache(&lwin, &cache, SANDBOX_PATH "/dir"));
+	assert_int_equal(0, cache.entries.nentries);
+	flist_free_cache(&lwin, &cache);
+
+	assert_success(chmod(SANDBOX_PATH "/dir", 0777));
+	assert_success(remove(SANDBOX_PATH "/dir/file"));
+	assert_success(rmdir(SANDBOX_PATH "/dir"));
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
