@@ -14,49 +14,34 @@
 #include "../../src/cmd_core.h"
 #include "../../src/filelist.h"
 #include "../../src/opt_handlers.h"
+#include "../../src/registers.h"
 
 #include "utils.h"
 
 static void print_func(const void *data, int column_id, const char buf[],
 		size_t offset, AlignType align, const char full_column[]);
-static void format_none(int id, const void *data, size_t buf_len, char buf[]);
 
 static int ncols;
 
 SETUP()
 {
-	curr_view = &lwin;
-	other_view = &rwin;
-
 	init_commands();
 
-	lwin.dir_entry = NULL;
-	lwin.list_rows = 0;
-	lwin.window_rows = 1;
-	lwin.sort[0] = SK_NONE;
-	ui_view_sort_list_ensure_well_formed(&lwin, lwin.sort);
+	view_setup(&lwin);
 	lwin.columns = columns_create();
-	lwin.view_columns = strdup("");
 	lwin.num_width_g = lwin.num_width = 4;
-	lwin.ls_view_g = lwin.ls_view = 0;
 	lwin.hide_dot_g = lwin.hide_dot = 1;
-	update_string(&lwin.sort_groups, "");
-	update_string(&lwin.sort_groups_g, "");
+	curr_view = &lwin;
 
-	rwin.dir_entry = NULL;
-	rwin.list_rows = 0;
-	rwin.window_rows = 1;
-	rwin.sort[0] = SK_NONE;
-	ui_view_sort_list_ensure_well_formed(&rwin, rwin.sort);
+	view_setup(&rwin);
 	rwin.columns = columns_create();
-	rwin.view_columns = strdup("");
 	rwin.num_width_g = rwin.num_width = 4;
-	rwin.ls_view_g = rwin.ls_view = 0;
 	rwin.hide_dot_g = rwin.hide_dot = 1;
+	other_view = &rwin;
 
 	/* Name+size matches default column view setting ("-{name},{}"). */
-	columns_add_column_desc(SK_BY_NAME, &format_none);
-	columns_add_column_desc(SK_BY_SIZE, &format_none);
+	columns_setup_column(SK_BY_NAME);
+	columns_setup_column(SK_BY_SIZE);
 	columns_set_line_print_func(&print_func);
 
 	opt_handlers_setup();
@@ -66,19 +51,17 @@ TEARDOWN()
 {
 	opt_handlers_teardown();
 
-	reset_cmds();
+	vle_cmds_reset();
+
+	view_teardown(&lwin);
+	view_teardown(&rwin);
 
 	columns_free(lwin.columns);
 	lwin.columns = NULL;
-	update_string(&lwin.view_columns, NULL);
-	update_string(&lwin.sort_groups, NULL);
-	update_string(&lwin.sort_groups_g, NULL);
-
 	columns_free(rwin.columns);
 	rwin.columns = NULL;
-	update_string(&rwin.view_columns, NULL);
 
-	columns_clear_column_descs();
+	columns_teardown();
 }
 
 static void
@@ -88,12 +71,6 @@ print_func(const void *data, int column_id, const char buf[], size_t offset,
 	ncols += (column_id != FILL_COLUMN_ID);
 }
 
-static void
-format_none(int id, const void *data, size_t buf_len, char buf[])
-{
-	buf[0] = '\0';
-}
-
 TEST(lsview_block_columns_update_on_sort_change)
 {
 	assert_success(exec_commands("set viewcolumns=", curr_view, CIT_COMMAND));
@@ -150,161 +127,6 @@ TEST(fails_to_set_sort_group_with_wrong_regexp)
 	assert_failure(exec_commands("set sortgroups=.*,*", &lwin, CIT_COMMAND));
 }
 
-TEST(classify_parsing_of_types)
-{
-	const char type_decs[FT_COUNT][2][9] = {
-		[FT_DIR][DECORATION_PREFIX][0] = '-',
-		[FT_DIR][DECORATION_SUFFIX][0] = '1',
-		[FT_REG][DECORATION_PREFIX][0] = '*',
-		[FT_REG][DECORATION_SUFFIX][0] = '/',
-	};
-
-	assert_success(exec_commands("set classify=*:reg:/,-:dir:1", &lwin,
-				CIT_COMMAND));
-
-	assert_int_equal(0,
-			memcmp(&cfg.type_decs, &type_decs, sizeof(cfg.type_decs)));
-	assert_int_equal(0, cfg.name_dec_count);
-
-	assert_string_equal("-:dir:1,*:reg:/",
-			get_option_value("classify", OPT_GLOBAL));
-}
-
-TEST(classify_parsing_of_exprs)
-{
-	const char type_decs[FT_COUNT][2][9] = {};
-
-	assert_success(
-			exec_commands(
-				"set classify=*::!{*.c}::/,123::*.c,,*.b::753,b::/.*-.*/i::q,-::*::1",
-				&lwin, CIT_COMMAND));
-
-	assert_int_equal(0,
-			memcmp(&cfg.type_decs, &type_decs, sizeof(cfg.type_decs)));
-	assert_int_equal(4, cfg.name_dec_count);
-	assert_string_equal("*", cfg.name_decs[0].prefix);
-	assert_string_equal("/", cfg.name_decs[0].suffix);
-	assert_string_equal("123", cfg.name_decs[1].prefix);
-	assert_string_equal("753", cfg.name_decs[1].suffix);
-	assert_string_equal("b", cfg.name_decs[2].prefix);
-	assert_string_equal("q", cfg.name_decs[2].suffix);
-	assert_string_equal("-", cfg.name_decs[3].prefix);
-	assert_string_equal("1", cfg.name_decs[3].suffix);
-
-	assert_string_equal("*::!{*.c}::/,123::*.c,,*.b::753,b::/.*-.*/i::q,-::*::1",
-			get_option_value("classify", OPT_GLOBAL));
-}
-
-TEST(classify_suffix_prefix_lengths)
-{
-	char type_decs[FT_COUNT][2][9];
-	memcpy(&type_decs, &cfg.type_decs, sizeof(cfg.type_decs));
-
-	assert_failure(exec_commands("set classify=123456789::{*.c}::/", &lwin,
-				CIT_COMMAND));
-	assert_int_equal(0,
-			memcmp(&cfg.type_decs, &type_decs, sizeof(cfg.type_decs)));
-
-	assert_failure(exec_commands("set classify=::{*.c}::123456789", &lwin,
-				CIT_COMMAND));
-	assert_int_equal(0,
-			memcmp(&cfg.type_decs, &type_decs, sizeof(cfg.type_decs)));
-
-	assert_success(exec_commands("set classify=12345678::{*.c}::12345678", &lwin,
-				CIT_COMMAND));
-}
-
-TEST(classify_pattern_list)
-{
-	dir_entry_t entry = {
-		.name = "binary-data",
-		.origin = TEST_DATA_PATH "/test-data/read",
-		.name_dec_num = -1,
-	};
-
-	const char *prefix, *suffix;
-
-	assert_success(exec_commands("set classify=<::{*-data}{*-data}::>", &lwin,
-				CIT_COMMAND));
-
-	ui_get_decors(&entry, &prefix, &suffix);
-	assert_string_equal("<", prefix);
-	assert_string_equal(">", suffix);
-}
-
-TEST(classify_can_be_set_to_empty_value)
-{
-	dir_entry_t entry = {
-		.name = "binary-data",
-		.origin = TEST_DATA_PATH "/test-data/read",
-		.name_dec_num = -1,
-	};
-
-	const char *prefix, *suffix;
-
-	assert_success(exec_commands("set classify=", &lwin, CIT_COMMAND));
-
-	ui_get_decors(&entry, &prefix, &suffix);
-	assert_string_equal("", prefix);
-	assert_string_equal("", suffix);
-}
-
-TEST(classify_account_assumes_trailing_slashes_for_dirs)
-{
-	dir_entry_t entry = {
-		.name = "read",
-		.type = FT_DIR,
-		.origin = TEST_DATA_PATH,
-		.name_dec_num = -1,
-	};
-
-	const char *prefix, *suffix;
-
-	assert_success(exec_commands("set classify=[:dir:],*::*ad::@", &lwin,
-				CIT_COMMAND));
-
-	ui_get_decors(&entry, &prefix, &suffix);
-	assert_string_equal("[", prefix);
-	assert_string_equal("]", suffix);
-}
-
-TEST(classify_state_is_not_changed_if_format_is_wong)
-{
-	assert_success(exec_commands("set classify=*::*ad::@", &lwin, CIT_COMMAND));
-	assert_int_equal(1, cfg.name_dec_count);
-	assert_failure(exec_commands("set classify=*:*ad:@", &lwin, CIT_COMMAND));
-	assert_int_equal(1, cfg.name_dec_count);
-}
-
-TEST(classify_does_not_stop_on_empty_prefix)
-{
-	dir_entry_t entry = {
-		.name = "read",
-		.origin = TEST_DATA_PATH,
-		.name_dec_num = -1,
-	};
-
-	const char *prefix, *suffix;
-
-	assert_success(exec_commands("set classify=:dir:/,:link:@,:fifo:|", &lwin,
-				CIT_COMMAND));
-
-	entry.type = FT_DIR;
-	ui_get_decors(&entry, &prefix, &suffix);
-	assert_string_equal("", prefix);
-	assert_string_equal("/", suffix);
-
-	entry.type = FT_LINK;
-	ui_get_decors(&entry, &prefix, &suffix);
-	assert_string_equal("", prefix);
-	assert_string_equal("@", suffix);
-
-	entry.type = FT_FIFO;
-	ui_get_decors(&entry, &prefix, &suffix);
-	assert_string_equal("", prefix);
-	assert_string_equal("|", suffix);
-}
-
 TEST(suggestoptions_all_values)
 {
 	cfg.sug.flags = 0;
@@ -729,5 +551,90 @@ TEST(tuioptions)
 	assert_true(cfg.use_unicode_characters);
 }
 
+TEST(setting_tabscope_works)
+{
+	assert_success(exec_commands("set tabscope=pane", &lwin, CIT_COMMAND));
+	assert_true(cfg.pane_tabs);
+
+	assert_success(exec_commands("set tabscope=global", &lwin, CIT_COMMAND));
+	assert_false(cfg.pane_tabs);
+}
+
+TEST(setting_showtabline_works)
+{
+	assert_success(exec_commands("set showtabline=0", &lwin, CIT_COMMAND));
+	assert_int_equal(STL_NEVER, cfg.show_tab_line);
+	assert_success(exec_commands("set showtabline=never", &lwin, CIT_COMMAND));
+	assert_int_equal(STL_NEVER, cfg.show_tab_line);
+
+	assert_success(exec_commands("set showtabline=1", &lwin, CIT_COMMAND));
+	assert_int_equal(STL_MULTIPLE, cfg.show_tab_line);
+	assert_success(exec_commands("set showtabline=multiple", &lwin, CIT_COMMAND));
+	assert_int_equal(STL_MULTIPLE, cfg.show_tab_line);
+
+	assert_success(exec_commands("set showtabline=2", &lwin, CIT_COMMAND));
+	assert_int_equal(STL_ALWAYS, cfg.show_tab_line);
+	assert_success(exec_commands("set showtabline=always", &lwin, CIT_COMMAND));
+	assert_int_equal(STL_ALWAYS, cfg.show_tab_line);
+}
+
+TEST(shortmess)
+{
+	cfg.trunc_normal_sb_msgs = 0;
+	cfg.shorten_title_paths = 0;
+	cfg.short_term_mux_titles = 0;
+
+	assert_success(exec_commands("set shortmess=Mp", &lwin, CIT_COMMAND));
+	assert_true(cfg.short_term_mux_titles);
+	assert_false(cfg.trunc_normal_sb_msgs);
+	assert_true(cfg.shorten_title_paths);
+
+	assert_success(exec_commands("set shortmess=T", &lwin, CIT_COMMAND));
+	assert_false(cfg.short_term_mux_titles);
+	assert_true(cfg.trunc_normal_sb_msgs);
+	assert_false(cfg.shorten_title_paths);
+}
+
+TEST(histcursor)
+{
+	cfg.ch_pos_on = 0;
+
+	assert_success(exec_commands("set histcursor=startup", &lwin, CIT_COMMAND));
+	assert_int_equal(CHPOS_STARTUP, cfg.ch_pos_on);
+
+	assert_success(exec_commands("set histcursor=direnter,dirmark", &lwin,
+				CIT_COMMAND));
+	assert_int_equal(CHPOS_ENTER | CHPOS_DIRMARK, cfg.ch_pos_on);
+}
+
+TEST(quickview)
+{
+	assert_success(exec_commands("set quickview", &lwin, CIT_COMMAND));
+	assert_true(curr_stats.preview.on);
+	assert_success(exec_commands("set invquickview", &lwin, CIT_COMMAND));
+	assert_false(curr_stats.preview.on);
+}
+
+TEST(syncregs)
+{
+	assert_false(regs_sync_enabled());
+	assert_success(exec_commands("set syncregs=test1", &lwin, CIT_COMMAND));
+	assert_true(regs_sync_enabled());
+	assert_success(exec_commands("set syncregs=test2", &lwin, CIT_COMMAND));
+	assert_true(regs_sync_enabled());
+	assert_success(exec_commands("set syncregs=test1", &lwin, CIT_COMMAND));
+	assert_true(regs_sync_enabled());
+	assert_success(exec_commands("set syncregs=", &lwin, CIT_COMMAND));
+	assert_false(regs_sync_enabled());
+}
+
+TEST(mediaprg, IF(not_windows))
+{
+	assert_success(exec_commands("set mediaprg=prg", &lwin, CIT_COMMAND));
+	assert_string_equal("prg", cfg.media_prg);
+	assert_success(exec_commands("set mediaprg=", &lwin, CIT_COMMAND));
+	assert_string_equal("", cfg.media_prg);
+}
+
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
 /* vim: set cinoptions+=t0 filetype=c : */
