@@ -13,12 +13,21 @@
 #include "../../src/utils/string_array.h"
 #include "../../src/filetype.h"
 
+#include "utils.h"
+
 static void check_only_one_line_displayed(void);
 
 SETUP()
 {
 	curr_view = &lwin;
 	other_view = &rwin;
+
+	opt_handlers_setup();
+}
+
+TEARDOWN()
+{
+	opt_handlers_teardown();
 }
 
 TEST(no_extra_line_with_extra_padding)
@@ -135,7 +144,7 @@ TEST(quick_view_picks_entry)
 	char origin[] = "/path";
 	char name[] = "name";
 	dir_entry_t entry = { .origin = origin, .name = name, .type = FT_REG };
-	char path[PATH_MAX];
+	char path[PATH_MAX + 1];
 
 	qv_get_path_to_explore(&entry, path, sizeof(path));
 	assert_string_equal("/path/name", path);
@@ -146,7 +155,7 @@ TEST(quick_view_picks_current_directory)
 	char origin[] = "/path";
 	char name[] = "..";
 	dir_entry_t entry = { .origin = origin, .name = name, .type = FT_DIR };
-	char path[PATH_MAX];
+	char path[PATH_MAX + 1];
 
 	qv_get_path_to_explore(&entry, path, sizeof(path));
 	assert_string_equal("/path", path);
