@@ -10,7 +10,7 @@ TEST(reset_of_str_option_not_differ_no_callback)
 	int res;
 
 	fusehome_handler_calls = 0;
-	res = set_options("fusehome&");
+	res = set_options("fusehome&", OPT_GLOBAL);
 	assert_int_equal(0, res);
 	assert_int_equal(0, fusehome_handler_calls);
 }
@@ -19,16 +19,16 @@ TEST(reset_of_str_option_differ_callback)
 {
 	int res;
 
-	res = set_options("fusehome='abc'");
+	res = set_options("fusehome='abc'", OPT_GLOBAL);
 	assert_int_equal(0, res);
 	assert_string_equal("abc", value);
 
 	fusehome_handler_calls = 0;
-	res = set_options("fusehome&");
+	res = set_options("fusehome&", OPT_GLOBAL);
 	assert_int_equal(0, res);
 	assert_int_equal(1, fusehome_handler_calls);
 	assert_string_equal("fusehome-default", value);
 }
 
 /* vim: set tabstop=2 softtabstop=2 shiftwidth=2 noexpandtab cinoptions-=(0 : */
-/* vim: set cinoptions+=t0 : */
+/* vim: set cinoptions+=t0 filetype=c : */
