@@ -258,12 +258,13 @@ struct stic_test_data
     int main(int argc, char *argv[]) \
     { \
         int r; \
-        if(stic_setup_once_func != NULL) \
+        const int file_has_tests = (stic_get_fixture_name() != NULL); \
+        if(!file_has_tests && stic_setup_once_func != NULL) \
         { \
             stic_setup_once_func(); \
         } \
         r = stic_testrunner(argc, argv, stic_suite, stic_setup_func, stic_teardown_func) == 0; \
-        if(stic_teardown_once_func != NULL) \
+        if(!file_has_tests && stic_teardown_once_func != NULL) \
         { \
             stic_teardown_once_func(); \
         } \
@@ -282,26 +283,33 @@ static void (*stic_setup_once_func)(void);
 static void (*stic_teardown_func)(void);
 static void (*stic_teardown_once_func)(void);
 
+static struct stic_test_data *const *const stic_test_data[] = {
+    TEST_DATA_STRUCTS_REF
+};
+
+static const char * stic_get_fixture_name(void)
+{
+    size_t i;
+    for(i = 0U; i < STIC_ARRAY_LEN(stic_test_data); ++i)
+    {
+        if(*stic_test_data[i])
+        {
+            return (*stic_test_data[i])->f;
+        }
+    }
+    return NULL;
+}
+
 static void stic_fixture(void)
 {
     extern const char *stic_current_test_name;
     extern stic_void_void stic_current_test;
 
-    const char *fixture_name = NULL;
     size_t i;
-    static struct stic_test_data *const *const test_data[] = {
-        TEST_DATA_STRUCTS_REF
-    };
+    int has_any_tests = 0;
 
-    for(i = 0; i < STIC_ARRAY_LEN(test_data); ++i)
-    {
-        if(*test_data[i])
-        {
-            fixture_name = (*test_data[i])->f;
-            break;
-        }
-    }
-    if(fixture_name == NULL)
+    const char *fixture_name = stic_get_fixture_name();
+    if(fixture_name == NULL || !stic_should_run(fixture_name, NULL))
     {
         return;
     }
@@ -311,23 +319,30 @@ static void stic_fixture(void)
     fixture_setup(stic_setup_func);
     fixture_teardown(stic_teardown_func);
 
-    if(stic_setup_once_func != NULL)
+    for(i = 0; i < STIC_ARRAY_LEN(stic_test_data); ++i)
     {
-        stic_current_test_name = "<setup once>";
-        stic_current_test = stic_setup_once_func;
-        stic_setup_once_func();
-    }
-
-    for(i = 0; i < STIC_ARRAY_LEN(test_data); ++i)
-    {
-        struct stic_test_data *td = *test_data[i];
+        struct stic_test_data *td = *stic_test_data[i];
         if(td == NULL) continue;
+        if(!stic_should_run(fixture_name, td->n)) continue;
+
+        /* Since predicates can rely on setup once, check predicates after
+         * invoking setup once. */
+        if(!has_any_tests)
+        {
+            has_any_tests = 1;
+            if(stic_setup_once_func != NULL)
+            {
+                stic_current_test_name = "<setup once>";
+                stic_current_test = stic_setup_once_func;
+                stic_setup_once_func();
+            }
+        }
+
         if(td->p != NULL && !td->p())
         {
             stic_skip_test(fixture_name, td->n);
             continue;
         }
-        if(!stic_should_run(fixture_name, td->n)) continue;
 
         stic_current_test_name = td->n;
         stic_current_test = td->t;
@@ -339,7 +354,7 @@ static void stic_fixture(void)
         stic_run_test(fixture_name, td->n);
     }
 
-    if(stic_teardown_once_func != NULL)
+    if(has_any_tests && stic_teardown_once_func != NULL)
     {
         stic_current_test_name = "<teardown once>";
         stic_current_test = stic_teardown_once_func;
