@@ -6,7 +6,7 @@
 TEST(empty_command_line)
 {
 	const char cmd[] = "";
-	char command[NAME_MAX];
+	char command[NAME_MAX + 1];
 	const char *args;
 
 	args = extract_cmd_name(cmd, 0, sizeof(command), command);
@@ -17,7 +17,7 @@ TEST(empty_command_line)
 TEST(command_name_only)
 {
 	const char cmd[] = "cmd";
-	char command[NAME_MAX];
+	char command[NAME_MAX + 1];
 	const char *args;
 
 	args = extract_cmd_name(cmd, 0, sizeof(command), command);
@@ -28,7 +28,7 @@ TEST(command_name_only)
 TEST(leading_whitespase_ignored)
 {
 	const char cmd[] = "  \t  cmd";
-	char command[NAME_MAX];
+	char command[NAME_MAX + 1];
 	const char *args;
 
 	args = extract_cmd_name(cmd, 0, sizeof(command), command);
@@ -39,7 +39,7 @@ TEST(leading_whitespase_ignored)
 TEST(with_argument_list)
 {
 	const char cmd[] = "cmd arg1 arg2";
-	char command[NAME_MAX];
+	char command[NAME_MAX + 1];
 	const char *args;
 
 	args = extract_cmd_name(cmd, 0, sizeof(command), command);
@@ -50,7 +50,7 @@ TEST(with_argument_list)
 TEST(whitespace_after_command_ignored)
 {
 	const char cmd[] = "cmd   \t  arg1 arg2";
-	char command[NAME_MAX];
+	char command[NAME_MAX + 1];
 	const char *args;
 
 	args = extract_cmd_name(cmd, 0, sizeof(command), command);
@@ -61,7 +61,7 @@ TEST(whitespace_after_command_ignored)
 TEST(fusemount_part_removed)
 {
 	const char cmd[] = "FUSE_MOUNT|archivemount %SOURCE_FILE %DESTINATION_DIR";
-	char command[NAME_MAX];
+	char command[NAME_MAX + 1];
 	const char *args;
 
 	args = extract_cmd_name(cmd, 0, sizeof(command), command);
@@ -72,7 +72,7 @@ TEST(fusemount_part_removed)
 TEST(fusemount2_part_removed)
 {
 	const char cmd[] = "FUSE_MOUNT2|sshfs %PARAM %DESTINATION_DIR";
-	char command[NAME_MAX];
+	char command[NAME_MAX + 1];
 	const char *args;
 
 	args = extract_cmd_name(cmd, 0, sizeof(command), command);
@@ -85,7 +85,7 @@ TEST(fusemount2_part_removed)
 TEST(quoted_command_raw_ok)
 {
 	const char cmd[] = "\"quoted cmd\"   \t  arg1 arg2";
-	char command[NAME_MAX];
+	char command[NAME_MAX + 1];
 	const char *args;
 
 	args = extract_cmd_name(cmd, 1, sizeof(command), command);
@@ -96,7 +96,7 @@ TEST(quoted_command_raw_ok)
 TEST(quoted_command_coocked_ok)
 {
 	const char cmd[] = "\"quoted cmd\"   \t  arg1 arg2";
-	char command[NAME_MAX];
+	char command[NAME_MAX + 1];
 	const char *args;
 
 	args = extract_cmd_name(cmd, 0, sizeof(command), command);
